@@ -237,20 +237,20 @@ protected:
 	*/
 	void CommandError(const PString & msg);
 
-    // Adds regular expression filter
-    void AddFilter(
+	// Adds regular expression filter
+	void AddFilter(
 	// filter vector
-	std::vector<PString>& regexFilters,
+	std::vector<PString> & regexFilters,
 	// Regex to be matched against messages
 	const PString& regex
 	);
 
-    /** Remove regular expression filter located
+	/** Remove regular expression filter located
 	at the given index from the specified vector
-    */
+	*/
     void RemoveFilter(
 	// filter vector
-	std::vector<PString>& regexFilters,
+	std::vector<PString> & regexFilters,
 	// Index of filter to be removed
 	unsigned int index
 	);
@@ -258,25 +258,25 @@ protected:
     // Checks whether the given string is to be exclude
     bool IsExcludeMessage(
 	// String to be check against the exclude regular expressions
-	const PString &msg
+	const PString & msg
 	) const;
 
     // Print a list of all filters in the specified vector
     void PrintFilters(
 	// filter vector
-	std::vector<PString>& regexFilters
+	std::vector<PString> & regexFilters
 	);
 
     // Checks whether the given string is to be include
     bool IsIncludeMessage(
 	// String to be check against the include regular expressions
-	const PString &msg
+	const PString & msg
 	) const;
     
     // Match the given string against filters held by the specified vector
     bool MatchFilter(
 	// filter vector
-	const std::vector<PString>& regexFilters,
+	const std::vector<PString> & regexFilters,
 	// String to be matched
 	const PString &msg
 	) const;
@@ -286,7 +286,7 @@ protected:
 	/// command being currently entered (and not yet completed)
 	PString m_currentCmd;
 	/// GkStatus instance that created this client
-	GkStatus* m_gkStatus;
+	GkStatus * m_gkStatus;
 	/// status interface user that is logged in
 	PString	m_user;
 	/// for atomic access to the m_numExecutingCommands counter
@@ -407,7 +407,7 @@ PBoolean SSHStatusClient::Accept(PSocket & socket)
 		return false;
     }
 	ssh_bind_set_fd(sshbind, socket.GetHandle());
-    if(ssh_bind_accept(sshbind, session) == SSH_ERROR) {
+	if(ssh_bind_accept(sshbind, session) == SSH_ERROR) {
 		PTRACE(1, "ssh_bind_accept() failed: " << ssh_get_error(sshbind));
 		return false;
     }
@@ -417,7 +417,7 @@ PBoolean SSHStatusClient::Accept(PSocket & socket)
 		return false;
 	}
 	
-	// set handle for SocketReader
+	// set handle for SocketsReader
 	os_handle = ssh_get_fd(session);
 
 	return true;
@@ -436,7 +436,7 @@ bool SSHStatusClient::Authenticate()
 			PTRACE(1, "ssh read error: " << ssh_get_error(session));
             break;
 		}
-        switch(ssh_message_type(message)) {
+		switch(ssh_message_type(message)) {
             case SSH_REQUEST_AUTH:
                 switch(ssh_message_subtype(message)) {
                     case SSH_AUTH_METHOD_PASSWORD:
@@ -519,7 +519,7 @@ bool SSHStatusClient::Authenticate()
             }
             ssh_message_free(message);
         }
-    } while(message && !shell);
+	} while(message && !shell);
 	if (!message)
 		return false;
 
@@ -639,7 +639,9 @@ PBoolean TelnetSocket::Accept(PSocket & socket)
 	Address raddr, laddr;
 	WORD rport = 0, lport = 0;
 	GetPeerAddress(raddr, rport);
+	UnmapIPv4Address(raddr);
 	GetLocalAddress(laddr, lport);
+	UnmapIPv4Address(laddr);
 	SetName(AsString(raddr, rport) + "=>" + AsString(laddr, lport));
 	SetReadTimeout(0);
 #else
@@ -818,9 +820,7 @@ GkStatus::GkStatus() : Singleton<GkStatus>("GkStatus"), SocketsReader(500)
 	Execute();
 }
 
-void GkStatus::AuthenticateClient(
-	StatusClient* newClient
-	)
+void GkStatus::AuthenticateClient(StatusClient* newClient)
 {
 	if (newClient->Authenticate()) {
 		newClient->SetTraceLevel(GkConfig()->GetInteger("StatusTraceLevel", MAX_STATUS_TRACE_LEVEL));
@@ -867,7 +867,7 @@ public:
 
 private:
 	/// message to be sent
-	const PString& m_message;
+	const PString & m_message;
 	/// output trace level assigned to the message
 	int m_traceLevel;
 };
@@ -952,9 +952,9 @@ void GkStatus::PrintHelp(
 
 int GkStatus::ParseCommand(
 	/// message to be parsed
-	const PString& msg,
+	const PString & msg,
 	/// message split into tokens upon successful return
-	PStringArray& args
+	PStringArray & args
 	)
 {
 	// the 'Tokenise' doesn't seem to work correctly for leading spaces
@@ -979,7 +979,7 @@ int GkStatus::ParseCommand(
 			else
 				expandedCmd = iter;
 		}
-		iter++;
+		++iter;
 	}
 	if( expandedCmd != m_commands.end() ) {
 		PTRACE(6, "STATUS\tExpanded prefix '" << command << "' into command '" 
@@ -1006,6 +1006,7 @@ void GkStatus::OnStart()
 	m_commands["printcurrentcallsverbose"] = e_PrintCurrentCallsVerbose;
 	m_commands["cv"] = e_PrintCurrentCallsVerbose;
 	m_commands["!!"] = e_PrintCurrentCallsVerbose;
+	m_commands["printcurrentcallsports"] = e_PrintCurrentCallsPorts;
 	m_commands["find"] = e_Find;
 	m_commands["f"] = e_Find;
 	m_commands["findverbose"] = e_FindVerbose;
@@ -1045,10 +1046,8 @@ void GkStatus::OnStart()
 	m_commands["quit"] = e_Exit;
 	m_commands["q"] = e_Exit;
 	m_commands["trace"] = e_Trace;
-#if PTRACING
 	m_commands["rotatelog"] = e_RotateLog;
 	m_commands["setlog"] = e_SetLogFilename;
-#endif
 	m_commands["addincludefilter"] = e_AddIncludeFilter;
 	m_commands["removeincludefilter"] = e_RemoveIncludeFilter;
 	m_commands["addexcludefilter"] = e_AddExcludeFilter;
@@ -1125,6 +1124,7 @@ StatusClient::StatusClient(
 		    AddFilter(m_excludeFilterRegex, regexArray[k]);
 	    }
 	}
+	m_isFilteringActive = Toolkit::AsBool(GkConfig()->GetString(filteringsec, "Enable", "0"));
 }
 
 bool StatusClient::ReadCommand(
@@ -1349,11 +1349,12 @@ void StatusClient::DoDebug(
 
 bool StatusClient::CheckAuthRule(
 	/// authentication rule to be used
-	const PString& rule
+	const PString & rule
 	)
 {
 	PIPSocket::Address peerAddress;
 	GetPeerAddress(peerAddress);
+	UnmapIPv4Address(peerAddress);
 	const PString peer = peerAddress.AsString();
 
 	bool result = false;
@@ -1540,6 +1541,10 @@ void StatusClient::ExecCommand(
 		// print list of currently ongoing calls
 		SoftPBX::PrintCurrentCalls(this, TRUE);
 		break;
+	case GkStatus::e_PrintCurrentCallsPorts:
+		// print list of currently ongoing calls with their dynamic ports
+		SoftPBX::PrintCurrentCallsPorts(this);
+		break;
 	case GkStatus::e_Statistics:
 		SoftPBX::PrintStatistics(this, TRUE);
 		break;
@@ -1752,7 +1757,6 @@ void StatusClient::ExecCommand(
 		WriteString("Output trace level is " + PString(m_traceLevel) + "\r\n");
 		break;
 
-#if PTRACING
 	case GkStatus::e_RotateLog:
 	    if (Gatekeeper::RotateLogFile())
 			WriteString("Log file rotation succeeded\r\n");
@@ -1769,29 +1773,36 @@ void StatusClient::ExecCommand(
 		} else
 			CommandError("Syntax Error: setlog <logfilepath>");
 		break;				
-#endif
 
 	case GkStatus::e_AddIncludeFilter:
 	    if (args.GetSize() == 2) {
 			AddFilter(m_includeFilterRegex, args[1]);
+			PString msg("IncludeFilter added\r\n");
+			WriteData(msg, msg.GetLength());
 	    } else
 			CommandError("Syntax Error: addincludefilter REGEX");
 	    break;
 	case GkStatus::e_RemoveIncludeFilter:
 	    if (args.GetSize() == 2) {
 			RemoveFilter(m_includeFilterRegex, atoi(args[1]));
+			PString msg("IncludeFilter removed\r\n");
+			WriteData(msg, msg.GetLength());
 	    } else
 			CommandError("Syntax Error: removeincludefilter FILTER_INDEX");
 	    break;
 	case GkStatus::e_AddExcludeFilter:
 	    if (args.GetSize() == 2) {
 			AddFilter(m_excludeFilterRegex, args[1]);
+			PString msg("ExcludeFilter added\r\n");
+			WriteData(msg, msg.GetLength());
 	    } else
 			CommandError("Syntax Error: addexcludefilter REGEX");
 	    break;
 	case GkStatus::e_RemoveExcludeFilter:
 	    if (args.GetSize() == 2) {
 			RemoveFilter(m_excludeFilterRegex, atoi(args[1]));
+			PString msg("ExcludeFilter removed\r\n");
+			WriteData(msg, msg.GetLength());
 	    } else
 			CommandError("Syntax Error: removeincludefilter FILTER_INDEX");
 	    break;
@@ -1937,14 +1948,11 @@ void StatusClient::PrintFilters(
 }
 
 // class StatusListener
-StatusListener::StatusListener(
-	const Address& addr, 
-	WORD lport
-	)
+StatusListener::StatusListener(const Address & addr, WORD lport)
 {
 	const unsigned queueSize = GkConfig()->GetInteger("ListenQueueLength", GK_DEF_LISTEN_QUEUE_LENGTH);
 	if (!Listen(addr, queueSize, lport, PSocket::CanReuseAddress)) {
-		PTRACE(1, "STATUS\tCould not open listening socket at " << addr << ':' << lport
+		PTRACE(1, "STATUS\tCould not open listening socket at " << AsString(addr, lport)
 			<< " - error " << GetErrorCode(PSocket::LastGeneralError) << '/'
 			<< GetErrorNumber(PSocket::LastGeneralError) << ": " 
 			<< GetErrorText(PSocket::LastGeneralError)
@@ -1952,9 +1960,16 @@ StatusListener::StatusListener(
 		Close();
 	}
 	SetName(AsString(addr, GetPort()));
+	m_addr = addr;
+	Toolkit::Instance()->PortNotification(StatusPort, PortOpen, "tcp", m_addr, lport);
 }
 
-ServerSocket *StatusListener::CreateAcceptor() const
+StatusListener::~StatusListener()
+{
+	Toolkit::Instance()->PortNotification(StatusPort, PortClose, "tcp", m_addr, GetPort());
+}
+
+ServerSocket * StatusListener::CreateAcceptor() const
 {
 	static int StaticInstanceNo = 0;
 #ifdef HAS_LIBSSH
