@@ -13,7 +13,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKSTATUS_H
-#define GKSTATUS_H "@(#) $Id: GkStatus.h,v 1.38 2011/02/11 09:18:09 willamowius Exp $"
+#define GKSTATUS_H "@(#) $Id: GkStatus.h,v 1.41 2011/11/28 12:33:40 willamowius Exp $"
 
 #include <map>
 #include "yasocket.h"
@@ -101,6 +101,7 @@ public:
 		e_PrintAllCached,
 		e_PrintCurrentCalls,
 		e_PrintCurrentCallsVerbose,    /// extra line per call starting with '#'.
+		e_PrintCurrentCallsPorts,
 		e_Find,                        /// find an endpoint
 		e_FindVerbose,
 		e_DisconnectIp,                /// disconnect a call by endpoint IP number
@@ -131,10 +132,8 @@ public:
 		e_BindAndRouteToGateway,       /// Route a call to a specified alias + destinationCallSignalAddr + specify bind IP
 		e_RouteReject,                 /// Reject to Route a call upon ARQ (send ARJ)
 		e_Trace,                       /// change trace level for status interface output
-#if PTRACING
 		e_RotateLog,                   /// Force log file rotation
 		e_SetLogFilename,              /// Change log file location
-#endif
 		e_AddIncludeFilter,            /// Add include filter
 		e_RemoveIncludeFilter,         /// Remove include filter
 		e_AddExcludeFilter,            /// Add exclude filter
@@ -195,13 +194,16 @@ public:
 		WORD port
 		);
 		
-	virtual ~StatusListener() {}
+	virtual ~StatusListener();
 
 	/** Create a new StatusClient socket that will be used
 		to accept a next incoming connection.
 		Override from class TCPListenSocket.
 	*/
 	virtual ServerSocket *CreateAcceptor() const;
+
+protected:
+	Address m_addr;
 };
 
 #endif // GKSTATUS_H
