@@ -3,7 +3,7 @@
 // ProxyChannel.h
 //
 // Copyright (c) Citron Network Inc. 2001-2003
-// Copyright (c) 2002-2010, Jan Willamowius
+// Copyright (c) 2002-2011, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,12 +13,14 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef PROXYCHANNEL_H
-#define PROXYCHANNEL_H "@(#) $Id: ProxyChannel.h,v 1.64 2010/11/26 10:33:31 willamowius Exp $"
+#define PROXYCHANNEL_H "@(#) $Id: ProxyChannel.h,v 1.93 2011/11/02 16:06:50 willamowius Exp $"
 
 #include <vector>
 #include <list>
+#include <map>
 #include "yasocket.h"
 #include "RasTbl.h"
+#include "gktimer.h"
 
 
 class Q931;
@@ -52,6 +54,7 @@ typedef H225SignalingMsg<H225_Facility_UUIE> FacilityMsg;
 struct SetupAuthData;
 
 extern const char *RoutedSec;
+extern const char *ProxySection;
 
 void PrintQ931(int, const char *, const char *, const Q931 *, const H225_H323_UserInformation *);
 
@@ -123,7 +126,7 @@ public:
 
 	void RemoveRemoteSocket();
 	void SetRemoteSocket(TCPProxySocket * ret) { remote=ret; }
-	TCPProxySocket * GetRemoteSocket() const { return remote; }
+	//TCPProxySocket * GetRemoteSocket() const { return remote; }
 
 private:
 	TCPProxySocket();
@@ -182,11 +185,12 @@ public:
 	void SendReleaseComplete(H225_ReleaseCompleteReason::Choices);
 
 	bool HandleH245Mesg(PPER_Stream &, bool & suppress, H245Socket * h245sock = NULL);
-	bool IsNATSocket() const { return m_isnatsocket; }
 	void OnH245ChannelClosed() { m_h245socket = NULL; }
 	Address GetLocalAddr() { return localAddr; }
 	Address GetPeerAddr() { return peerAddr; }
 	Address GetMasqAddr() { return masqAddr; }
+	PINDEX GetCallNumber() const { return m_call ? m_call->GetCallNumber() : 0; }
+	H225_CallIdentifier GetCallIdentifier() const { return m_call ? m_call->GetCallIdentifier() : 0; }
 	void BuildFacilityPDU(Q931 &, int, const PObject * = NULL);
 	void BuildProgressPDU(Q931 &, PBoolean fromDestination);
 	void BuildProceedingPDU(Q931 & ProceedingPDU, const H225_CallIdentifier & callId, unsigned crv);
@@ -195,6 +199,7 @@ public:
 	bool RerouteCall(CallLeg which, const PString & destination, bool h450transfer);
 	void RerouteCaller(PString destination);
 	void RerouteCalled(PString destination);
+	void PerformConnecting();
 
 	// override from class ServerSocket
 	virtual void Dispatch();
@@ -204,6 +209,10 @@ public:
 	void RemoveH245Handler();
 	void SaveTCS(const H245_TerminalCapabilitySet & tcs) { m_savedTCS = tcs; }
 	H245_TerminalCapabilitySet GetSavedTCS() const { return m_savedTCS; }
+#ifdef HAS_H235_MEDIA
+    bool HandleH235TCS(H245_TerminalCapabilitySet & tcs);
+    bool HandleH235OLC(H245_OpenLogicalChannel & olc);
+#endif
 	bool CompareH245Socket(H245Socket * sock) const { return sock == m_h245socket; }	// compare pointers !
 	
 protected:
@@ -211,6 +220,20 @@ protected:
 	
 	void SetRemote(CallSignalSocket *);
 	bool CreateRemote(H225_Setup_UUIE &setupBody);
+public:
+#ifdef HAS_H46018
+	bool CreateRemote(const H225_TransportAddress & addr);
+	bool OnSCICall(H225_CallIdentifier callID, H225_TransportAddress sigAdr);
+	bool IsCallFromTraversalServer() const { return m_callFromTraversalServer; }
+	bool IsCallToTraversalServer() const { return m_callToTraversalServer; }
+	void SetSessionMultiplexDestination(WORD session, bool isRTCP, const H323TransportAddress & toAddress, H46019Side side);
+	void SetLCMultiplexDestination(unsigned lc, bool isRTCP, const H323TransportAddress & toAddress, H46019Side side);
+	void SetLCMultiplexID(unsigned lc, bool isRTCP, PUInt32b multiplexID, H46019Side side);
+	void SetLCMultiplexSocket(unsigned lc, bool isRTCP, int multiplexSocket, H46019Side side);
+	const H245Handler * GetH245Handler() const { return m_h245handler; }
+#endif
+
+protected:
 	CallSignalSocket * GetRemote() const { return (CallSignalSocket *)remote; }
 
 	void ForwardCall(FacilityMsg *msg);
@@ -237,9 +260,9 @@ protected:
 	template<class UUIE> bool HandleH245Address(UUIE & uu)
 	{
 		if (uu.HasOptionalField(UUIE::e_h245Address)) {
-			 if (m_call)
+			if (m_call)
 				m_call->SetH245ResponseReceived();
-			 if (SetH245Address(uu.m_h245Address))
+			if (SetH245Address(uu.m_h245Address))
 				return (m_h245handler != 0);
 			uu.RemoveOptionalField(UUIE::e_h245Address);
 			return true;
@@ -322,6 +345,11 @@ private:
 	PBYTEArray m_rawSetup;
 	PMutex infomutex;    // Information PDU processing Mutex
 	H245_TerminalCapabilitySet m_savedTCS;	// saved tcs to re-send
+#ifdef HAS_H46018
+	bool m_callFromTraversalServer; // is this call from a traversal server ?
+	bool m_callToTraversalServer;
+	bool m_senderSupportsH46019Multiplexing;
+#endif
 };
 
 class CallSignalListener : public TCPListenSocket {
@@ -330,11 +358,115 @@ class CallSignalListener : public TCPListenSocket {
 #endif
 public:
 	CallSignalListener(const Address &, WORD);
+	~CallSignalListener();
 
 	// override from class TCPListenSocket
 	virtual ServerSocket *CreateAcceptor() const;
+
+protected:
+	Address m_addr;
 };
 
+#ifdef HAS_H46018
+
+class MultiplexRTPListener : public UDPSocket {
+#ifndef LARGE_FDSET
+	PCLASSINFO ( MultiplexRTPListener, UDPSocket )
+#endif
+public:
+	MultiplexRTPListener(WORD pt, WORD buffSize = 1536);
+	virtual ~MultiplexRTPListener();
+
+	virtual int GetOSSocket() const { return os_handle; }
+
+	virtual void ReceiveData();
+protected:
+	BYTE * wbuffer;
+	WORD wbufsize, buflen;
+};
+
+class H46019Channel
+{
+public:
+	H46019Channel(const H225_CallIdentifier & callid, WORD session, void * openedBy);
+
+	void Dump() const;
+
+	bool sideAReady(bool isRTCP) const { return isRTCP ? IsSet(m_addrA_RTCP) : IsSet(m_addrA); }
+	bool sideBReady(bool isRTCP) const { return isRTCP ? IsSet(m_addrB_RTCP) : IsSet(m_addrB); }
+	H46019Channel SwapSides() const; // return a copy with side A and B swapped
+
+	bool IsKeepAlive(void * data, unsigned len, bool isRTCP) { return isRTCP ? true : (len == 12); };
+
+	void HandlePacket(PUInt32b receivedMultiplexID, const H323TransportAddress & fromAddress, void * data, unsigned len, bool isRTCP);
+	static void Send(PUInt32b sendMultiplexID, const H323TransportAddress & toAddress, int ossocket, void * data, unsigned len);
+
+//protected:
+	H225_CallIdentifier m_callid;
+	WORD m_session;
+	void * m_openedBy;	// pointer to H245ProxyHandler used as an ID
+	void * m_otherSide;	// pointer to H245ProxyHandler used as an ID
+	H323TransportAddress m_addrA;
+	H323TransportAddress m_addrA_RTCP;
+	H323TransportAddress m_addrB;
+	H323TransportAddress m_addrB_RTCP;
+	PUInt32b m_multiplexID_fromA;
+	PUInt32b m_multiplexID_toA;
+	PUInt32b m_multiplexID_fromB;
+	PUInt32b m_multiplexID_toB;
+	int m_osSocketToA;
+	int m_osSocketToA_RTCP;
+	int m_osSocketToB;
+	int m_osSocketToB_RTCP;
+	bool m_EnableRTCPStats;
+};
+
+class MultiplexedRTPReader : public SocketsReader {
+public:
+	MultiplexedRTPReader();
+	virtual ~MultiplexedRTPReader();
+
+	virtual int GetRTPOSSocket() const { return m_multiplexRTPListener ? m_multiplexRTPListener->GetOSSocket() : INVALID_OSSOCKET; }
+	virtual int GetRTCPOSSocket() const { return m_multiplexRTCPListener ? m_multiplexRTCPListener->GetOSSocket() : INVALID_OSSOCKET; }
+
+protected:
+	virtual void OnStart();
+	virtual void ReadSocket(IPSocket * socket);
+
+	MultiplexRTPListener * m_multiplexRTPListener;
+	MultiplexRTPListener * m_multiplexRTCPListener;
+};
+
+// handles multiplexed RTP and keepAlives
+class MultiplexedRTPHandler : public Singleton<MultiplexedRTPHandler> {
+public:
+	MultiplexedRTPHandler();
+	virtual ~MultiplexedRTPHandler();
+
+	virtual void OnReload() { /* currently not runtime changable */ }
+
+	virtual void AddChannel(const H46019Channel & cha);
+	virtual void UpdateChannel(const H46019Channel & cha);
+	virtual H46019Channel GetChannel(const H225_CallIdentifier & callid, WORD session, void * openedBy) const;
+	virtual void RemoveChannels(const H225_CallIdentifier & callid);
+	virtual void DumpChannels(const PString & msg = "") const;
+
+	virtual void HandlePacket(PUInt32b receivedMultiplexID, const H323TransportAddress & fromAddress, void * data, unsigned len, bool isRTCP);
+
+	virtual int GetRTPOSSocket() const { return m_reader ? m_reader->GetRTPOSSocket() : INVALID_OSSOCKET; }
+	virtual int GetRTCPOSSocket() const { return m_reader ? m_reader->GetRTCPOSSocket() : INVALID_OSSOCKET; }
+
+	virtual PUInt32b GetMultiplexID(const H225_CallIdentifier & callid, WORD session, void * to);
+	virtual PUInt32b GetNewMultiplexID();
+
+protected:
+	MultiplexedRTPReader * m_reader;
+	mutable PReadWriteMutex listLock;
+	list<H46019Channel> m_h46019channels;
+	PUInt32b idCounter; // we should make sure this counter is _not_ reset on reload
+};
+#endif
+
 class ProxyHandler : public SocketsReader {
 public:
 	ProxyHandler(const PString& name);
