@@ -6,7 +6,7 @@
 // Avoid including large h225.h in RasSrv.h
 //
 // Copyright (c) Citron Network Inc. 2001-2003
-// Copyright (c) 2006-2010, Jan Willamowius
+// Copyright (c) 2006-2011, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -16,7 +16,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASPDU_H
-#define RASPDU_H "@(#) $Id: RasPDU.h,v 1.17 2010/10/25 15:01:51 willamowius Exp $"
+#define RASPDU_H "@(#) $Id: RasPDU.h,v 1.21 2011/11/27 13:30:05 willamowius Exp $"
 
 #include <list>
 #include "yasocket.h"
@@ -32,6 +32,7 @@ class RasListener;
 class MulticastListener;
 class CallSignalListener;
 class StatusListener;
+class MultiplexRTPListener;
 class RasServer;
 
 const unsigned MaxRasTag = H225_RasMessage::e_serviceControlResponse;
@@ -108,10 +109,6 @@ public:
 	void Release();
 
 	static void Initialize();
-	static RegistrationTable *GetEndpointTbl()
-	        { return EndpointTbl; }
-	static CallTable *GetCallTbl()
-	        { return CallTbl; }
 protected:
 	RasMsg(GatekeeperMessage *m) : m_msg(m) {}
 	RasMsg(const RasMsg &);
@@ -174,14 +171,10 @@ public:
 	bool IsReachable(const Address *) const;
 
 	RasListener *GetRasListener() const { return m_rasListener; }
-	MulticastListener *GetMulticastListener() const { return m_multicastListener; }
-	CallSignalListener *GetCallSignalListener() const { return m_callSignalListener; }
-	StatusListener *GetStatusListener() const { return m_statusListener; }
 
 	WORD GetRasPort() const { return m_rasPort; }
-	WORD GetMulticastPort() const { return m_multicastPort; }
 	WORD GetSignalPort() const { return m_signalPort; }
-	WORD GetStatusPort() const { return m_statusPort; }
+	Address GetAddress() const { return m_address; }
 
 protected:
 	bool ValidateSocket(IPSocket *, WORD &);
@@ -343,7 +336,7 @@ template<class RAS>
 class HookedPDU : public RasPDU<RAS> {
 public:
 	HookedPDU(GatekeeperMessage *m, RasMsg *p) : RasPDU<RAS>(m), m_opdu(p) {}
-	~HookedPDU() { m_opdu->Release(); }
+	~HookedPDU() { m_opdu->Release(); delete m_opdu; }
 
 	virtual bool Process() { return m_opdu->Process(); }
 
