@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASTBL_H
-#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.199 2011/07/02 11:51:38 shorne Exp $"
+#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.223 2011/12/19 01:14:02 shorne Exp $"
 
 #include <list>
 #include <map>
@@ -22,8 +22,13 @@
 #include "h225.h"
 #include "sigmsg.h"
 #include "config.h"
+#include "gktimer.h"
 #include "h323util.h"
 
+#ifdef HAS_H235_MEDIA
+#include "h235auth.h"
+#endif
+
 
 namespace Routing {
 	class Route;
@@ -33,10 +38,32 @@ class USocket;
 class CallSignalSocket;
 class RasServer;
 class Q931;
+class H323TransportAddress;
 
 enum CallLeg { Caller, Called };
 enum RerouteState { NoReroute, RerouteInitiated, Rerouting };
+enum H46019Side { SideA, SideB };
+const int INVALID_OSSOCKET = -1;
+const WORD INVALID_RTP_SESSION = 0;
+const PUInt32b INVALID_MULTIPLEX_ID = 0;
+const unsigned GNUGK_KEEPALIVE_RTP_PAYLOADTYPE = 127;	// GnuGk always sends this fixed payload type
 
+enum PortType { RASPort=1, Q931Port=2, H245Port=3, RTPPort=4, T120Port=5, RadiusPort=6, StatusPort=7 };
+enum PortAction { PortOpen=1, PortClose=2 };
+
+class DynamicPort
+{
+public:
+	DynamicPort(PortType type, PIPSocket::Address ip, WORD port) { m_type = type; m_ip = ip; m_port = port; }
+
+	bool operator==(const DynamicPort & other) const { return m_type == other.m_type && m_ip == other.m_ip && m_port == other.m_port; }
+
+	PortType m_type;
+	PIPSocket::Address m_ip;
+	WORD m_port;
+};
+
+ 
 // Template of smart pointer
 // The class T must have Lock() & Unlock() methods
 template<class T> class SmartPtr {
@@ -63,6 +90,7 @@ private:
 	T *pt;
 };
 
+enum H46019TraversalType { None, TraversalClient, TraversalServer };
 
 class EndpointRec
 {
@@ -147,7 +175,7 @@ public:
 	PString PrintPrefixCapacities() const;
 
 	void SetNAT(bool nat);
-	void SetNATAddress(const PIPSocket::Address &);
+	void SetNATAddress(const PIPSocket::Address &, WORD port = 0);
 	void SetSocket(CallSignalSocket *);
 	void SetH46024(bool support);
 	void SetH46024A(bool support);
@@ -195,10 +223,8 @@ public:
 	int GetEPNATType() const { return (int)m_epnattype; }
 	static PString GetEPNATTypeString(EPNatTypes nat);
 	bool SupportPreemption() const { return m_registrationPreemption; }
-	H225_AlternateGK GetAssignedGatekeeper() const { return m_assignedGatekeeper; }
 
 	int  Priority() const { return m_registrationPriority; }
-	bool HasNATSocket() const;
 	PTime GetUpdatedTime() const;
 
 	void SetUsesH460P(bool uses);
@@ -267,8 +293,10 @@ public:
 	PString GetAdditionalDestinationAlias() const { return m_additionalDestAlias; }
 
 	bool IsH46018Disabled() const { return m_h46018disabled; }
-	void SetUsesH46018(bool uses) { m_usesH46018 = uses; m_nat = uses; }
-	bool UsesH46018() const { return m_usesH46018; }
+	void SetTraversalRole(H46019TraversalType val) { m_traversalType = val; m_nat = (val == TraversalClient); }
+	H46019TraversalType GetTraversalRole() { return m_traversalType; }
+	bool IsTraversalServer() const { return m_traversalType == TraversalServer; }
+	bool IsTraversalClient() const { return m_traversalType == TraversalClient; }
 
 	// smart pointer for EndpointRec
 	typedef SmartPtr<EndpointRec> Ptr;
@@ -347,8 +375,8 @@ protected:
 	EPNatTypes m_epnattype;
 	bool m_usesH46023, m_H46024, m_H46024a, m_H46024b, m_natproxy, m_internal, m_remote;
 	bool m_h46018disabled;
-	bool m_usesH46018;
 	bool m_usesH460P;
+	H46019TraversalType m_traversalType;	// this is not what GnuGk acts like, but what this EPRec is a proxy for
 	
 	long m_bandwidth;	// bandwidth currently occupied by this endpoint
 	long m_maxBandwidth; // maximum bandwidth allowed for this endpoint
@@ -462,16 +490,16 @@ public:
 	EPQoS() { Init(); }
 	EPQoS(PTime lastMsg) { Init(); m_lastMsg = lastMsg; }
 	~EPQoS() { }
-
+ 
 	void Init();
 	PString AsString() const;
-
+ 
 	void IncrementCalls() { m_numCalls++; }
 	void SetAudioPacketLossPercent(float val) { m_audioPacketLossPercent = val; }
 	void SetAudioJitter(unsigned val) { m_audioJitter = val; }
 	void SetVideoPacketLossPercent(float val) { m_videoPacketLossPercent = val; }
 	void SetVideoJitter(unsigned val) { m_videoJitter = val; }
-
+ 
 protected:
 	PTime m_lastMsg;
 	unsigned m_numCalls;
@@ -489,12 +517,13 @@ public:
 	RegistrationTable();
 	~RegistrationTable();
 
-	endptr InsertRec(H225_RasMessage & rrq, PIPSocket::Address = INADDR_ANY);
+	endptr InsertRec(H225_RasMessage & rrq, PIPSocket::Address = GNUGK_INADDR_ANY);
+	endptr InsertRec(const H225_Setup_UUIE & setupBody, H225_TransportAddress addr);
 	void RemoveByEndptr(const endptr & eptr);
 
 	endptr FindByEndpointId(const H225_EndpointIdentifier & endpointId) const;
-	endptr FindBySignalAdr(const H225_TransportAddress &, PIPSocket::Address = INADDR_ANY) const;
-	endptr FindBySignalAdrIgnorePort(const H225_TransportAddress &, PIPSocket::Address = INADDR_ANY) const;
+	endptr FindBySignalAdr(const H225_TransportAddress &, PIPSocket::Address = GNUGK_INADDR_ANY) const;
+	endptr FindBySignalAdrIgnorePort(const H225_TransportAddress &, PIPSocket::Address = GNUGK_INADDR_ANY) const;
 	endptr FindOZEPBySignalAdr(const H225_TransportAddress &) const;
 	endptr FindByAliases(const H225_ArrayOf_AliasAddress & alias) const;
 	endptr FindFirstEndpoint(const H225_ArrayOf_AliasAddress & alias);
@@ -528,6 +557,7 @@ private:
 	endptr InternalInsertEP(H225_RasMessage &);
 	endptr InternalInsertOZEP(H225_RasMessage &, H225_LocationConfirm &);
 	endptr InternalInsertOZEP(H225_RasMessage &, H225_AdmissionConfirm &);
+	endptr InternalInsertOZEP(const H225_Setup_UUIE & setupBody, H225_TransportAddress addr);
 
 	void InternalPrint(USocket *, bool, std::list<EndpointRec *> *, PString &);
 	void InternalStatistics(const std::list<EndpointRec *> *, unsigned & s, unsigned & t, unsigned & g, unsigned & n) const;
@@ -571,14 +601,41 @@ private:
 
 template<class> class RasPDU;
 
-// record of one active call
-#ifdef HAS_H460
- class H4609_QosMonitoringReportData;
-#ifdef HAS_H46024B
- class H323TransportAddress;
+#ifdef HAS_H46018
+enum KeepAliveType { RTP, RTCP };
+
+class H46019KeepAlive
+{
+public:
+	H46019KeepAlive();
+	~H46019KeepAlive();
+
+	void SendKeepAlive(GkTimer* timer);
+	void StopKeepAlive();
+
+	unsigned flcn;
+	KeepAliveType type;
+#ifdef hasIPV6
+	sockaddr_in6 dest;
+#else
+	sockaddr_in dest;
 #endif
+	unsigned interval;
+	unsigned seq;
+	int ossocket;
+	PUInt32b multiplexID;
+	GkTimerManager::GkTimerHandle timer;
+};
 #endif
 
+// record of one active call
+#ifdef HAS_H460
+class H4609_QosMonitoringReportData;
+#ifdef HAS_H46024B
+class H323TransportAddress;
+#endif
+#endif
+ 
 #ifdef HAS_H46018
 // direction definitions for H.460.19
 #define H46019_NONE		0	// m_h46019dir = 0 ' No party needs H.460.19, so skip the code
@@ -637,6 +694,12 @@ public:
 		int proxyMode = ProxyDetect
 		);
 	
+	/// build a new call record with just a callID and transport adr (after H.460.18 SCI)
+	CallRec(
+		H225_CallIdentifier callID,
+		H225_TransportAddress sigAdr
+		);
+ 
 	CallRec(
 		CallRec *oldCall
 		);
@@ -707,21 +770,21 @@ public:
 	/** Can Signalling be offloaded
 	  */
 	bool NATSignallingOffload(bool isAnswer) const;
-
+ 
 #ifdef HAS_H46024B
     /** GetSignallingSocket */
     CallSignalSocket * H46024BSignalSocket(bool response);
-
+ 
 	/** Initiate Probe */
-	void H46024BInitiate(WORD sessionID, const H323TransportAddress & fwd, const H323TransportAddress & rev);
-
+	void H46024BInitiate(WORD sessionID, const H323TransportAddress & fwd, const H323TransportAddress & rev, unsigned muxID=0);
+ 
 	/** Response Probe */
 	void H46024BRespond();
-
+ 
     /** Set session callback flag */
 	void H46024BSessionFlag(WORD sessionID);
 #endif
-
+ 
 #endif
 
 	/** Return whether the endpoints are registered at the same gatekeeper so
@@ -766,7 +829,7 @@ public:
 
 	void SetConnected();
 
-	void Disconnect(bool = false); // Send Release Complete?
+	void Disconnect(bool = false); // send ReleaseComplete ?
 	void RemoveAll();
 	void RemoveSocket();
 	void SendReleaseComplete(const H225_CallTerminationCause * = 0);
@@ -800,6 +863,7 @@ public:
 		const PString & timestampFormat = PString::Empty()
 		) const;
 	PString PrintOn(bool verbose) const;
+	PString PrintPorts() const;
 
 	void Lock();
 	void Unlock();
@@ -860,7 +924,6 @@ public:
 
 	void SetSRC_media_IP(const PString & IP);
 	void SetDST_media_IP(const PString & IP);
-
 	void SetRTCP_sdes(bool isSRC, const PString & val);
 	void SetRTCP_SRC_sdes(const PString & val);
 	void SetRTCP_DST_sdes(const PString & val);
@@ -894,7 +957,7 @@ public:
 	int GetRTCP_DST_jitter_max() const { return m_rtcp_destination_jitter_max; }
 	int GetRTCP_DST_jitter_min() const { return m_rtcp_destination_jitter_min; }
 	int GetRTCP_DST_jitter_avg() const { return m_rtcp_destination_jitter_avg; }
-
+ 
 	// set video RTCP stats
 	void SetRTCP_SRC_video_packet_count(long val);
 	void SetRTCP_SRC_video_packet_lost(long val);
@@ -902,7 +965,7 @@ public:
 	void SetRTCP_DST_video_packet_count(long val);
 	void SetRTCP_DST_video_packet_lost(long val);
 	void SetRTCP_DST_video_jitter(int val);
-
+ 
 	// get video RTCP stats
 	long GetRTCP_SRC_video_packet_count() const { return m_rtcp_source_video_packet_count; }
 	long GetRTCP_SRC_video_packet_lost() const { return m_rtcp_source_video_packet_lost; }
@@ -977,6 +1040,12 @@ public:
 		m_acctUpdateTime = tm;
 	}
 
+	/* Reset timeout.
+		Used when switching from GKRouted to Direct call model to 
+		avoid Signalling timeouts
+	*/ 
+	void ResetTimeOut();
+
 	/** Check if:
 		- a signaling channel associated with this call is not timed out
 		  and the call should be disconnected (removed from CallTable);
@@ -1194,18 +1263,43 @@ public:
 	RerouteState GetRerouteState() const { return m_rerouteState; }
 	void SetRerouteDirection(CallLeg dir) { m_rerouteDirection = dir; }
 	CallLeg GetRerouteDirection() const { return m_rerouteDirection; }
-
+ 
 	void SetBindHint(const PString & ip) { m_bindHint = ip; }
 	PString GetBindHint() const { return m_bindHint; }
-
+ 
 	void SetCallerID(const PString & id) { m_callerID = id; }
 	PString GetCallerID() const { return m_callerID; }
 
+	void AddDynamicPort(const DynamicPort & port);
+	void RemoveDynamicPort(const DynamicPort & port);
+
 #ifdef HAS_H46018
+	bool IsH46018ReverseSetup() const { return m_h46018ReverseSetup; }	
+	void SetH46018ReverseSetup(bool val) { m_h46018ReverseSetup = val; }
 	bool H46019Required() const;
 	void StoreSetup(SignalingMsg * msg);
 	PBYTEArray RetrieveSetup();
 	int GetH46019Direction() const;
+
+	void AddRTPKeepAlive(unsigned flcn, const H323TransportAddress & keepAliveRTPAddr, unsigned keepAliveInterval, PUInt32b multiplexID);
+	void StartRTPKeepAlive(unsigned flcn, int RTPOSSocket);
+	void AddRTCPKeepAlive(unsigned flcn, const H245_UnicastAddress & keepAliveRTCPAddr, unsigned keepAliveInterval, PUInt32b multiplexID);
+	void StartRTCPKeepAlive(unsigned flcn, int RTCPOSSocket);
+	void RemoveKeepAlives(unsigned flcn);
+	void RemoveKeepAllAlives();
+	
+	void SetSessionMultiplexDestination(WORD session, void * openedBy, bool isRTCP, const H323TransportAddress & toAddress, H46019Side side);
+	void SetLCMultiplexDestination(unsigned lc, void * openedBy, bool isRTCP, const H323TransportAddress & toAddress, H46019Side side);
+	void SetLCMultiplexID(unsigned lc, void * openedBy, bool isRTCP, PUInt32b multiplexID, H46019Side side);
+	void SetLCMultiplexSocket(unsigned lc, void * openedBy, bool isRTCP, int multiplexSocket, H46019Side side);
+#endif
+
+#ifdef HAS_H235_MEDIA
+    typedef NATType EncDir;
+    H235Authenticators & GetAuthenticators() { return m_authenticators; }
+    void SetMediaEncryption(EncDir dir);
+    bool IsMediaEncryption()                 { return m_encyptDir != none;  }
+    EncDir GetEncryptDirection()             { return m_encyptDir;      }
 #endif
 
 private:
@@ -1285,21 +1379,21 @@ private:
 	long m_rtcp_destination_video_packet_count;
 	long m_rtcp_source_video_packet_lost;
 	long m_rtcp_destination_video_packet_lost;
-
+ 
 	int m_rtcp_source_video_jitter_min;
 	int m_rtcp_source_video_jitter_max;
 	int m_rtcp_source_video_jitter_avg;
-
+ 
 	int m_rtcp_destination_video_jitter_min;
 	int m_rtcp_destination_video_jitter_max;
 	int m_rtcp_destination_video_jitter_avg;
-
+ 
 	int m_rtcp_source_video_jitter_avg_count;
 	long m_rtcp_source_video_jitter_avg_sum;
-
+ 
 	int m_rtcp_destination_video_jitter_avg_count;
 	long m_rtcp_destination_video_jitter_avg_sum;
-
+ 
 	/// current timeout (or duration limit) for the call
 	time_t m_timeout;
 	/// timestamp for call timeout measuring
@@ -1351,13 +1445,14 @@ private:
 #ifdef HAS_H46023
 	NatStrategy m_natstrategy;
 #endif
-
+ 
 #ifdef HAS_H46024B
 	struct H46024Balternate {
 		 H245_TransportAddress forward;
 		 H245_TransportAddress reverse;
+		 unsigned multiplexID;
 	};
-
+ 
 	std::map<WORD,H46024Balternate> m_H46024Balternate;
 	void BuildH46024AnnexBMessage(bool initiate,H245_MultimediaSystemControlMessage & h245msg, const std::map<WORD,H46024Balternate> & alt);
 	list<int> m_h46024Bflag;
@@ -1400,12 +1495,24 @@ private:
 #ifdef HAS_H46018
 	/// processed Setup data ready to be sent to the callee (for H.460.18)
 	PBYTEArray m_processedSetup;
+	std::map<unsigned, H46019KeepAlive> m_RTPkeepalives;
+	std::map<unsigned, H46019KeepAlive> m_RTCPkeepalives;
 #endif
 	PUInt64 m_clientAuthId;
 	PString m_bindHint;	// outgoing IP or empty
 	RerouteState m_rerouteState;	// is Pause&Reroute transfer in progress ?
+	bool m_h46018ReverseSetup;
+	bool m_callfromTraversalClient;
+	bool m_callfromTraversalServer;
 	CallLeg m_rerouteDirection;
 	PString m_callerID;	// forced caller ID or empty
+	PMutex m_portListMutex;
+	list<DynamicPort> m_dynamicPorts;
+
+#ifdef HAS_H235_MEDIA
+    H235Authenticators m_authenticators;
+    EncDir m_encyptDir;
+#endif
 };
 
 typedef CallRec::Ptr callptr;
@@ -1454,6 +1561,7 @@ public:
 	void DropCalledParty();
 
 	void PrintCurrentCalls(USocket *client, bool verbose=FALSE) const;
+	void PrintCurrentCallsPorts(USocket *client) const;
 	PString PrintStatistics() const;
 
 #ifdef HAS_H460
@@ -1580,7 +1688,7 @@ inline H225_EndpointIdentifier EndpointRec::GetEndpointIdentifier() const
   
 inline int EndpointRec::GetTimeToLive() const
 {
-	if (m_nat || UsesH46018()) {
+	if (m_nat || IsTraversalClient()) {
 		// force timeToLive to 5 - 30 sec, 19 sec if not set
 		return m_timeToLive == 0 ? 19 : max(5, min(30, m_timeToLive));
 	}
@@ -1638,12 +1746,12 @@ inline void EndpointRec::SetH46024(bool support)
 {
 	m_H46024 = support;
 }
-
+ 
 inline void EndpointRec::SetH46024A(bool support)
 {
 	m_H46024a = support;
 }
-
+ 
 inline void EndpointRec::SetH46024B(bool support)
 {
 	m_H46024b = support;
@@ -1658,7 +1766,7 @@ inline bool EndpointRec::SupportH46024() const
 {
 	return m_H46024;
 }
-
+ 
 inline bool EndpointRec::SupportH46024A() const
 {
 	return m_H46024a;
@@ -1668,7 +1776,7 @@ inline bool EndpointRec::SupportH46024B() const
 {
 	return m_H46024b;
 }
-
+ 
 inline bool EndpointRec::UseH46024B() const
 {
 	int nat = (int)m_epnattype;
@@ -1742,11 +1850,6 @@ inline bool EndpointRec::IsUpdated(const PTime *now) const
 	return (!ttl || (*now - m_updatedTime).GetSeconds() < ttl);
 }
 
-inline bool EndpointRec::HasNATSocket() const
-{
-	return m_natsocket;
-}
-
 inline PTime EndpointRec::GetUpdatedTime() const
 { 
 	PWaitAndSignal lock(m_usedLock);
@@ -1919,6 +2022,11 @@ inline void CallRec::SetDisconnectCauseTranslated( unsigned causeCode )
 	m_disconnectCauseTranslated = causeCode;
 }
 
+inline void CallRec::ResetTimeOut()
+{
+	m_timeout = 0;
+}
+
 inline bool CallRec::IsTimeout(const time_t now) const
 {
 	bool result = (m_timeout > 0) && (now >= m_timer) && ((now - m_timer) >= m_timeout);
