@@ -50,7 +50,7 @@ const char *SectionName[] = {
 const long DEFAULT_ROUTE_REQUEST_TIMEOUT = 10;
 const char* const CTIsection = "CTI::Agents";
 
-bool DNSPolicy::m_resolveNonLocalLRQs = false;
+bool DNSPolicy::m_resolveNonLocalLRQs = true;
 
 Route::Route() : m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_priority(1)
 {
@@ -194,19 +194,13 @@ template<> H225_ArrayOf_AliasAddress *FacilityRequest::GetAliases()
 bool Policy::Handle(SetupRequest& request)
 {
 	if( IsActive() ) {
-#if PTRACING
 		const PString tagname = request.GetWrapper()->GetTagName();
 		const unsigned crv = request.GetWrapper()->GetCallReference();
 		PTRACE(5, "ROUTING\tChecking policy " << m_name
-			<< " for request " << tagname << " CRV=" << crv
-			);
-#endif
+			<< " for request " << tagname << " CRV=" << crv);
 		if (OnRequest(request)) {
-#if PTRACING
 			PTRACE(5, "ROUTING\tPolicy " << m_name
-				<< " applied to the request " << tagname << " CRV=" << crv
-				);
-#endif
+				<< " applied to the request " << tagname << " CRV=" << crv);
 			return true;
 		}
 	}
@@ -216,19 +210,14 @@ bool Policy::Handle(SetupRequest& request)
 bool Policy::Handle(FacilityRequest& request)
 {
 	if( IsActive() ) {
-#if PTRACING
 		const PString tagname = request.GetWrapper()->GetTagName();
 		const unsigned crv = request.GetWrapper()->GetCallReference();
 		PTRACE(5, "ROUTING\tChecking policy " << m_name
 			<< " for request " << tagname << " CRV=" << crv
 			);
-#endif
 		if (OnRequest(request)) {
-#if PTRACING
 			PTRACE(5, "ROUTING\tPolicy " << m_name
-				<< " applied to the request " << tagname << " CRV=" << crv
-				);
-#endif
+				<< " applied to the request " << tagname << " CRV=" << crv);
 			return true;
 		}
 	}
@@ -368,7 +357,7 @@ bool AliasesPolicy::OnRequest(FacilityRequest & request)
 }
 
 
-std::map<PString, H225_TransportAddress> ExplicitPolicy::m_destMap;
+std::map<PString, PString> ExplicitPolicy::m_destMap;
 
 ExplicitPolicy::ExplicitPolicy()
 {
@@ -382,24 +371,96 @@ void ExplicitPolicy::OnReload()
 	for (PINDEX i = 0; i < mappings.GetSize(); ++i) {
 		PString src = mappings.GetKeyAt(i);
 		PString dest = mappings.GetDataAt(i);
-		if (!dest.IsEmpty()) {
-			H225_TransportAddress addr;
-			if (GetTransportAddress(dest, 1720, addr)) {
-				m_destMap[src] = addr;
+		H225_TransportAddress srcAddr;
+		if (GetTransportAddress(src, GK_DEF_ENDPOINT_SIGNAL_PORT, srcAddr)) {
+			if (!dest.IsEmpty()) {
+				if (IsIPAddress(dest)) {
+					// test parse IP, but store again as string
+					H225_TransportAddress destAddr;
+					if (GetTransportAddress(dest, GK_DEF_ENDPOINT_SIGNAL_PORT, destAddr)) {
+						m_destMap[AsDotString(srcAddr, false)] = AsDotString(destAddr, false);
+					} else {
+						PTRACE(1, "Error parsing dest entry in [Routing::Explicit]: " << src << "=" << dest);
+					}
+				} else {
+					// store anything else as string, will be used as alias
+					m_destMap[AsDotString(srcAddr, false)] = dest;
+				}
 			} else {
-				PTRACE(1, "Error parsing entry in [Routing::Explicit]: " << src << "=" << dest);
+				PTRACE(1, "Error: Empty dest entry in [Routing::Explicit]: " << src << "=");
 			}
+		} else {
+			PTRACE(1, "Error parsing src entry in [Routing::Explicit]: " << src << "=" << dest);
 		}
 	}
 }
 
-void ExplicitPolicy::MapDestination(H225_TransportAddress & addr)
+void ExplicitPolicy::MapDestination(H225_AdmissionRequest & arq)
 {
+	H225_TransportAddress & addr = arq.m_destCallSignalAddress;
 	PString orig = AsDotString(addr, false);	// original IP without port
-	std::map<PString, H225_TransportAddress>::const_iterator i = m_destMap.find(orig);
+	std::map<PString, PString>::const_iterator i = m_destMap.find(orig);
 	if (i != m_destMap.end()) {
-		addr = i->second;
-		PTRACE(4, "[Routing::Explicit]: map destination " << orig << " to " << AsDotString(i->second));
+		PString newDest = i->second;
+		if (IsIPAddress(newDest)) {
+			// just rewrite the IP
+			H225_TransportAddress destAddr;
+			GetTransportAddress(newDest, GK_DEF_ENDPOINT_SIGNAL_PORT, destAddr); // ignore result, we have parsed these before
+			addr = destAddr;
+		} else {
+			// delete IP and set a new destination alias
+			arq.RemoveOptionalField(H225_AdmissionRequest::e_destCallSignalAddress);
+			arq.IncludeOptionalField(H225_AdmissionRequest::e_destinationInfo);
+			arq.m_destinationInfo.SetSize(1);
+			H323SetAliasAddress(newDest, arq.m_destinationInfo[0]);
+		}
+		PTRACE(4, "[Routing::Explicit]: map destination " << orig << " to " << newDest);
+	}
+}
+
+void ExplicitPolicy::MapDestination(H225_Setup_UUIE & setupBody)
+{
+	H225_TransportAddress & addr = setupBody.m_destCallSignalAddress;
+	PString orig = AsDotString(addr, false);	// original IP without port
+	std::map<PString, PString>::const_iterator i = m_destMap.find(orig);
+	if (i != m_destMap.end()) {
+		PString newDest = i->second;
+		if (IsIPAddress(newDest)) {
+			// just rewrite the IP
+			H225_TransportAddress destAddr;
+			GetTransportAddress(newDest, GK_DEF_ENDPOINT_SIGNAL_PORT, destAddr); // ignore result, we have parsed these before
+			addr = destAddr;
+		} else {
+			// delete IP and set a new destination alias
+			setupBody.RemoveOptionalField(H225_Setup_UUIE::e_destCallSignalAddress);
+			setupBody.IncludeOptionalField(H225_Setup_UUIE::e_destinationAddress);
+			setupBody.m_destinationAddress.SetSize(1);
+			H323SetAliasAddress(newDest, setupBody.m_destinationAddress[0]);
+		}
+		PTRACE(4, "[Routing::Explicit]: map destination " << orig << " to " << newDest);
+	}
+}
+
+void ExplicitPolicy::MapDestination(H225_Facility_UUIE & facilityBody)
+{
+	H225_TransportAddress & addr = facilityBody.m_alternativeAddress;
+	PString orig = AsDotString(addr, false);	// original IP without port
+	std::map<PString, PString>::const_iterator i = m_destMap.find(orig);
+	if (i != m_destMap.end()) {
+		PString newDest = i->second;
+		if (IsIPAddress(newDest)) {
+			// just rewrite the IP
+			H225_TransportAddress destAddr;
+			GetTransportAddress(newDest, GK_DEF_ENDPOINT_SIGNAL_PORT, destAddr); // ignore result, we have parsed these before
+			addr = destAddr;
+		} else {
+			// delete IP and set a new destination alias
+			facilityBody.RemoveOptionalField(H225_Facility_UUIE::e_alternativeAddress);
+			facilityBody.IncludeOptionalField(H225_Facility_UUIE::e_alternativeAliasAddress);
+			facilityBody.m_alternativeAliasAddress.SetSize(1);
+			H323SetAliasAddress(newDest, facilityBody.m_alternativeAliasAddress[0]);
+		}
+		PTRACE(4, "[Routing::Explicit]: map destination " << orig << " to " << newDest);
 	}
 }
 
@@ -407,11 +468,12 @@ bool ExplicitPolicy::OnRequest(AdmissionRequest & request)
 {
 	H225_AdmissionRequest & arq = request.GetRequest();
 	if (arq.HasOptionalField(H225_AdmissionRequest::e_destCallSignalAddress)) {
-		MapDestination(arq.m_destCallSignalAddress);
+		MapDestination(arq);
+		// check if the mapping removed the destination IP
+		if (!arq.HasOptionalField(H225_AdmissionRequest::e_destCallSignalAddress))
+			return false;
 		Route route(m_name, arq.m_destCallSignalAddress);
-		route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
-			route.m_destAddr
-			);
+		route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(route.m_destAddr);
 #ifdef HAS_H46023
         if (!route.m_destEndpoint && 
            arq.HasOptionalField(H225_AdmissionRequest::e_genericData) &&
@@ -434,9 +496,7 @@ bool ExplicitPolicy::OnRequest(SetupRequest & request)
 	if (setup.HasOptionalField(H225_Setup_UUIE::e_destCallSignalAddress)) {
 		// don't map IP here, for Setup already done in OnSetup()
 		Route route(m_name, setup.m_destCallSignalAddress);
-		route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
-			route.m_destAddr
-			);
+		route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(route.m_destAddr);
 		return request.AddRoute(route);
 	}
 	return false;
@@ -446,11 +506,12 @@ bool ExplicitPolicy::OnRequest(FacilityRequest & request)
 {
 	H225_Facility_UUIE & facility = request.GetRequest();
 	if (facility.HasOptionalField(H225_Facility_UUIE::e_alternativeAddress)) {
-		MapDestination(facility.m_alternativeAddress);
+		MapDestination(facility);
+		// check if the mapping removed the destination IP
+		if (!facility.HasOptionalField(H225_Facility_UUIE::e_alternativeAddress))
+			return false;
 		Route route(m_name, facility.m_alternativeAddress);
-		route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
-			route.m_destAddr
-			);
+		route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(route.m_destAddr);
 		return request.AddRoute(route);
 	}
 	return false;
@@ -580,16 +641,31 @@ bool ParentPolicy::OnRequest(FacilityRequest & facility_obj)
 	return !(facility_obj.GetFlags() & RoutingRequest::e_fromParent) && m_gkClient->SendARQ(facility_obj);
 }
 
+
 DNSPolicy::DNSPolicy()
 {
 	m_name = "DNS";
 	m_resolveNonLocalLRQs = Toolkit::AsBool(GkConfig()->GetString("Routing::DNS", "ResolveNonLocalLRQ", "1"));
 }
 
-bool DNSPolicy::FindByAliases(
-	RoutingRequest &request,
-	H225_ArrayOf_AliasAddress &aliases
-	)
+bool DNSPolicy::DNSLookup(const PString & hostname, PIPSocket::Address & addr) const
+{
+	struct addrinfo hints;
+	struct addrinfo * result = NULL;
+	memset(&hints, 0, sizeof(hints));
+	if (Toolkit::Instance()->IsIPv6Enabled()) {
+		hints.ai_family = AF_UNSPEC;
+	} else {
+		hints.ai_family = AF_INET;
+	}
+	if (getaddrinfo(hostname, NULL, &hints, &result) != 0)
+		return false;
+	addr = PIPSocket::Address(result->ai_family, result->ai_addrlen, result->ai_addr);
+	freeaddrinfo(result);
+	return true;
+}
+
+bool DNSPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
 	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
 		// don't apply DNS to dialedDigits
@@ -601,11 +677,12 @@ bool DNSPolicy::FindByAliases(
 		PString domain = (at != P_MAX_INDEX) ? alias.Mid(at + 1) : alias;
 		if (domain.Find("ip$") == 0)
 			domain.Replace("ip$", "", false);
-		H225_TransportAddress dest;
-		if (GetTransportAddress(domain, GK_DEF_ENDPOINT_SIGNAL_PORT, dest)) {
-			PIPSocket::Address addr;
-			if (!(GetIPFromTransportAddr(dest, addr) && addr.IsValid()))
-				continue;
+		PStringArray parts = SplitIPAndPort(domain, GK_DEF_ENDPOINT_SIGNAL_PORT);
+		domain = parts[0];
+		WORD port = (WORD)parts[1].AsUnsigned();
+		PIPSocket::Address addr;
+		if (DNSLookup(domain, addr) && addr.IsValid()) {
+			H225_TransportAddress dest = SocketToH225TransportAddr(addr, port);
 			if (Toolkit::Instance()->IsGKHome(addr)) {
 				// check if the domain is my IP, if so route to local endpoint if available
 				H225_ArrayOf_AliasAddress find_aliases;
@@ -643,11 +720,12 @@ bool DNSPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddre
 		PString domain = (at != P_MAX_INDEX) ? alias.Mid(at + 1) : alias;
 		if (domain.Find("ip$") == 0)
 			domain.Replace("ip$", "", false);
-		H225_TransportAddress dest;
-		if (GetTransportAddress(domain, GK_DEF_ENDPOINT_SIGNAL_PORT, dest)) {
-			PIPSocket::Address addr;
-			if (!(GetIPFromTransportAddr(dest, addr) && addr.IsValid()))
-				continue;
+		PStringArray parts = SplitIPAndPort(domain, GK_DEF_ENDPOINT_SIGNAL_PORT);
+		domain = parts[0];
+		WORD port = (WORD)parts[1].AsUnsigned();
+		PIPSocket::Address addr;
+		if (DNSLookup(domain, addr) && addr.IsValid()) {
+			H225_TransportAddress dest = SocketToH225TransportAddr(addr, port);
 			if (Toolkit::Instance()->IsGKHome(addr)) {
 				// only apply DNS policy to LRQs that resolve locally
 				H225_ArrayOf_AliasAddress find_aliases;
@@ -677,7 +755,7 @@ bool DNSPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddre
 		}
 	}
 	if (!m_resolveNonLocalLRQs) {
-		PTRACE(4, "ROUTING\tPolicy DNS only routes LRQs that resolve locally");
+		PTRACE(4, "ROUTING\tPolicy DNS configured to only route LRQs that resolve locally");
 	}
 	return false;
 }
@@ -720,7 +798,7 @@ void VirtualQueue::OnReload()
 	m_requestTimeout = GkConfig()->GetInteger(
 		CTIsection, 
 		GkConfig()->HasKey(CTIsection,"RequestTimeout")
-			?"RequestTimeout":"CTI_Timeout", 
+			? "RequestTimeout" : "CTI_Timeout", 
 		DEFAULT_ROUTE_REQUEST_TIMEOUT
 		) * 1000;
 	m_requestTimeout = PMAX(100,m_requestTimeout);	// min wait: 100 msec
@@ -858,20 +936,20 @@ bool VirtualQueue::IsDestinationVirtualQueue(
 bool VirtualQueue::RouteToAlias(
 	/// aliases for the routing target (an agent that the call will be routed to) 
 	/// that will replace the original destination info
-	const H225_ArrayOf_AliasAddress& agent,
+	const H225_ArrayOf_AliasAddress & agent,
 	/// ip that will replace the destinationCallSignalAddress (RouteToGateway)
 	/// used only if set (!= NULL)
-	const PString& destinationip,
+	const PString & destinationip,
 	/// identifier of the endpoint associated with the route request
-	const PString& callingEpId, 
+	const PString & callingEpId, 
 	/// CRV of the call associated with the route request
 	unsigned crv,
 	/// callID of the call associated with the route request
-	const PString& callID,
+	const PString & callID,
 	// outgoing IP or empty
-	const PString& bindIP,
+	const PString & bindIP,
 	// caller ID or empty
-	const PString& callerID,
+	const PString & callerID,
 	/// should this call be rejected
 	bool reject
 	)
@@ -945,7 +1023,7 @@ bool VirtualQueue::RouteToAlias(
 	)
 {
 	H225_ArrayOf_AliasAddress alias;
-	if (targetAlias != "") {
+	if (targetAlias != "" && targetAlias != "-") {
 		alias.SetSize(1);
 		H323SetAliasAddress(targetAlias, alias[0]);
 	}
@@ -1047,7 +1125,7 @@ bool VirtualQueuePolicy::OnRequest(AdmissionRequest & request)
 			PString calledIP = "unknown";
 			PString vendorInfo;
 			if (arq.HasOptionalField(H225_AdmissionRequest::e_destCallSignalAddress)) {
-				calledIP = AsString(arq.m_destCallSignalAddress, false);
+				calledIP = AsDotString(arq.m_destCallSignalAddress, false);
 			}
 			if (ep->GetEndpointType().HasOptionalField(H225_EndpointType::e_vendor)) {
 				if (ep->GetEndpointType().m_vendor.HasOptionalField(H225_VendorIdentifier::e_productId)) {
@@ -1069,11 +1147,9 @@ bool VirtualQueuePolicy::OnRequest(AdmissionRequest & request)
 				if (!arq.HasOptionalField(H225_AdmissionRequest::e_destCallSignalAddress)) {
 					arq.IncludeOptionalField(H225_AdmissionRequest::e_destCallSignalAddress);
 				}
-				PStringArray adr_parts = callSigAdr->Tokenise(":", FALSE);
+				PStringArray adr_parts = SplitIPAndPort(*callSigAdr, GK_DEF_ENDPOINT_SIGNAL_PORT);
 				PString ip = adr_parts[0];
 				WORD port = (WORD)(adr_parts[1].AsInteger());
-				if (port == 0)
-					port = GK_DEF_ENDPOINT_SIGNAL_PORT;
 				arq.m_destCallSignalAddress = SocketToH225TransportAddr(ip, port);
 			}
 			delete callSigAdr;
@@ -1135,11 +1211,9 @@ bool VirtualQueuePolicy::OnRequest(LocationRequest & request)
 			request.SetCallerID(*callerID);
 			if (!reject && !callSigAdr->IsEmpty()) {
 				// 'explicit' policy can't handle LRQs, so we do it directly
-				PStringArray adr_parts = callSigAdr->Tokenise(":", FALSE);
+				PStringArray adr_parts = SplitIPAndPort(*callSigAdr, GK_DEF_ENDPOINT_SIGNAL_PORT);
 				PString ip = adr_parts[0];
 				WORD port = (WORD)(adr_parts[1].AsInteger());
-				if (port == 0)
-					port = GK_DEF_ENDPOINT_SIGNAL_PORT;
 				Route route("vqueue", SocketToH225TransportAddr(ip, port));
 				route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
 					route.m_destAddr
@@ -1183,7 +1257,9 @@ bool VirtualQueuePolicy::OnRequest(SetupRequest & request)
 		PString * bindIP = new PString();
 		PString * callerID = new PString();
 		PString callid = AsString(setup.m_callIdentifier.m_guid);
-		PString src = AsString(setup.m_sourceAddress);
+		PString src;
+		if (setup.HasOptionalField(H225_Setup_UUIE::e_sourceAddress))
+			src = AsString(setup.m_sourceAddress);
 		PIPSocket::Address localAddr;
 		WORD localPort;
 		request.GetWrapper()->GetLocalAddr(localAddr, localPort);
@@ -1225,11 +1301,9 @@ bool VirtualQueuePolicy::OnRequest(SetupRequest & request)
 			if (!setup.HasOptionalField(H225_Setup_UUIE::e_destCallSignalAddress)) {
 				setup.IncludeOptionalField(H225_Setup_UUIE::e_destCallSignalAddress);
 			}
-			PStringArray adr_parts = callSigAdr->Tokenise(":", FALSE);
+			PStringArray adr_parts = SplitIPAndPort(*callSigAdr, GK_DEF_ENDPOINT_SIGNAL_PORT);
 			PString ip = adr_parts[0];
 			WORD port = (WORD)(adr_parts[1].AsInteger());
-			if (port == 0)
-				port = GK_DEF_ENDPOINT_SIGNAL_PORT;
 			setup.m_destCallSignalAddress = SocketToH225TransportAddr(ip, port);
 		}
 		delete callSigAdr;
@@ -1285,10 +1359,8 @@ NumberAnalysisPolicy::NumberAnalysisPolicy()
 	stable_sort(m_prefixes.begin(), m_prefixes.end(), PrefixGreater());
 	
 	PTRACE(5, "ROUTING\t" << m_name << " policy loaded with " << m_prefixes.size()
-		<< " prefix entries"
-		);
-		
-#if PTRACING
+		<< " prefix entries");
+
 	if (PTrace::CanTrace(6)) {
 		ostream &strm = PTrace::Begin(6, __FILE__, __LINE__);
 		strm << "ROUTING\t" << m_name << " policy prefixes:" << endl;
@@ -1298,7 +1370,6 @@ NumberAnalysisPolicy::NumberAnalysisPolicy()
 				<< m_prefixes[i].m_maxLength << endl;
 		PTrace::End(strm);
 	}
-#endif /* PTRACING */
 }
 
 bool NumberAnalysisPolicy::OnRequest(AdmissionRequest & request)
@@ -1362,6 +1433,12 @@ bool NumberAnalysisPolicy::OnRequest(SetupRequest & request)
 }
 
 
+ENUMPolicy::ENUMPolicy()
+{
+	m_name = "ENUM";
+	m_resolveLRQs = Toolkit::AsBool(GkConfig()->GetString("Routing::ENUM", "ResolveLRQ", "0"));
+}
+
 bool ENUMPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
 #if P_DNS
@@ -1398,10 +1475,14 @@ bool ENUMPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddre
 	return false;
 }
 
-bool ENUMPolicy::FindByAliases(LocationRequest & /* request */, H225_ArrayOf_AliasAddress & /* aliases */)
+bool ENUMPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
-    PTRACE(4, "ROUTING\tPolicy ENUM not supported for LRQ");
-	return false; // ENUMPolicy::FindByAliases((RoutingRequest&)request, aliases);
+	if (m_resolveLRQs) {
+		return ENUMPolicy::FindByAliases((RoutingRequest&)request, aliases);
+	} else {
+		PTRACE(4, "ROUTING\tPolicy ENUM configured not to resolve LRQs");
+		return false;
+	}
 }
 
 
@@ -1575,7 +1656,9 @@ bool SqlPolicy::OnRequest(SetupRequest & request)
 	WORD localPort;
 	request.GetWrapper()->GetLocalAddr(localAddr, localPort);
 	PString calledIP = localAddr;	// TODO: only correct if a gatekeeper IP was called, should we use explicit IP if present ?
-	PString caller = AsString(setup.m_sourceAddress, FALSE);
+	PString caller;
+	if (setup.HasOptionalField(H225_Setup_UUIE::e_sourceAddress))
+		caller = AsString(setup.m_sourceAddress, FALSE);
 	PString callingStationId = request.GetCallingStationId();
 	PString callid = AsString(setup.m_callIdentifier.m_guid);
 	PString messageType = "Setup";
@@ -1672,11 +1755,9 @@ void SqlPolicy::DatabaseLookup(
 			int row = 0;
 			do {
 				PString destinationIp = resultRow[0].first;
-				PStringArray adr_parts = destinationIp.Tokenise(":", FALSE);
+				PStringArray adr_parts = SplitIPAndPort(destinationIp, GK_DEF_ENDPOINT_SIGNAL_PORT);
 				PString ip = adr_parts[0];
 				WORD port = (WORD)(adr_parts[1].AsInteger());
-				if (port == 0)
-					port = GK_DEF_ENDPOINT_SIGNAL_PORT;
 
 				Route route("Sql", SocketToH225TransportAddr(ip, port));
 				route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(route.m_destAddr);
@@ -1710,11 +1791,9 @@ void SqlPolicy::DatabaseLookup(
 			do {
 				PString destinationAlias = resultRow[0].first;
 				PString destinationIp = resultRow[1].first;
-				PStringArray adr_parts = destinationIp.Tokenise(":", FALSE);
+				PStringArray adr_parts = SplitIPAndPort(destinationIp, GK_DEF_ENDPOINT_SIGNAL_PORT);
 				PString ip = adr_parts[0];
 				WORD port = (WORD)(adr_parts[1].AsInteger());
-				if (port == 0)
-					port = GK_DEF_ENDPOINT_SIGNAL_PORT;
 
 				Route route("Sql", SocketToH225TransportAddr(ip, port));
 				route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(route.m_destAddr);
@@ -1742,8 +1821,13 @@ CatchAllPolicy::CatchAllPolicy()
 	PConfig* cfg = GkConfig();
 	m_catchAllAlias = cfg->GetString(defaultPolicySection, "CatchAllAlias", "catchall");
 	m_catchAllIP = cfg->GetString(defaultPolicySection, "CatchAllIP", "");
-	if (!m_catchAllIP.IsEmpty() && (m_catchAllIP.Find(':') == P_MAX_INDEX)) {
-		m_catchAllIP += ":1720";	// assume default port if none specified
+	if (!m_catchAllIP.IsEmpty()) {
+		PStringArray parts = SplitIPAndPort(m_catchAllIP, GK_DEF_ENDPOINT_SIGNAL_PORT);
+		if (IsIPv6Address(parts[0])) {
+			m_catchAllIP = "[" + parts[0] + "]:" + parts[1];
+		} else {
+			m_catchAllIP = parts[0] + ":" + parts[1];
+		}
 	}
 }
 
