@@ -78,6 +78,12 @@ void SoftPBX::PrintCurrentCalls(USocket *client, bool verbose)
 	CallTable::Instance()->PrintCurrentCalls(client, verbose);
 }
 
+void SoftPBX::PrintCurrentCallsPorts(USocket *client)
+{
+	PTRACE(3, "GK\tSoftPBX: PrintCurrentCallsPorts");
+	CallTable::Instance()->PrintCurrentCallsPorts(client);
+}
+
 void SoftPBX::PrintStatistics(USocket *client, bool)
 {
 	PTRACE(3, "GK\tSoftPBX: PrintStatistics");
@@ -134,10 +140,13 @@ void SoftPBX::UnregisterIp(const PString & Ip)
 	PTRACE(3, "GK\tSoftPBX: UnregisterIp " << Ip);
 
 	endptr ep;
-	if (Ip.Find(':') == P_MAX_INDEX)
+	if ((IsIPv4Address(Ip) && Ip.Find(':') == P_MAX_INDEX)
+		|| (IsIPv6Address(Ip) && Ip.Find("]:") == P_MAX_INDEX)
+		) {
 		ep = RegistrationTable::Instance()->FindBySignalAdrIgnorePort(callSignalAddress);	// no port specified
-	else
+	} else {
 		ep = RegistrationTable::Instance()->FindBySignalAdr(callSignalAddress);	// port specify, search for it
+	}
 	if (!ep) {
 		PString msg("IP " + Ip + " not found!");
 		PTRACE(1, "GK\tSoftPBX: " + msg);
@@ -213,7 +222,9 @@ void SoftPBX::DisconnectIp(const PString & Ip)
 	PTRACE(3, "GK\tSoftPBX: DisconnectIp " << Ip);
 
 	callptr Call;
-	if (Ip.Find(':') == P_MAX_INDEX) {
+	if ((IsIPv4Address(Ip) && Ip.Find(':') == P_MAX_INDEX)
+		|| (IsIPv6Address(Ip) && Ip.Find("]:") == P_MAX_INDEX)
+		) {
 		// no port specified
 		while (Call = CallTable::Instance()->FindBySignalAdrIgnorePort(callSignalAddress)) {
 			unsigned CallNumber = Call->GetCallNumber();
