@@ -2,7 +2,7 @@
 //
 // Toolkit base class for the GnuGk
 //
-// Copyright (c) 2000-2010, Jan Willamowius
+// Copyright (c) 2000-2011, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -28,7 +28,6 @@
 #include "clirw.h"
 #include "capctrl.h"
 #include "RasSrv.h"
-#include "RasTbl.h"
 #include "Toolkit.h"
 #include "gk_const.h"
 
@@ -70,10 +69,14 @@ NetworkAddress::NetworkAddress(
 {
 	// normalize the address
 	BYTE rawdata[16];
-	const unsigned sz = addr.GetSize();
-	for (unsigned i = 0; i < sz; i++)
-		rawdata[i] = addr[i] & nm[i];
-	m_address = PIPSocket::Address(sz, rawdata);
+	if (addr.GetSize() == nm.GetSize()) {
+		const unsigned sz = addr.GetSize();
+		for (unsigned i = 0; i < sz; i++)
+			rawdata[i] = addr[i] & nm[i];
+		m_address = PIPSocket::Address(sz, rawdata);
+	} else {
+		PTRACE(1, "Error: Non-matching network and netmask");
+	}
 }
 	
 NetworkAddress::NetworkAddress(
@@ -159,7 +162,7 @@ bool NetworkAddress::operator>>(const PIPSocket::Address &addr) const
 	return true;
 }
 
-int NetworkAddress::Compare(const NetworkAddress &addr) const
+int NetworkAddress::Compare(const NetworkAddress & addr) const
 {
 	int diff = m_address.GetSize() - addr.m_address.GetSize();
 	if (diff == 0) {
@@ -248,7 +251,7 @@ Toolkit::RouteTable::RouteEntry::RouteEntry(
 
 inline bool Toolkit::RouteTable::RouteEntry::Compare(const Address *ip) const
 {
-	return (*ip == destination) || ((*ip & net_mask) == network);
+	return (*ip == destination) || (((*ip & net_mask) == network) && (ip->GetVersion() == network.GetVersion()));
 }
 
 // class Toolkit::RouteTable
@@ -256,6 +259,9 @@ void Toolkit::RouteTable::InitTable()
 {
 	// workaround for OS that don't support GetRouteTable
 	PIPSocket::GetHostAddress(defAddr);
+#ifdef hasIPV6
+	PIPSocket::GetHostAddress(defAddrV6);
+#endif
 
 	ClearTable();
 	if (!CreateTable())
@@ -267,43 +273,88 @@ void Toolkit::RouteTable::InitTable()
 	// if we only have 1 Home IP, then thats also the default IP
 	if (home.size() == 1) {
 		defAddr = home[0];
+#ifdef hasIPV6
+		defAddrV6 = home[0];
+#endif
 	}
 	// Bind= always sets the default IP
 	PString bind = GkConfig()->GetString("Bind", "");
 	if (!bind.IsEmpty()) {
 		defAddr = bind;
+#ifdef hasIPV6
+		defAddrV6 = bind;
+#endif
 	}
 
 	// if we do not already have a valid entry, try and retrieve the default interface
-	if (defAddr.IsLoopback() || !defAddr.IsValid()) {
+	if ((defAddr.GetVersion() != 4) || defAddr.IsLoopback() || !defAddr.IsValid()) {
 		// Set default IP according to route table
-		PIPSocket::Address defGW;
-		PIPSocket::GetGatewayAddress(defGW);
-		if (defGW.AsString() == "0.0.0.0") {
+		defAddr = GetDefaultIP(4);
+		if ((defAddr.GetVersion() != 4) || defAddr.IsLoopback() || !defAddr.IsValid()) {
 			// no default gateway, use first interface as default
 			PIPSocket::GetNetworkInterface(defAddr);
-		} else {
-			// use the default gateway
-			defAddr = GetLocalAddress(defGW);
 		}
-    }
-	// if we have a list of Home IPs and the default address is not in it, use the first Home IP,
+	}
+#ifdef hasIPV6
+	// if we do not already have a valid entry, try and retrieve the default interface
+	if ((defAddrV6.GetVersion() != 6) || defAddrV6.IsLoopback() || !defAddrV6.IsValid()) {
+		// Set default IP according to route table
+		defAddrV6 = GetDefaultIP(6);
+		if ((defAddrV6.GetVersion() != 6) || defAddrV6.IsLoopback() || !defAddrV6.IsValid()) {
+			// no default gateway, use first interface as default
+			PIPSocket::GetNetworkInterface(defAddrV6);
+		}
+	}
+#endif
+
+	// if we have a list of Home IPs and the default address is not in it, use the first IPv4 Home IP,
 	// unless the default IP was explicitely specified in Bind=
 	if (bind.IsEmpty() &&
 		!home.empty() && (find(home.begin(), home.end(), defAddr) == home.end())) {
-		defAddr = home[0];
-	} 
+		for (unsigned i=0; i < home.size(); ++i) {
+			if (home[i].GetVersion() == 4) {
+				defAddr = home[i];
+				break;
+			}
+		}
+	}
+#ifdef hasIPV6
+	if (bind.IsEmpty() &&
+		!home.empty() && (find(home.begin(), home.end(), defAddrV6) == home.end())) {
+		for (unsigned i=0; i < home.size(); ++i) {
+			if (home[i].GetVersion() == 6) {
+				defAddrV6 = home[i];
+				break;
+			}
+		}
+	}
+#endif
 
-#if PTRACING
 	for (RouteEntry *entry = rtable_begin; entry != rtable_end; ++entry) {
-		PTRACE(2, "Network=" << entry->GetNetwork() << '/' << entry->GetNetMask() <<
-			  ", IP=" << entry->GetDestination());
+		PTRACE(2, "Network=" << NetworkAddress(entry->GetNetwork(), entry->GetNetMask()).AsString() <<
+				", IP=" << entry->GetDestination());
+	}
+#ifdef hasIPV6
+	if (Toolkit::Instance()->IsIPv6Enabled())
+		PTRACE(2, "Default IP IPv4=" << defAddr << " IPv6=" << defAddrV6);
+	else
+#endif
+	{
+		PTRACE(2, "Default IP=" << defAddr);
 	}
-	PTRACE(2, "Default IP=" << defAddr);
 	if (defAddr.IsLoopback()) {
 		PTRACE(1, "WARNING: Your default IP=" << defAddr << " is a loopback address. That probably won't work!");
 	}
-#endif
+}
+
+// get default route from route table, because GetGatewayAddress() is broken until PTLib 2.11.1
+PIPSocket::Address Toolkit::RouteTable::GetDefaultIP(unsigned version)
+{
+	for (RouteEntry *entry = rtable_begin; entry != rtable_end; ++entry) {
+		if ((entry->GetNetMask() == 0) && (entry->GetDestination().GetVersion() == version))
+			return GetLocalAddress(entry->GetDestination());
+	}
+	return Address(0);
 }
 
 void Toolkit::RouteTable::ClearTable()
@@ -322,6 +373,15 @@ void Toolkit::RouteTable::AddInternalNetwork(const NetworkAddress & network)
 		m_internalnetworks.push_back(network);
 }
 
+PIPSocket::Address Toolkit::RouteTable::GetLocalAddress(unsigned version) const
+{
+#ifdef hasIPV6
+	if (version == 6)
+		return defAddrV6;
+#endif
+	return defAddr;
+}
+
 PIPSocket::Address Toolkit::RouteTable::GetLocalAddress(const Address & addr) const
 {
 	// look through internal networks and make sure we don't return the external IP for them
@@ -330,37 +390,55 @@ PIPSocket::Address Toolkit::RouteTable::GetLocalAddress(const Address & addr) co
 			// check if internal network is in route table, but don't use the default route
 			RouteEntry *entry = find_if(rtable_begin, rtable_end,
 				bind2nd(mem_fun_ref(&RouteEntry::Compare), &addr));
-			if ((entry != rtable_end) && (entry->GetNetMask() != INADDR_ANY))
+			if ((entry != rtable_end) && (entry->GetNetMask() != INADDR_ANY)
+#ifdef hasIPV6
+				&& (entry->GetNetMask() != in6addr_any)
+#endif
+				) {
 				return entry->GetDestination();
-			else
+			}
+			else {
+#ifdef hasIPV6
+				if (addr.GetVersion() == 6)
+					return defAddrV6;
+#endif
 				return defAddr;
+			}
 		}
 	}
 
 	// if external IP is configured
-    if (!ExtIP) {
-      if (DynExtIP) {  // if dynamic resolve DNS entry
-        PIPSocket::Address extip;
-		H323TransportAddress ex = H323TransportAddress(ExtIP);
-		ex.GetIpAddress(extip);
-          if (extip.IsValid()) {
-              return extip;
-          } else {
-              PTRACE(2,"NAT\tERROR: ExtIP " << ExtIP << " unresolvable." );
-          }
-      } else {  // If valid IP then use the ExtIP value
-          PIPSocket::Address extip(ExtIP);
-          if (extip.IsValid()) {
-              return extip;
-          } else {
-              PTRACE(2,"NAT\tERROR: ExtIP " << ExtIP << " unuseable." );
-          }
-      }
+	if (!ExtIP.IsEmpty()) {
+		if (DynExtIP) {  // if dynamic resolve DNS entry
+			PIPSocket::Address extip;
+			H323TransportAddress ex = H323TransportAddress(ExtIP);
+			ex.GetIpAddress(extip);
+			if (extip.IsValid()) {
+				return extip;
+			} else {
+				PTRACE(2,"NAT\tERROR: ExtIP " << ExtIP << " unresolvable." );
+			}
+		} else {  // If valid IP then use the ExtIP value
+			PIPSocket::Address extip(ExtIP);
+			if (extip.IsValid()) {
+				return extip;
+			} else {
+				PTRACE(2,"NAT\tERROR: ExtIP " << ExtIP << " unuseable." );
+			}
+		}
 	}
 
 	RouteEntry *entry = find_if(rtable_begin, rtable_end,
 		bind2nd(mem_fun_ref(&RouteEntry::Compare), &addr));
-	return (entry != rtable_end) ? entry->GetDestination() : defAddr;
+	if (entry != rtable_end) {
+		return entry->GetDestination();
+	}
+#ifdef hasIPV6
+	if (addr.GetVersion() == 6) {
+		return defAddrV6;
+	}
+#endif
+	return defAddr;
 }
 
 bool Toolkit::RouteTable::CreateRouteTable(const PString & extroute)
@@ -377,16 +455,32 @@ bool Toolkit::RouteTable::CreateRouteTable(const PString & extroute)
 		PTRACE(1, "Error: Can't get route table");
 		return false;
 	}
+	// filter out route with destination localhost
+	// we can't use those for routing calls, unless the net itself is localhost
+	for(PINDEX i=0; i < r_table.GetSize(); ++i) {
+		if ((r_table[i].GetDestination().IsLoopback())
+			&& !r_table[i].GetNetwork().IsLoopback()) {
+				r_table.RemoveAt(i--);
+			}
+	}
+	// filter out IPv6 networks if IPv6 is not enabled
+	for(PINDEX i=0; i < r_table.GetSize(); ++i) {
+		if ((r_table[i].GetNetwork().GetVersion() == 6)
+			&& !Toolkit::Instance()->IsIPv6Enabled()) {
+				r_table.RemoveAt(i--);
+			}
+	}
 
 	if (/*!extroute &&*/ AsBool(GkConfig()->GetString(ProxySection, "Enable", "0"))) {
-	  for (PINDEX i = 0; i < r_table.GetSize(); ++i) {
-		if (r_table[i].GetNetwork().IsRFC1918() && r_table[i].GetNetMask().AsString() != "255.255.255.255") {
-		  PString intAddr = r_table[i].GetNetwork().AsString() + "/" + r_table[i].GetNetMask().AsString();
-		  m_internalnetworks.resize( m_internalnetworks.size() + 1);
-		  m_internalnetworks[m_internalnetworks.size() - 1] = NetworkAddress(intAddr);
-		  PTRACE(2, "Internal Network Detected " << m_internalnetworks.back().AsString()); 
-		} 
-	  }
+		for (PINDEX i = 0; i < r_table.GetSize(); ++i) {
+			if (r_table[i].GetNetwork().IsRFC1918()
+				&& (r_table[i].GetNetMask().AsString() != "255.255.255.255")
+				&& (r_table[i].GetNetMask().AsString() != "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff")) {
+				m_internalnetworks.resize( m_internalnetworks.size() + 1);
+				m_internalnetworks[m_internalnetworks.size() - 1] = NetworkAddress(r_table[i].GetNetwork(), r_table[i].GetNetMask());
+				PTRACE(2, "Internal Network Detected " << m_internalnetworks.back().AsString()); 
+			}
+		}
 	}
 
 	int i = (!extroute) ? r_table.GetSize()+1 : r_table.GetSize();
@@ -397,8 +491,7 @@ bool Toolkit::RouteTable::CreateRouteTable(const PString & extroute)
 			::new (rtable_end++) RouteEntry(extroute);
 		} else {
 			PIPSocket::RouteEntry & r_entry = r_table[r];
-			//if (r_entry.GetNetMask() != INADDR_ANY && r_entry.GetNetwork().GetVersion() == 4 && r_entry.GetDestination().GetVersion() == 4) {
-			if (r_entry.GetNetMask() != INADDR_ANY && r_entry.GetNetwork().GetVersion() != 6) {
+			if (!r_entry.GetNetMask().IsAny()) {
 				::new (rtable_end++) RouteEntry(r_entry, if_table);
 			}
 		}
@@ -433,13 +526,13 @@ bool Toolkit::VirtualRouteTable::CreateTable()
 		ExtIP = extip;
 		PString extroute;
 		if (!DynExtIP) 
-		  extroute = ext.AsString() + "/0";
+			extroute = ext.AsString() + "/0";
 
 		CreateRouteTable(extroute);
 		PTRACE(1,"External IP = " << ExtIP << " dynamic " << DynExtIP);
 		return true;
 	} else
-        DynExtIP = false;
+		DynExtIP = false;
 
 	return false;
 }
@@ -447,9 +540,9 @@ bool Toolkit::VirtualRouteTable::CreateTable()
 bool Toolkit::VirtualRouteTable::IsMasquerade(PIPSocket::Address & addr) const
 {
 	if (!ExtIP) {
-	  H323TransportAddress ex = H323TransportAddress(ExtIP);
-	  ex.GetIpAddress(addr);
-	  return true;
+		H323TransportAddress ex = H323TransportAddress(ExtIP);
+		ex.GetIpAddress(addr);
+		return true;
 	}
 
 	return false;
@@ -618,14 +711,14 @@ int Toolkit::ProxyCriterion::SelectRoutingMode(const Address & ip1, const Addres
 
 int Toolkit::ProxyCriterion::IsInternal(const Address & ip) const
 {
-   // Return the network Id. Addresses may be on different internal networks
+	// Return the network Id. Addresses may be on different internal networks
 	int retval = 0;
 	std::vector<NetworkAddress>::const_iterator i = m_internalnetworks.begin();
 	while (i != m_internalnetworks.end()) {
 		retval++;
 		if (ip << *i++)
 			return retval;
-    }
+	}
 	return 0;
 }
 
@@ -654,9 +747,9 @@ void Toolkit::RewriteData::AddSection(PConfig *config, const PString & section)
 		if ((n_size = rules.size()) > 0) {
 			// Add any existing rules to be resorted
 			if (m_size > 0) {
-             for (PINDEX j = 0; j < m_size; ++j) {
-				 rules[Key(j)] = Value(j);
-			 }
+				for (PINDEX j = 0; j < m_size; ++j) {
+					rules[Key(j)] = Value(j);
+				}
 			}
 			m_size = m_size + n_size;
 			// replace array constructor with explicit memory allocation
@@ -702,44 +795,34 @@ void Toolkit::RewriteTool::LoadConfig(PConfig *config)
 	m_Rewrite->AddSection(config,AliasRewriteSection);
 }
 
-bool Toolkit::IsNumeric(const PString & s) 
-{
-	 PINDEX j;
-     for (j = 0; j < s.GetLength(); ++j)
-	       if (!isdigit(static_cast<unsigned char>(s[j])))
-	   	         break;
-
-	 return (j >= s.GetLength());
-}
-
 bool Toolkit::RewriteTool::RewritePString(PString & s) const
 {
 	bool changed = false;
 
 	// If URL remove the domain if default domain
-	 PINDEX at = s.Find('@');
-	 if (at != P_MAX_INDEX) {
-		 PString num = s.Left(at);
-		 PString domain = s.Mid(at+1);
-		 PIPSocket::Address domIP(domain);
+	PINDEX at = s.Find('@');
+	if (at != P_MAX_INDEX) {
+		PString num = s.Left(at);
+		PString domain = s.Mid(at+1);
+		PIPSocket::Address domIP(domain);
 
-         // Check if we have a default domain and strip it
-		 for (PINDEX i=0; i < m_defaultDomain.GetSize(); i++) {
-			 if (domain == m_defaultDomain[i]) {
-			   PTRACE(2, "\tRewriteDomain: " << s << " to " << num);
-			   s = num;
-			   changed = true;
-			   break;
-			 }
-		 }
+		// Check if we have a default domain and strip it
+		for (PINDEX i=0; i < m_defaultDomain.GetSize(); i++) {
+			if (domain == m_defaultDomain[i]) {
+				PTRACE(2, "\tRewriteDomain: " << s << " to " << num);
+				s = num;
+				changed = true;
+				break;
+			}
+		}
 
-	     // Check that the domain is not a local IP address.
-		 if (!changed && domIP.IsValid() && Toolkit::Instance()->IsGKHome(domIP)) {
- 		   PTRACE(2, "\tRemoveDomain: " << domain << " to " << num);
-		   s = num;
-		   changed = true;
-		 }
-	 }
+		// Check that the domain is not a local IP address.
+		if (!changed && domIP.IsValid() && Toolkit::Instance()->IsGKHome(domIP)) {
+			PTRACE(2, "\tRemoveDomain: " << domain << " to " << num);
+			s = num;
+			changed = true;
+		}
+	}
 
 	// remove trailing character
 	if (s.GetLength() > 1 && s[s.GetLength() - 1] == m_TrailingChar) {
@@ -755,7 +838,7 @@ bool Toolkit::RewriteTool::RewritePString(PString & s) const
 		const char *prefix = m_Rewrite->Key(i);
 		if (prefix == s){
 			s = m_Rewrite->Value(i);
-            return true;
+			return true;
 		}
 		const int len = MatchPrefix(s, prefix);
 		// try a prefix match through all keys
@@ -848,15 +931,17 @@ void Toolkit::GWRewriteTool::PrintData()
 		return;
 	}
 
-	for (PINDEX i = 0; i < m_GWRewrite.GetSize(); ++i) {
-		// In
-		for (rule_iterator = m_GWRewrite.GetDataAt(i).m_entry_data.first.begin(); rule_iterator != m_GWRewrite.GetDataAt(i).m_entry_data.first.end(); ++rule_iterator) {
-			PTRACE(3, "GK\t" << m_GWRewrite.GetKeyAt(i) << " (in): " << (*rule_iterator).first << " = " << (*rule_iterator).second);
-		}
+	if (PTrace::CanTrace(3) && (m_GWRewrite.GetSize() < 100)) {
+		for (PINDEX i = 0; i < m_GWRewrite.GetSize(); ++i) {
+			// In
+			for (rule_iterator = m_GWRewrite.GetDataAt(i).m_entry_data.first.begin(); rule_iterator != m_GWRewrite.GetDataAt(i).m_entry_data.first.end(); ++rule_iterator) {
+				PTRACE(3, "GK\t" << m_GWRewrite.GetKeyAt(i) << " (in): " << (*rule_iterator).first << " = " << (*rule_iterator).second);
+			}
 
-		// Out
-		for (rule_iterator = m_GWRewrite.GetDataAt(i).m_entry_data.second.begin(); rule_iterator != m_GWRewrite.GetDataAt(i).m_entry_data.second.end(); ++rule_iterator) {
-			PTRACE(3, "GK\t" << m_GWRewrite.GetKeyAt(i) << " (out): " << (*rule_iterator).first << " = " << (*rule_iterator).second);
+			// Out
+			for (rule_iterator = m_GWRewrite.GetDataAt(i).m_entry_data.second.begin(); rule_iterator != m_GWRewrite.GetDataAt(i).m_entry_data.second.end(); ++rule_iterator) {
+				PTRACE(3, "GK\t" << m_GWRewrite.GetKeyAt(i) << " (out): " << (*rule_iterator).first << " = " << (*rule_iterator).second);
+			}
 		}
 	}
 	PTRACE(2, "GK\tLoaded " << m_GWRewrite.GetSize() << " GW entries with rewrite info");
@@ -1461,9 +1546,10 @@ PConfig* Toolkit::ReloadConfig()
 
 	// read the toolkit config values
 
-	// read the gatekeeper name from the config file, because it might be uased as a key into the SQL config
+	// read the gatekeeper name from the config file, because it might be used as a key into the SQL config
 	m_GKName = Config()->GetString("Name", "OpenH323GK");
 
+	PTrace::SetLevel(GkConfig()->GetInteger("TraceLevel", PTrace::GetLevel()));
 
 	// set the max size of an array in an ASN encoded message (eg. max length of alias list)
 	PINDEX maxArraySize = GkConfig()->GetInteger("MaxASNArraySize", 0);
@@ -1482,6 +1568,9 @@ PConfig* Toolkit::ReloadConfig()
 
 	// update the gatekeeper name, in case it was set in the SQL config
 	m_GKName = m_Config->GetString("Name", "OpenH323GK");
+#ifdef HAS_H235_MEDIA
+	m_H235HalfCallMediaEnabled	= m_Config->GetBoolean(RoutedSec, "EnableH235HalfCallMedia", 0);
+#endif
 #ifdef HAS_H46018
 	m_H46018Enabled	= m_Config->GetBoolean(RoutedSec, "EnableH46018", 0);
 #endif
@@ -1498,7 +1587,7 @@ PConfig* Toolkit::ReloadConfig()
 		GKHome = "";
 	}
 	if (m_GKHome.empty() || !GKHome)
-		SetGKHome(GKHome.Tokenise(",:;", false));
+		SetGKHome(GKHome.Tokenise(",;", false));
 	
 	m_RouteTable.InitTable();
 	m_VirtualRouteTable.InitTable();
@@ -1533,7 +1622,24 @@ PConfig* Toolkit::ReloadConfig()
 
 	ParseTranslationMap(m_receivedCauseMap, m_Config->GetString(RoutedSec, "TranslateReceivedQ931Cause", ""));
 	ParseTranslationMap(m_sentCauseMap, m_Config->GetString(RoutedSec, "TranslateSentQ931Cause", ""));
-	
+
+	// read [PortNotifications]
+	m_portOpenNotifications[RASPort] = m_Config->GetString("PortNotifications", "RASPortOpen", "");
+	m_portOpenNotifications[Q931Port] = m_Config->GetString("PortNotifications", "Q931PortOpen", "");
+	m_portOpenNotifications[H245Port] = m_Config->GetString("PortNotifications", "H245PortOpen", "");
+	m_portOpenNotifications[RTPPort] = m_Config->GetString("PortNotifications", "RTPPortOpen", "");
+	m_portOpenNotifications[T120Port] = m_Config->GetString("PortNotifications", "T120PortOpen", "");
+	m_portOpenNotifications[StatusPort] = m_Config->GetString("PortNotifications", "StatusPortOpen", "");
+	m_portOpenNotifications[RadiusPort] = m_Config->GetString("PortNotifications", "RadiusPortOpen", "");
+
+	m_portCloseNotifications[RASPort] = m_Config->GetString("PortNotifications", "RASPortClose", "");
+	m_portCloseNotifications[Q931Port] = m_Config->GetString("PortNotifications", "Q931PortClose", "");
+	m_portCloseNotifications[H245Port] = m_Config->GetString("PortNotifications", "H245PortClose", "");
+	m_portCloseNotifications[RTPPort] = m_Config->GetString("PortNotifications", "RTPPortClose", "");
+	m_portCloseNotifications[T120Port] = m_Config->GetString("PortNotifications", "T120PortClose", "");
+	m_portCloseNotifications[StatusPort] = m_Config->GetString("PortNotifications", "StatusPortClose", "");
+	m_portCloseNotifications[RadiusPort] = m_Config->GetString("PortNotifications", "RadiusPortClose", "");
+
 	return m_Config;
 }
 
@@ -1655,8 +1761,7 @@ bool Toolkit::AssignedAliases::LoadSQL(PConfig * cfg)
 
 bool Toolkit::AssignedAliases::DatabaseLookup(
 		const PString & alias,
-		PStringArray & newAliases	
-        )
+		PStringArray & newAliases)
 {
 	if (!m_sqlactive)
 		return false;
@@ -1677,7 +1782,7 @@ bool Toolkit::AssignedAliases::DatabaseLookup(
 		delete result;
 		return false;
 	}
-	
+
 	bool success = false;
 
 	if (result->GetNumRows() < 1)
@@ -1709,19 +1814,19 @@ void Toolkit::AssignedAliases::LoadConfig(PConfig * m_config)
 	gkAssignedAliases.clear();
 
 #if HAS_DATABASE
-  if (LoadSQL(m_config)) {
-	  m_sqlactive = true;
-  } else
+	if (LoadSQL(m_config)) {
+		m_sqlactive = true;
+	} else
 #endif
-  {
-   	const PStringToString kv = m_config->GetAllKeyValues(AssignedAliasSection);
-	for (PINDEX i=0; i < kv.GetSize(); i++) {
-		PString data = kv.GetDataAt(i);
-		PStringArray datalines = data.Tokenise(" ,;\t");
-		for (PINDEX j=0; j < datalines.GetSize(); j++)
-		   gkAssignedAliases.push_back(std::pair<PString, PString>(kv.GetKeyAt(i),datalines[j]));
+	{
+		const PStringToString kv = m_config->GetAllKeyValues(AssignedAliasSection);
+		for (PINDEX i=0; i < kv.GetSize(); i++) {
+			PString data = kv.GetDataAt(i);
+			PStringArray datalines = data.Tokenise(" ,;\t");
+			for (PINDEX j=0; j < datalines.GetSize(); j++)
+				gkAssignedAliases.push_back(std::pair<PString, PString>(kv.GetKeyAt(i),datalines[j]));
+		}
 	}
-  }
 }
 
 #ifdef H323_H350
@@ -1730,7 +1835,9 @@ bool Toolkit::CreateH350Session(H350_Session * session)
 {
 	PString ldap = GkConfig()->GetString(H350Section, "ServerName", "127.0.0.1");
 	PString port = GkConfig()->GetString(H350Section, "ServerPort", "389");
-	PString server = ldap + ":" + port;
+	PString server = ldap + ":" + port;	// IPv4
+	if (IsIPv6Address(ldap))
+		server = "[" + ldap + "]:" + port;	// IPv6
 
 	PString user = GkConfig()->GetString(H350Section, "BindUserDN", "");
 	PString password = Toolkit::Instance()->ReadPassword(H350Section, "BindUserPW");
@@ -1745,7 +1852,7 @@ bool Toolkit::CreateH350Session(H350_Session * session)
 	bool startTLS = Toolkit::AsBool(GkConfig()->GetString(H350Section, "StartTLS", "0"));
 
 	if (!session->Open(server)) {
-		PTRACE(1,"H350\tCannot locate H.350 Server");
+		PTRACE(1,"H350\tCannot locate H.350 Server " << server);
 		return false;
 	}
 	if (startTLS) {
@@ -1794,16 +1901,16 @@ bool Toolkit::AssignedAliases::QueryH350Directory(const PString & alias, PString
 
 	H350_Session session;
 	if (!Toolkit::Instance()->CreateH350Session(&session)) {
-	   PTRACE(1, "H350\tAssigned Alias: Could not connect to directory server");
-	   return false;
+		PTRACE(1, "H350\tAssigned Alias: Could not connect to directory server");
+		return false;
 	}
 
 	H350_Session::LDAP_RecordList rec;
 	int count = session.Search(search,filter,rec);
 	if (count <= 0) {
-	   PTRACE(4, "H350\tAssigned Alias: No Record Found");
-	   session.Close();
-	   return false;
+		PTRACE(4, "H350\tAssigned Alias: No Record Found");
+		session.Close();
+		return false;
 	}
 
 	// locate the record
@@ -1856,43 +1963,41 @@ bool Toolkit::AssignedAliases::QueryAssignedAliases(const PString & alias, PStri
 
 bool Toolkit::AssignedAliases::GetAliases(const H225_ArrayOf_AliasAddress & alias, H225_ArrayOf_AliasAddress & aliaslist)
 {
-
 	if (alias.GetSize() == 0)
-		    return false;
+		return false;
 
-    PStringArray newaliases;
+	PStringArray newaliases;
 	bool found = false;
 	for (PINDEX h=0; h < alias.GetSize(); h++) {
-	  if (QueryAssignedAliases(H323GetAliasAddressString(alias[h]), newaliases))
-		    found = true;
+		if (QueryAssignedAliases(H323GetAliasAddressString(alias[h]), newaliases))
+			found = true;
 	}
 
 	if (!found) {
 		for (PINDEX i=0; i < alias.GetSize(); i++) {
-		  PString search = H323GetAliasAddressString(alias[i]);
-
-		  for (unsigned j=0; j < gkAssignedAliases.size(); j++) {
-             PTRACE(5,"Alias\tCompare " << gkAssignedAliases[j].first << " to " << search);
-			  if (gkAssignedAliases[j].first == search) {
-				   newaliases.AppendString(gkAssignedAliases[j].second);
-				   if (!found) found = true;
-			  }
-		  }
+			PString search = H323GetAliasAddressString(alias[i]);
+			for (unsigned j=0; j < gkAssignedAliases.size(); j++) {
+				PTRACE(5,"Alias\tCompare " << gkAssignedAliases[j].first << " to " << search);
+				if (gkAssignedAliases[j].first == search) {
+					newaliases.AppendString(gkAssignedAliases[j].second);
+					if (!found) found = true;
+				}
+			}
 		}
 	}
 
 	// Create the Assigned Alias List
 	if (found) {
-	  // add existing items to the end of the list
+		// add existing items to the end of the list
 		if (aliaslist.GetSize() > 0) {
 			for (PINDEX l=0; l < aliaslist.GetSize(); l++) {
-			  PString a = H323GetAliasAddressString(aliaslist[l]);
-			    bool located = false;
+				PString a = H323GetAliasAddressString(aliaslist[l]);
+				bool located = false;
 			    for (PINDEX m=0; m < newaliases.GetSize(); m++) {
-                       if (newaliases[m] == a) located = true;
+					if (newaliases[m] == a) located = true;
 				}
-			   if (!located)
-                  newaliases.AppendString(a);
+				if (!located)
+					newaliases.AppendString(a);
 			}
 		}
 
@@ -1925,7 +2030,7 @@ bool Toolkit::AssignedGatekeepers::LoadSQL(PConfig * cfg)
 	delete m_sqlConn;
 	PString authName = "AssignedGatekeepers::SQL";
 
-   if (cfg->GetSections().GetStringsIndex(authName) == P_MAX_INDEX)
+	if (cfg->GetSections().GetStringsIndex(authName) == P_MAX_INDEX)
 		return false;
 
 	const PString driverName = cfg->GetString(authName, "Driver", "");
@@ -2006,39 +2111,39 @@ bool Toolkit::AssignedGatekeepers::DatabaseLookup(
 				PTRACE(1, "AssignSQL\tQuery Invalid value found.");
 				continue;
 			}
-		    if (!success) success = true;
+			if (!success) success = true;
 		    PTRACE(5, "AssignSQL\tQuery result: " << retval[0]);
-		    newGks.AppendString(retval[0]);
+			newGks.AppendString(retval[0]);
 		}
 	}
 	delete result;
 
-   return success;
+	return success;
 }
-#endif   // HAS_DATABASE
+#endif	// HAS_DATABASE
 
 void Toolkit::AssignedGatekeepers::LoadConfig(PConfig * m_config)
 {
 	assignedGKList.clear();
 
 #if HAS_DATABASE
-  if (LoadSQL(m_config)) {
-	  m_sqlactive = true;
-  } else
+	if (LoadSQL(m_config)) {
+		m_sqlactive = true;
+	} else
 #endif
-  {
-	const PStringToString kv = m_config->GetAllKeyValues(AssignedGatekeeperSection);
-	for (PINDEX i=0; i < kv.GetSize(); i++)
-		   assignedGKList.push_back(std::pair<PString, PString>(kv.GetKeyAt(i),kv.GetDataAt(i)));
-  }
+	{
+		const PStringToString kv = m_config->GetAllKeyValues(AssignedGatekeeperSection);
+		for (PINDEX i=0; i < kv.GetSize(); i++)
+			assignedGKList.push_back(std::pair<PString, PString>(kv.GetKeyAt(i),kv.GetDataAt(i)));
+	}
 }
 
 #ifdef H323_H350
 bool Toolkit::AssignedGatekeepers::QueryH350Directory(const PString & alias, const PIPSocket::Address & ip, PStringArray & addresses)
 {
 	// support gatekeeper discovery
-    if (!Toolkit::AsBool(GkConfig()->GetString(H350Section, "GatekeeperDiscovery", "0")))
-	   return false;
+	if (!Toolkit::AsBool(GkConfig()->GetString(H350Section, "GatekeeperDiscovery", "0")))
+		return false;
 
 	// search the directory
 	PString search = GkConfig()->GetString(H350Section, "SearchBaseDN", "");
@@ -2048,24 +2153,24 @@ bool Toolkit::AssignedGatekeepers::QueryH350Directory(const PString & alias, con
 
 	PString filter;
 	switch (aliasaddress.GetTag()) {
-	  case H225_AliasAddress::e_dialedDigits:
+		case H225_AliasAddress::e_dialedDigits:
             filter = "h323IdentitydialedDigits=" + alias;
-            break;
-	  case H225_AliasAddress::e_h323_ID:
+			break;
+		case H225_AliasAddress::e_h323_ID:
             filter = "h323Identityh323-ID=" + alias;
-            break;
-	  case H225_AliasAddress::e_url_ID:
+			break;
+		case H225_AliasAddress::e_url_ID:
 		    filter = "h323IdentityURL-ID=" + alias;
-		    break;
-	  default:
+			break;
+		default:
 			PTRACE(4, "H350\tAssigned GK: unhandled alias type " << aliasaddress.GetTagName());
 			return false;
 	}
 
 	H350_Session session;
 	if (!Toolkit::Instance()->CreateH350Session(&session)) {
-	   PTRACE(1, "H350\tAssigned GK: Could not connect to Server.");
-	   return false;
+		PTRACE(1, "H350\tAssigned GK: Could not connect to Server.");
+		return false;
 	}
 
 	H350_Session::LDAP_RecordList rec;
@@ -2111,11 +2216,11 @@ bool Toolkit::AssignedGatekeepers::QueryAssignedGK(const PString & alias, const
 #ifdef hasSRV
 static PString DNStoIP(const PString & dns)
 {
-   H323TransportAddress iface(dns);
-   PIPSocket::Address ip;
-   WORD port = GK_DEF_UNICAST_RAS_PORT;
-   iface.GetIpAndPort(ip, port);
-   return PString(ip + ":" + port);
+	H323TransportAddress iface(dns);
+	PIPSocket::Address ip;
+	WORD port = GK_DEF_UNICAST_RAS_PORT;
+	iface.GetIpAndPort(ip, port);
+	return AsString(ip, port);
 }
 #endif
 
@@ -2126,28 +2231,28 @@ bool Toolkit::AssignedGatekeepers::GetAssignedGK(const PString & alias, const PI
 	found = QueryAssignedGK(alias, ip, assignedGK);
 
 	if (!found) {
-		  for (unsigned j=0; j < assignedGKList.size(); j++) {
-		    PString match = assignedGKList[j].first.Trim();
+		for (unsigned j=0; j < assignedGKList.size(); j++) {
+			PString match = assignedGKList[j].first.Trim();
 			if (match.Left(1) != "^") {
-			  // prefix match
-			  if (MatchPrefix(alias,assignedGKList[j].first)) {
-				   assignedGK.AppendString(assignedGKList[j].second);
-				   if (!found) found = true;
-			  }
+				// prefix match
+				if (MatchPrefix(alias,assignedGKList[j].first)) {
+					assignedGK.AppendString(assignedGKList[j].second);
+					if (!found) found = true;
+				}
 			} else {
-	          // regex match for IP address
-			  if (MatchRegex(ip.AsString(), match)) {
-				   assignedGK.AppendString(assignedGKList[j].second);
-				   if (!found) found = true;
-		      }
+				// regex match for IP address
+				if (MatchRegex(ip.AsString(), match)) {
+					assignedGK.AppendString(assignedGKList[j].second);
+					if (!found) found = true;
+				}
 			}
-		  }
+		}
 	}
 
 	if (found) {
-      PStringArray ipaddresses;
+		PStringArray ipaddresses;
 		for (PINDEX k=0; k < assignedGK.GetSize(); k++) {
-           PString number = assignedGK[k];
+			PString number = assignedGK[k];
 
 			if (IsIPAddress(number))
 				ipaddresses.AppendString(number);
@@ -2173,17 +2278,15 @@ bool Toolkit::AssignedGatekeepers::GetAssignedGK(const PString & alias, const PI
 		}
 
 		for (PINDEX k = 0; k < ipaddresses.GetSize(); k++) {
-           PString num = ipaddresses[k];
-           WORD port = GK_DEF_UNICAST_RAS_PORT;
-		   PStringArray tokens = num.Tokenise(":", FALSE);
-		   if (tokens.GetSize() == 2)
-               port = (WORD)tokens[1].AsUnsigned();
+			PString num = ipaddresses[k];
+			PStringArray tokens = SplitIPAndPort(num, GK_DEF_UNICAST_RAS_PORT);
+			WORD port = (WORD)tokens[1].AsUnsigned();
 
-		   H225_AlternateGK * alt = new H225_AlternateGK;
-		     alt->m_rasAddress = SocketToH225TransportAddr(PIPSocket::Address(tokens[0]),port);
-		     alt->m_needToRegister = true;
-		     alt->m_priority = k;
-		   gklist.Append(alt);
+			H225_AlternateGK * alt = new H225_AlternateGK;
+			alt->m_rasAddress = SocketToH225TransportAddr(PIPSocket::Address(tokens[0]),port);
+			alt->m_needToRegister = true;
+			alt->m_priority = k;
+			gklist.Append(alt);
 		}
 	}
 		
@@ -2208,7 +2311,7 @@ void Toolkit::QoSMonitor::LoadConfig(PConfig * cfg)
 	delete m_sqlConn;
 	PString authName = "GkQoSMonitor::SQL";
 
-   if (cfg->GetSections().GetStringsIndex(authName) == P_MAX_INDEX)
+	if (cfg->GetSections().GetStringsIndex(authName) == P_MAX_INDEX)
 		return;
 
 	const PString driverName = cfg->GetString(authName, "Driver", "");
@@ -2286,8 +2389,8 @@ bool Toolkit::QoSMonitor::PostRecord(const std::map<PString, PString>& params)
 bool Toolkit::RewriteE164(H225_AliasAddress & alias)
 {
 	if ((alias.GetTag() != H225_AliasAddress::e_dialedDigits) &&
-         (alias.GetTag() != H225_AliasAddress::e_h323_ID) &&
-		 (alias.GetTag() != H225_AliasAddress::e_url_ID)) {
+		(alias.GetTag() != H225_AliasAddress::e_h323_ID) &&
+		(alias.GetTag() != H225_AliasAddress::e_url_ID)) {
 		if (alias.GetTag() != H225_AliasAddress::e_partyNumber)
 			return false;
 		H225_PartyNumber & partyNumber = alias;
@@ -2301,7 +2404,7 @@ bool Toolkit::RewriteE164(H225_AliasAddress & alias)
 	if (changed) {
 		if (E164.Find("@") != P_MAX_INDEX)
 			H323SetAliasAddress(E164, alias,H225_AliasAddress::e_url_ID);
-		else if (IsNumeric(E164))
+		else if (IsValidE164(E164))
 			H323SetAliasAddress(E164, alias, H225_AliasAddress::e_dialedDigits);
 		else if (alias.GetTag() != H225_AliasAddress::e_partyNumber)
 			H323SetAliasAddress(E164, alias, H225_AliasAddress::e_h323_ID);
@@ -2372,17 +2475,17 @@ bool Toolkit::GWRewriteE164(const PString & gw, bool direction, H225_ArrayOf_Ali
 
 bool Toolkit::isBehindNAT(PIPSocket::Address & externalIP) const
 {
-   return (m_VirtualRouteTable.IsMasquerade(externalIP));
+	return (m_VirtualRouteTable.IsMasquerade(externalIP));
 }
 
 #ifdef HAS_H46023
 bool Toolkit::IsH46023Enabled() const
 {
-	return (m_H46023Enabled && (             // is enabled and
+	return (m_H46023Enabled && (		// is enabled and
 #ifdef HAS_H46018
-		   m_H46018Enabled ||              // used with H.460.18 or
+			m_H46018Enabled ||			// used with H.460.18 or
 #endif
-		   m_Config->GetBoolean(RoutedSec, "SupportCallingNATedEndpoints",1))); // GnuGk Native NAT Support
+			m_Config->GetBoolean(RoutedSec, "SupportCallingNATedEndpoints",1))); // GnuGk Native NAT Support
 }
 
 void Toolkit::LoadH46023STUN()
@@ -2393,17 +2496,17 @@ void Toolkit::LoadH46023STUN()
 	PStringArray stunlist = stun.Tokenise(",");
 
 	for (PINDEX i = 0; i < stunlist.GetSize(); i++) {
-	    PIPSocket::Address ip; WORD port;
+		PIPSocket::Address ip; WORD port;
 		GetTransportAddress(stunlist[i], GK_DEF_STUN_PORT, ip, port);
 		if (ip.IsValid()) {
-		     int intID = m_ProxyCriterion.IsInternal(ip);
-			 std::map<int,H323TransportAddress>::const_iterator inf = m_H46023STUN.find(intID);
-			 if (inf == m_H46023STUN.end()) {
-				 if (intID > 0) {
-				    PTRACE(4,"Std23\tSTUN Internal " << ip << " IF " << intID-1);
-				 } else {
+			int intID = m_ProxyCriterion.IsInternal(ip);
+			std::map<int,H323TransportAddress>::const_iterator inf = m_H46023STUN.find(intID);
+			if (inf == m_H46023STUN.end()) {
+				if (intID > 0) {
+					PTRACE(4,"Std23\tSTUN Internal " << ip << " IF " << intID-1);
+				} else {
 					PTRACE(4,"Std23\tSTUN Public Server " << stunlist[i] << " (" << ip << ")");
-				 }
+				}
 				m_H46023STUN.insert(pair<int, H323TransportAddress>(intID, H323TransportAddress(ip,port)));
 			 }
 		}
@@ -2440,11 +2543,59 @@ GkPresence & Toolkit::GetPresenceHandler()
 }
 #endif
 
-std::vector<NetworkAddress> Toolkit::GetInternalNetworks() {
-
-    return !GkConfig()->GetString("ExternalIP", "").IsEmpty() ? m_VirtualRouteTable.GetInternalNetworks() : m_RouteTable.GetInternalNetworks();
+std::vector<NetworkAddress> Toolkit::GetInternalNetworks()
+{
+	return !GkConfig()->GetString("ExternalIP", "").IsEmpty() ? m_VirtualRouteTable.GetInternalNetworks() : m_RouteTable.GetInternalNetworks();
 }
 
+bool Toolkit::IsIPv6Enabled() const
+{
+#ifdef hasIPV6
+	return AsBool(GkConfig()->GetString("EnableIPv6", 0));
+#else
+	return false;
+#endif
+}
+
+void Toolkit::PortNotification(PortType type, PortAction action, const PString & protocol,
+								const PIPSocket::Address & addr, WORD port,
+								const H225_CallIdentifier & callID)
+{
+	PTRACE(5, "Port Notification " << ((action == PortOpen) ? "OPEN " : "CLOSE ") << type << " " << protocol << " " << ::AsString(addr, port));
+
+	// book keeping for status port command
+	if (callID != H225_CallIdentifier(0)) {
+		callptr call = CallTable::Instance()->FindCallRec(callID);
+		if (call) {
+			if (action == PortOpen)
+				call->AddDynamicPort(DynamicPort(type, addr, port));
+			else
+				call->RemoveDynamicPort(DynamicPort(type, addr, port));
+		}
+	}
+
+	// execute notification command
+	PString cmd;
+	if (action == PortOpen) {
+		cmd = m_portOpenNotifications[type];
+	} else if (action == PortClose) {
+		cmd = m_portCloseNotifications[type];
+	}
+
+	if (cmd.IsEmpty())
+		return;
+
+	// set port arguments
+	cmd.Replace("%p", protocol);
+	cmd.Replace("%n", PString(port));
+	cmd.Replace("%i", ::AsString(addr));
+
+	if(system(cmd) == -1) {
+		PTRACE(1, "Error executing: " << cmd);
+	}
+}
+
+
 PString Toolkit::GetGKHome(vector<PIPSocket::Address> & GKHome) const
 {
 	GKHome = m_GKHome;
@@ -2480,12 +2631,30 @@ void Toolkit::SetGKHome(const PStringArray & home)
 		}
 
 		// if no home interfaces specified, set all IPs from interface table
-		// except INADDR_ANY and any non IPv4 adresses
+		// except INADDR_ANY
 		if (m_GKHome.empty()) {
 			for (PINDEX n = 0; n < it.GetSize(); ++n) {
-				const PIPSocket::Address addr = it[n].GetAddress();
-				if (addr != INADDR_ANY && addr.GetVersion() == 4) {
-					m_GKHome.push_back(addr);
+				m_GKHome.push_back(it[n].GetAddress());
+			}
+		}
+		// remove INADDR_ANY
+		for (size_t n = 0; n < m_GKHome.size(); ++n) {
+			if ((m_GKHome[n] == INADDR_ANY)
+#ifdef hasIPV6
+				|| m_GKHome[n].IsLinkLocal()
+				|| ((m_GKHome[n].GetVersion() == 6) && m_GKHome[n].IsLoopback())
+#endif
+				) {
+				m_GKHome.erase(m_GKHome.begin() + n);
+				--n;	// re-test the new element on position n
+			}
+		}
+		// if IPv6 is not enabled, remove _all_ IPv6 addresses
+		if (!IsIPv6Enabled()) {
+			for (size_t n = 0; n < m_GKHome.size(); ++n) {
+				if (m_GKHome[n].GetVersion() == 6) {
+					m_GKHome.erase(m_GKHome.begin() + n);
+					--n;	// re-test the new element on position n
 				}
 			}
 		}
@@ -2502,7 +2671,7 @@ void Toolkit::SetGKHome(const PStringArray & home)
 			sortedHomes.push_back(m_GKHome[j]);
 		} else {
 			sortedHomes.push_front(m_GKHome[j]);
-		}		
+		}
 	}
 	m_GKHome.assign(sortedHomes.begin(), sortedHomes.end());
 }
@@ -2572,7 +2741,7 @@ void Toolkit::GetNetworkFromString(
 	)
 {
 	if (s *= "ALL") {
-		network = netmask = INADDR_ANY;
+		network = netmask = GNUGK_INADDR_ANY;
 		return;
 	}
 
@@ -2645,7 +2814,7 @@ PString Toolkit::AsString(
 	if (localtime_r(&t, tmptr) != tmptr) {
 #else
 	tmptr = localtime(&t);
-	if (tmptr  == NULL) {
+	if (tmptr == NULL) {
 #endif
 		PTRACE(0, "TOOLKIT\tCould not apply timestamp formatting - using default");
 		return tm.AsString( "hh:mm:ss.uuu z www MMM d yyyy" );
@@ -2784,9 +2953,9 @@ unsigned Toolkit::TranslateSentCause(unsigned cause) const
 #ifdef OpenH323Factory
 PStringList Toolkit::GetAuthenticatorList()
 {
-  PString auth = GkConfig()->GetString("Gatekeeper::Main", "Authenticators", "");
-  PStringArray authlist(auth.Tokenise(" ,;\t"));
+	PString auth = GkConfig()->GetString("Gatekeeper::Main", "Authenticators", "");
+	PStringArray authlist(auth.Tokenise(" ,;\t"));
 
-  return authlist;
+	return authlist;
 }
 #endif
