@@ -2,7 +2,7 @@
 //
 // Toolkit class for the GnuGk
 //
-// Copyright (c) 2000-2010, Jan Willamowius
+// Copyright (c) 2000-2011, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef TOOLKIT_H
-#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.78 2011/06/15 20:24:50 willamowius Exp $"
+#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.90 2011/12/21 17:02:07 willamowius Exp $"
 
 #include <vector>
 #include <map>
@@ -20,6 +20,7 @@
 #include <ptlib/sockets.h>
 #include "singleton.h"
 #include "config.h"
+#include "RasTbl.h"
 #ifdef HAS_H46023
 #include "transports.h"
 #endif
@@ -65,9 +66,7 @@ public:
 	    @return <0 if this address is lesser than #addr#, 0 if both are equal,
 	            >0 if this address is greater than #addr#.
 	*/
-	int Compare(
-		const NetworkAddress &addr
-		) const;
+	int Compare(const NetworkAddress & addr) const;
 
 	/// @return	A string representation of the address in the form A.B.C.D/netmasklen
 	PString AsString() const;
@@ -126,8 +125,9 @@ class Toolkit : public Singleton<Toolkit>
 	public:
 		RouteTable() : rtable_begin(0), rtable_end(0), DynExtIP(false), ExtIP("") { /* initialize later */ }
 		virtual ~RouteTable() { ClearTable(); }
-		Address GetLocalAddress() const { return defAddr; }
+		Address GetLocalAddress(unsigned version) const;
 		Address GetLocalAddress(const Address & dest) const;
+		Address GetDefaultIP(unsigned version);
 
 		void InitTable();
 		void ClearTable();
@@ -149,6 +149,9 @@ class Toolkit : public Singleton<Toolkit>
 
 		RouteEntry *rtable_begin, *rtable_end;
 		Address defAddr;
+#ifdef hasIPV6
+		Address defAddrV6;
+#endif
 
 		std::vector<NetworkAddress> m_internalnetworks;
     	bool DynExtIP;
@@ -328,7 +331,6 @@ class Toolkit : public Singleton<Toolkit>
 	bool RewriteE164(H225_ArrayOf_AliasAddress & aliases);
 
 	bool RewritePString(PString & s) { return m_Rewrite.RewritePString(s); }
-	bool IsNumeric(const PString & s);
 
 	// Class to allow correct use of STL inside PDictionary type
 	class GWRewriteEntry : public PObject {
@@ -364,6 +366,10 @@ class Toolkit : public Singleton<Toolkit>
 	void SetGKHome(const PStringArray &);
 	bool IsGKHome(const PIPSocket::Address & addr) const;
 
+#ifdef HAS_H235_MEDIA
+	bool IsH235HalfCallMediaEnabled() const { return m_H235HalfCallMediaEnabled; }
+#endif
+
 	bool isBehindNAT(PIPSocket::Address &) const;
 	std::vector<NetworkAddress> GetInternalNetworks();
 #ifdef HAS_H46018
@@ -379,6 +385,10 @@ class Toolkit : public Singleton<Toolkit>
 	bool IsH460PEnabled() const;
 	GkPresence & GetPresenceHandler();
 #endif
+	bool IsIPv6Enabled() const;
+	void PortNotification(PortType type, PortAction action, const PString & protocol,
+							const PIPSocket::Address & addr, WORD port,
+							const H225_CallIdentifier & callID = 0);
 
 	// accessors
 	/** Accessor and 'Factory' to the static Toolkit.
@@ -447,11 +457,11 @@ class Toolkit : public Singleton<Toolkit>
 	/** you may add more extension codes in descendant classes. This codes will not be transferred
 	 * or something it will be the return code of some methods for handling switches easy. */
 	enum {
-		iecUnknown     = -1,   /// internal extension code for an unknown triple(cntry,ext,manuf)
+		iecUnknown      = -1,  /// internal extension code for an unknown triple(cntry,ext,manuf)
 		iecFailoverRAS  = 1,   /// i.e.c. for "This RQ is a failover RQ" and must not be answerd.
-		iecNeighborId = 2,     /// i.e.c. for neighbor id sent inside LRQ/LCF nonStandardData parameter
+		iecNeighborId   = 2,   /// i.e.c. for neighbor id sent inside LRQ/LCF nonStandardData parameter
 		iecNATTraversal = 3,   /// i.e.c. for IP=/NAT= parameters sent inside RRQ/RCF nonStandardData
-		iecUserbase    = 1000 /// first guaranteed unused 'iec' by GnuGk Toolkit.
+		iecUserbase     = 1000 /// first guaranteed unused 'iec' by GnuGk Toolkit.
 	};
 	/** t35 extension or definitions as field for H225_NonStandardIdentifier */
 	enum T35CountryCodes {
@@ -583,7 +593,7 @@ protected:
 
 	RewriteTool m_Rewrite;
 	GWRewriteTool m_GWRewrite; // GW Based RewriteTool
-	AssignedAliases m_AssignedEPAliases;  // Assigned Aliases
+	AssignedAliases m_AssignedEPAliases;	// Assigned Aliases
 #ifdef h323v6
 	AssignedGatekeepers m_AssignedGKs;
 #endif
@@ -622,6 +632,9 @@ private:
 	/// global cause code translation
 	std::map<unsigned, unsigned> m_receivedCauseMap;
 	std::map<unsigned, unsigned> m_sentCauseMap;
+#ifdef HAS_H235_MEDIA
+	bool m_H235HalfCallMediaEnabled;
+#endif
 	// is H460.18 enabled ?
 #ifdef HAS_H46018
 	bool m_H46018Enabled;
@@ -635,7 +648,8 @@ private:
 #ifdef HAS_H460P
 	GkPresence m_presence;
 #endif
-
+	std::map<PortType, PString> m_portOpenNotifications;
+	std::map<PortType, PString> m_portCloseNotifications;
 };
 
 
