@@ -3,7 +3,7 @@
  *
  * Module for CLI/ANI manipulation.
  *
- * $Id: clirw.cxx,v 1.24 2011/03/23 12:13:40 willamowius Exp $
+ * $Id: clirw.cxx,v 1.25 2011/11/28 12:33:41 willamowius Exp $
  *
  * Copyright (c) 2005, Michal Zygmuntowicz
  * Copyright (c) 2007-2010, Jan Willamowius
@@ -456,7 +456,6 @@ CLIRewrite::CLIRewrite()
 	}
 	
 	PTRACE(5, "CLIRW\t" << inboundRules << " inbound rules loaded");
-#if PTRACING
 	if (PTrace::CanTrace(6)) {
 		ostream &strm = PTrace::Begin(6, __FILE__, __LINE__);
 		strm << "Inbound CLI rewrite rules:" << endl;
@@ -467,10 +466,8 @@ CLIRewrite::CLIRewrite()
 		}
 		PTrace::End(strm);
 	}
-#endif
 
 	PTRACE(5, "CLIRW\t" << outboundRules << " outbound rules loaded");
-#if PTRACING
 	if (PTrace::CanTrace(6)) {
 		ostream &strm = PTrace::Begin(6, __FILE__, __LINE__);
 		strm << "Outbound CLI rewrite rules:" << endl;
@@ -483,7 +480,6 @@ CLIRewrite::CLIRewrite()
 			}
 		PTrace::End(strm);
 	}
-#endif
 
 	m_processSourceAddress = Toolkit::AsBool(
 		cfg->GetString(CLIRewriteSection, ProcessSourceAddress, "1")
