@@ -2,7 +2,7 @@
 //
 // config.h configuration header
 //
-// Copyright (c) 2008-2010, Jan Willamowius
+// Copyright (c) 2008-2011, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,8 +13,10 @@
 
 
 #ifndef CONFIG_H
-#define CONFIG_H "@(#) $Id: config.h,v 1.7 2011/03/06 14:46:01 willamowius Exp $"
+#define CONFIG_H "@(#) $Id: config.h,v 1.9 2011/09/23 09:28:52 willamowius Exp $"
 
+#include <ptlib.h>
+#include <ptlib/ipsock.h>
 #include "pwlib_compat.h"
 
 #ifdef _WIN32
@@ -40,5 +42,7 @@
 #define		HAS_DATABASE 1
 #endif
 
+extern PIPSocket::Address GNUGK_INADDR_ANY;
+
 #endif // CONFIG_H
 
