@@ -38,13 +38,10 @@ using std::vector;
 
 PCREATE_PROCESS(Gatekeeper)
 
-/*
- * many things here should be members of Gatekeeper
- */
+PIPSocket::Address GNUGK_INADDR_ANY(INADDR_ANY);
 
 PReadWriteMutex ConfigReloadMutex;
 
-#if PTRACING
 PTextFile* Gatekeeper::m_logFile = NULL;
 PFilePath Gatekeeper::m_logFilename;
 PMutex Gatekeeper::m_logFileMutex;
@@ -53,7 +50,6 @@ int Gatekeeper::m_rotateMinute = 0;
 int Gatekeeper::m_rotateHour = 0;
 int Gatekeeper::m_rotateDay = 0;
 GkTimer* Gatekeeper::m_rotateTimer = GkTimerManager::INVALID_HANDLE;
-#endif // PTRACING
 
 PSemaphore ShutdownMutex(1,1);
 
@@ -67,25 +63,26 @@ PString pidfile("/var/run/gnugk.pid");
 
 void ShutdownHandler()
 {
-#if PTRACING
 	Gatekeeper::EnableLogFileRotation(false);
-#endif
 	// delete singleton objects
 	PTRACE(3, "GK\tDeleting global reference tables");
 
 	Job::StopAll();
-	delete CapacityControl::Instance();
-	delete CallTable::Instance();
-	delete RegistrationTable::Instance();
-	delete RasServer::Instance();
+	if (CapacityControl::InstanceExists())
+		delete CapacityControl::Instance();
+	if (CallTable::InstanceExists())
+		delete CallTable::Instance();
+	if (RegistrationTable::InstanceExists())
+		delete RegistrationTable::Instance();
+	if (RasServer::InstanceExists())
+		delete RasServer::Instance();
 	if (MakeCallEndPoint::InstanceExists())
 		delete MakeCallEndPoint::Instance();
-	delete Toolkit::Instance();
+	if (Toolkit::InstanceExists())
+		delete Toolkit::Instance();
 	PTRACE(3, "GK\tdelete ok");
 
-#if PTRACING
 	Gatekeeper::CloseLogFile();
-#endif
 }
 
 static const char * KnowConfigEntries[][2] = {
@@ -173,11 +170,15 @@ static const char * KnowConfigEntries[][2] = {
 	{ "Gatekeeper::Auth", "SimplePasswordAuth" },
 	{ "Gatekeeper::Auth", "default" },
 	{ "Gatekeeper::Main", "AlternateGKs" },
+	{ "Gatekeeper::Main", "Authenticators" },
 	{ "Gatekeeper::Main", "Bind" },
 	{ "Gatekeeper::Main", "CompareAliasCase" },
 	{ "Gatekeeper::Main", "CompareAliasType" },
 	{ "Gatekeeper::Main", "DefaultDomain" },
 	{ "Gatekeeper::Main", "DisconnectCallsOnShutdown" },
+#ifdef hasIPV6
+	{ "Gatekeeper::Main", "EnableIPv6" },
+#endif
 	{ "Gatekeeper::Main", "EncryptAllPasswords" },
 	{ "Gatekeeper::Main", "EndpointIDSuffix" },
 	{ "Gatekeeper::Main", "EndpointSignalPort" },
@@ -189,6 +190,7 @@ static const char * KnowConfigEntries[][2] = {
 	{ "Gatekeeper::Main", "Home" },
 	{ "Gatekeeper::Main", "ListenQueueLength" },
 	{ "Gatekeeper::Main", "MaxASNArraySize" },
+	{ "Gatekeeper::Main", "MaximumBandwidthPerCall" },
 	{ "Gatekeeper::Main", "MinimumBandwidthPerCall" },
 	{ "Gatekeeper::Main", "MulticastGroup" },
 	{ "Gatekeeper::Main", "MulticastPort" },
@@ -223,11 +225,13 @@ static const char * KnowConfigEntries[][2] = {
 	{ "GkPresence::SQL", "Driver" },
 	{ "GkPresence::SQL", "Host" },
 	{ "GkPresence::SQL", "Library" },
+	{ "GkPresence::SQL", "Password" },
 	{ "GkPresence::SQL", "MinPoolSize" },
 	{ "GkPresence::SQL", "QueryAdd" },
 	{ "GkPresence::SQL", "QueryDelete" },
 	{ "GkPresence::SQL", "QueryList" },
 	{ "GkPresence::SQL", "QueryUpdate" },
+	{ "GkPresence::SQL", "Username" },
 	{ "GkQoSMonitor", "CallEndOnly" },
 	{ "GkQoSMonitor", "DetailFile" },
 	{ "GkQoSMonitor", "Enable" },
@@ -239,10 +243,27 @@ static const char * KnowConfigEntries[][2] = {
 	{ "GkQoSMonitor::SQL", "Password" },
 	{ "GkQoSMonitor::SQL", "Query" },
 	{ "GkQoSMonitor::SQL", "Username" },
+	{ "GkStatus::Filtering", "Enable" },
+	{ "GkStatus::Filtering", "ExcludeFilter" },
+	{ "GkStatus::Filtering", "IncludeFilter" },
 	{ "LogFile", "Filename" },
 	{ "LogFile", "Rotate" },
 	{ "LogFile", "RotateDay" },
 	{ "LogFile", "RotateTime" },
+	{ "PortNotifications", "H245PortOpen" },
+	{ "PortNotifications", "H245PortClose" },
+	{ "PortNotifications", "Q931PortOpen" },
+	{ "PortNotifications", "Q931PortClose" },
+	{ "PortNotifications", "RadiusPortOpen" },
+	{ "PortNotifications", "RadiusPortClose" },
+	{ "PortNotifications", "RASPortOpen" },
+	{ "PortNotifications", "RASPortClose" },
+	{ "PortNotifications", "RTPPortOpen" },
+	{ "PortNotifications", "RTPPortClose" },
+	{ "PortNotifications", "StatusPortOpen" },
+	{ "PortNotifications", "StatusPortClose" },
+	{ "PortNotifications", "T120PortOpen" },
+	{ "PortNotifications", "T120PortClose" },
 	{ "Proxy", "DisableRTPQueueing" },
 	{ "Proxy", "Enable" },
 	{ "Proxy", "EnableRTCPStats" },
@@ -251,7 +272,12 @@ static const char * KnowConfigEntries[][2] = {
 	{ "Proxy", "ProxyAlways" },
 	{ "Proxy", "ProxyForNAT" },
 	{ "Proxy", "ProxyForSameNAT" },
+	{ "Proxy", "RTPMultiplexing" },
+	{ "Proxy", "RTPMultiplexPort" },
+	{ "Proxy", "RTCPMultiplexPort" },
 	{ "Proxy", "RTPPortRange" },
+	{ "Proxy", "RemoveMCInFastStartTransmitOffer" },
+	{ "Proxy", "SearchBothSidesOnCLC" },
 	{ "Proxy", "T120PortRange" },
 	{ "RadAcct", "AppendCiscoAttributes" },
 	{ "RadAcct", "DefaultAcctPort" },
@@ -322,16 +348,22 @@ static const char * KnowConfigEntries[][2] = {
 	{ "RoutedMode", "AcceptUnregisteredCalls" },
 	{ "RoutedMode", "ActivateFailover" },
 	{ "RoutedMode", "AlertingTimeout" },
+	{ "RoutedMode", "AlwaysRewriteSourceCallSignalAddress" },
+	{ "RoutedMode", "AutoProxyIPv4ToIPv6Calls" },
 	{ "RoutedMode", "CallSignalHandlerNumber" },
 	{ "RoutedMode", "CallSignalPort" },
+	{ "RoutedMode", "CalledTypeOfNumber" },
+	{ "RoutedMode", "CallingTypeOfNumber" },
 	{ "RoutedMode", "CpsCheckInterval" },
 	{ "RoutedMode", "CpsLimit" },
 	{ "RoutedMode", "DisableRetryChecks" },
 	{ "RoutedMode", "DropCallsByReleaseComplete" },
 	{ "RoutedMode", "ENUMservers" },
+	{ "RoutedMode", "EnableH235HalfCallMedia" },
 	{ "RoutedMode", "EnableH450.2" },
 	{ "RoutedMode", "EnableH46018" },
 	{ "RoutedMode", "EnableH46023" },
+	{ "RoutedMode", "EnableH460P" },
 	{ "RoutedMode", "FailoverCauses" },
 	{ "RoutedMode", "ForceNATKeepAlive" },
 	{ "RoutedMode", "ForwardOnFacility" },
@@ -343,10 +375,12 @@ static const char * KnowConfigEntries[][2] = {
 	{ "RoutedMode", "H4502EmulatorTransferMethod" },
 	{ "RoutedMode", "H46018NoNat" },
 	{ "RoutedMode", "H46023PublicIP" },
+	{ "RoutedMode", "H46023SignalGKRouted" },
 	{ "RoutedMode", "H46023STUN" },
 	{ "RoutedMode", "NATStdMin" },
 	{ "RoutedMode", "Q931PortRange" },
 	{ "RoutedMode", "RemoveCallOnDRQ" },
+	{ "RoutedMode", "RemoveFaxUDPOptionsFromRM" },
 	{ "RoutedMode", "RemoveH235Call" },
 	{ "RoutedMode", "RemoveH245AddressOnTunneling" },
 	{ "RoutedMode", "RemoveH460Call" },
@@ -366,10 +400,14 @@ static const char * KnowConfigEntries[][2] = {
 	{ "RoutedMode", "TranslateReceivedQ931Cause" },
 	{ "RoutedMode", "TreatUnregisteredNAT" },
 	{ "RoutedMode", "TranslateSentQ931Cause" },
+	{ "RoutedMode", "TranslateSorensonSourceInfo" },
 	{ "RoutedMode", "UseProvisionalRespToH245Tunneling" },
 	{ "Routing::CatchAll", "CatchAllAlias" },
 	{ "Routing::CatchAll", "CatchAllIP" },
 	{ "Routing::DNS", "ResolveNonLocalLRQ" },
+	{ "Routing::ENUM", "ResolveLRQ" },
+	{ "Routing::RDS", "ResolveLRQ" },
+	{ "Routing::SRV", "ResolveNonLocalLRQ" },
 	{ "Routing::Sql", "Database" },
 	{ "Routing::Sql", "Driver" },
 	{ "Routing::Sql", "Host" },
@@ -415,12 +453,14 @@ static const char * KnowConfigEntries[][2] = {
 	{ "SQLAuth", "Password" },
 	{ "SQLAuth", "RegQuery" },
 	{ "SQLAuth", "Username" },
+	{ "SQLConfig", "CacheTimeout" },
 	{ "SQLConfig", "ConfigQuery" },
 	{ "SQLConfig", "Database" },
 	{ "SQLConfig", "Driver" },
 	{ "SQLConfig", "GWPrefixesQuery" },
 	{ "SQLConfig", "Host" },
 	{ "SQLConfig", "Library" },
+	{ "SQLConfig", "MinPoolSize" },
 	{ "SQLConfig", "NeighborsQuery" },
 	{ "SQLConfig", "Password" },
 	{ "SQLConfig", "PermanentEndpointsQuery" },
@@ -437,9 +477,11 @@ static const char * KnowConfigEntries[][2] = {
 	{ "SQLPasswordAuth", "Query" },
 	{ "SQLPasswordAuth", "Username" },
 	{ "StatusAcct", "ConnectEvent" },
+	{ "StatusAcct", "RegisterEvent" },
 	{ "StatusAcct", "StartEvent" },
 	{ "StatusAcct", "StopEvent" },
 	{ "StatusAcct", "TimestampFormat" },
+	{ "StatusAcct", "UnregisterEvent" },
 	{ "StatusAcct", "UpdateEvent" },
 	{ "SyslogAcct", "ConnectEvent" },
 	{ "SyslogAcct", "StartEvent" },
@@ -461,15 +503,18 @@ static const char * KnowConfigEntries[][2] = {
 	{ "EP::", "TranslateSentQ931Cause" },
 	{ "Neighbor::", "AcceptForwardedLRQ" },
 	{ "Neighbor::", "AcceptPrefixes" },
+	{ "Neighbor::", "AuthUser" },
 	{ "Neighbor::", "Dynamic" },
 	{ "Neighbor::", "ForwardHopCount" },
 	{ "Neighbor::", "ForwardLRQ" },
 	{ "Neighbor::", "ForwardResponse" },
 	{ "Neighbor::", "GatekeeperIdentifier" },
+	{ "Neighbor::", "H46018Client" },
 	{ "Neighbor::", "Host" },
 	{ "Neighbor::", "Password" },
+	{ "Neighbor::", "SendAuthUser" },
+	{ "Neighbor::", "SendPassword" },
 	{ "Neighbor::", "SendPrefixes" },
-	{ "Neighbor::", "UseH46018" },
 
 	// uncheckable sections
 	{ "CapacityControl", "*" },
@@ -537,6 +582,7 @@ bool CheckConfig(PConfig * cfg, const PString & mainsection)
 			j++;
 		}
 		if (!found) {
+			cerr << "WARNING: Config section [" << sect << "] unknown" << endl;
 			PTRACE(0, "WARNING: Config section [" << sect << "] unknown");
 			warnings++;
 		} else if (!section_checkable) {
@@ -548,6 +594,7 @@ bool CheckConfig(PConfig * cfg, const PString & mainsection)
 				PCaselessString key = entries.GetKeyAt(j);
 				PString value = entries.GetDataAt(j);
 				if (value.IsEmpty()) {
+					cerr << "WARNING: Empty switch: [" << sect << "] " << key << "=" << endl;
 					PTRACE(0, "WARNING: Empty switch: [" << sect << "] " << key << "=");
 				}
 				unsigned k = 0;
@@ -561,6 +608,7 @@ bool CheckConfig(PConfig * cfg, const PString & mainsection)
 					}
 				}
 				if (!entry_found) {
+					cerr << "WARNING: Config entry [" << sect << "] " << key << "=" << value << " unknown" << endl;
 					PTRACE(0, "WARNING: Config entry [" << sect << "] " << key << "=" << value << " unknown");
 					warnings++;
 				}
@@ -579,15 +627,11 @@ bool CheckConfig(PConfig * cfg, const PString & mainsection)
 // or we get core dump
 void ExitGK()
 {
-#if PTRACING
 	Gatekeeper::EnableLogFileRotation(false);
-#endif
 
 	delete Toolkit::Instance();
 
-#if PTRACING
 	Gatekeeper::CloseLogFile();
-#endif
 	exit(0);
 }
 
@@ -658,9 +702,7 @@ BOOL WINAPI WinCtrlHandlerProc(DWORD dwCtrlType)
 
 	if( dwCtrlType == CTRL_LOGOFF_EVENT ) {
 		eventName = "CTRL_LOGOFF_EVENT";
-#if PTRACING
 		PTRACE(2,"GK\tGatekeeper received " <<eventName);
-#endif
 		// prevent shut down
 		return FALSE;
 	}
@@ -674,7 +716,7 @@ BOOL WINAPI WinCtrlHandlerProc(DWORD dwCtrlType)
 	else if( dwCtrlType == CTRL_SHUTDOWN_EVENT )
 		eventName = "CTRL_SHUTDOWN_EVENT";
 
-#if PTRACING
+#ifndef hasPTLibTraceOnShutdownBug
 	PTRACE(1,"GK\tGatekeeper shutdown due to "<<eventName);
 #endif
 
@@ -698,7 +740,7 @@ BOOL WINAPI WinCtrlHandlerProc(DWORD dwCtrlType)
 	return TRUE;
 }
 
-bool Gatekeeper::SetUserAndGroup(const PString& /*username*/)
+bool Gatekeeper::SetUserAndGroup(const PString & /*username*/)
 {
 	return false;
 }
@@ -721,9 +763,9 @@ BOOL SetDEP(__in DWORD dwFlags = PROCESS_DEP_ENABLE)
 
 #else	// _WIN32
 
-#  include <pwd.h>
+#include <pwd.h>
 
-bool Gatekeeper::SetUserAndGroup(const PString &username)
+bool Gatekeeper::SetUserAndGroup(const PString & username)
 {
 #if defined(P_PTHREADS) && !defined(P_THREAD_SAFE_CLIB)
 	static const size_t MAX_PASSWORD_BUFSIZE = 1024;
@@ -860,10 +902,8 @@ const PString Gatekeeper::GetArgumentsParseString() const
 #ifdef HAS_SETUSERNAME
 		 "u-user:"
 #endif
-#if PTRACING
 		 "t-trace."
 		 "o-output:"
-#endif
 		 "c-config:"
 		 "s-section:"
 		 "-pid:"
@@ -875,7 +915,7 @@ const PString Gatekeeper::GetArgumentsParseString() const
 }
 
 
-bool Gatekeeper::InitHandlers(const PArgList& args)
+bool Gatekeeper::InitHandlers(const PArgList & args)
 {
 #ifdef _WIN32
 	SetConsoleCtrlHandler(WinCtrlHandlerProc, TRUE);
@@ -921,9 +961,8 @@ bool Gatekeeper::InitHandlers(const PArgList& args)
 }
 
 
-bool Gatekeeper::InitLogging(const PArgList &args)
+bool Gatekeeper::InitLogging(const PArgList & args)
 {
-#if PTRACING
 	// Syslog is the default when compiled as service, but we don't want that
 	PTrace::ClearOptions(PTrace::SystemLogStream);
 	PTrace::SetOptions(PTrace::DateAndTime | PTrace::TraceLevel | PTrace::FileAndLine);
@@ -935,13 +974,12 @@ bool Gatekeeper::InitLogging(const PArgList &args)
 			return FALSE;
 		}
 	}
-#endif
 
 	return TRUE;
 }
 
 
-bool Gatekeeper::InitConfig(const PArgList &args)
+bool Gatekeeper::InitConfig(const PArgList & args)
 {
 	// get the name of the config file
 	PFilePath fp;
@@ -968,7 +1006,7 @@ bool Gatekeeper::InitConfig(const PArgList &args)
 }
 
 
-void Gatekeeper::PrintOpts(void)
+void Gatekeeper::PrintOpts()
 {
 	cout << "Options:\n"
 		"  -r  --routed       : Use gatekeeper routed call signaling\n"
@@ -980,10 +1018,8 @@ void Gatekeeper::PrintOpts(void)
 #ifdef HAS_SETUSERNAME
 		"  -u  --user name    : Run as this user\n"
 #endif
-#if PTRACING
 		"  -t  --trace        : Set trace verbosity\n"
 		"  -o  --output file  : Write trace to this file\n"
-#endif
 		"  -c  --config file  : Specify which config file to use\n"
 		"  -s  --section sec  : Specify which main section to use in the config file\n"
 		"      --pid file     : Specify the pid file\n"
@@ -1035,8 +1071,8 @@ void Gatekeeper::Main()
 
 		if ( !SetUserAndGroup(username) ) {
 			cout << "GNU Gatekeeper could not run as user "
-			     << username
-			     << endl;
+				<< username
+				<< endl;
 			return;
 		}
 	}
@@ -1070,6 +1106,8 @@ void Gatekeeper::Main()
 			fake_cmdline += " -o " + log_trace_file;
 	}
 	if (!fake_cmdline.IsEmpty()) {
+		for (int t=0; t < args.GetOptionCount('t'); t++)
+			fake_cmdline += " -t";
 		PArgList fake_args(fake_cmdline);
 		fake_args.Parse(GetArgumentsParseString());
 		InitLogging(fake_args);
@@ -1081,6 +1119,14 @@ void Gatekeeper::Main()
 	cout << welcome << '\n';
 	PTRACE(1, welcome);
 
+#ifdef hasIPV6
+	if (Toolkit::Instance()->IsIPv6Enabled()) {
+		PTRACE(1, "IPv6 enabled");
+		GNUGK_INADDR_ANY = PIPSocket::Address("::");
+		PIPSocket::SetDefaultIpAddressFamily(AF_INET6);
+	}
+#endif
+
 	vector<PIPSocket::Address> GKHome;
 	PString home(Toolkit::Instance()->GetGKHome(GKHome));
 	if (GKHome.empty()) {
@@ -1093,7 +1139,7 @@ void Gatekeeper::Main()
 	if (Toolkit::Instance()->isBehindNAT(addr))
 		cout << "Public IP: " << addr.AsString() << "\n\n";
 	else
-	    cout << "\n";
+		cout << "\n";
 
 	// Copyright notice
 	cout <<
@@ -1101,7 +1147,7 @@ void Gatekeeper::Main()
 		"modify it under the terms of the GNU General Public License version 2.\n"
 		"We also explicitly grant the right to link this code\n"
 		"with the OpenH323/H323Plus and OpenSSL library.\n"
-	     << endl;
+		<< endl;
 
 #ifdef HAS_H46018
 	cout << "This program contains H.460.18 and H.460.19 technology patented by Tandberg\n"
@@ -1143,7 +1189,11 @@ void Gatekeeper::Main()
 		RasSrv->SetRoutedMode(true, (args.GetOptionCount('r') > 1 || args.HasOption("h245routed")));
 	else if (args.HasOption('d'))
 		RasSrv->SetRoutedMode(false, false);
-	else
+#ifdef HAS_H235_MEDIA
+    else if (Toolkit::Instance()->IsH235HalfCallMediaEnabled()) 
+        RasSrv->SetRoutedMode(true, true);
+#endif 
+	else 
 		RasSrv->SetRoutedMode();
 
 	// Load ENUM servers
@@ -1168,6 +1218,7 @@ void Gatekeeper::Main()
 
 	// graceful shutdown
 	cerr << "\nShutting down gatekeeper . . . ";
+
 	ShutdownHandler();
 	cerr << "done\n";
 
@@ -1178,7 +1229,6 @@ void Gatekeeper::Main()
 #endif // _WIN32
 }
 
-#if PTRACING
 namespace {
 const char* const logConfigSectionName = "Logfile";
 }
@@ -1188,10 +1238,7 @@ const char* const Gatekeeper::m_intervalNames[] =
 	"Hourly", "Daily", "Weekly", "Monthly"
 };
 
-void Gatekeeper::GetRotateInterval(
-	PConfig& cfg,
-	const PString& section
-	)
+void Gatekeeper::GetRotateInterval(PConfig & cfg, const PString & section)
 {
 	PString s;
 
@@ -1243,9 +1290,7 @@ void Gatekeeper::GetRotateInterval(
 	}
 }
 
-void Gatekeeper::EnableLogFileRotation(
-	bool enable
-	)
+void Gatekeeper::EnableLogFileRotation(bool enable)
 {
 	PWaitAndSignal lock(m_logFileMutex);
 
@@ -1364,9 +1409,7 @@ void Gatekeeper::EnableLogFileRotation(
 	}
 }
 
-void Gatekeeper::RotateOnTimer(
-	GkTimer* timer
-	)
+void Gatekeeper::RotateOnTimer(GkTimer* timer)
 {
 	m_logFileMutex.Wait();
 	if (m_rotateInterval == Monthly) {
@@ -1392,9 +1435,7 @@ void Gatekeeper::RotateOnTimer(
 	RotateLogFile();
 }
 
-bool Gatekeeper::SetLogFilename(
-	const PString& filename
-	)
+bool Gatekeeper::SetLogFilename(const PString & filename)
 {
 	if (filename.IsEmpty())
 		return false;
@@ -1529,4 +1570,3 @@ void Gatekeeper::CloseLogFile()
 #endif
 	m_logFile = NULL;
 }
-#endif // PTRACING
