@@ -2,7 +2,7 @@
 //
 // gkauth.cxx
 //
-// Copyright (c) 2001-2010, Jan Willamowius
+// Copyright (c) 2001-2011, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -231,7 +231,6 @@ GkAuthenticator::GkAuthenticator(
 		}
 	}
 
-#if PTRACING
 	// convert bit flags to human readable names
 	PString rasFlagsStr, miscFlagsStr;
 	
@@ -263,7 +262,6 @@ GkAuthenticator::GkAuthenticator(
 	PTRACE(1, "GKAUTH\t" << GetName() << " rule added to check RAS: "
 		<< rasFlagsStr << ", OTHER: " << miscFlagsStr
 		);
-#endif	
 
 	m_h235Authenticators = NULL;
 }
@@ -644,19 +642,20 @@ GkAuthenticatorList::GkAuthenticatorList()
 #ifdef OpenH323Factory
 	PStringList authlist = Toolkit::Instance()->GetAuthenticatorList();
 
-    PFactory<H235Authenticator>::KeyList_T keyList = PFactory<H235Authenticator>::GetKeyList();
-    PFactory<H235Authenticator>::KeyList_T::const_iterator r;
-    for (r = keyList.begin(); r != keyList.end(); ++r) {
-      H235Authenticator * Auth = PFactory<H235Authenticator>::CreateInstance(*r);
-      if (authlist.GetSize() > 0) {
-		for (PINDEX i=0; i< authlist.GetSize(); i++) {
-			if (PString(Auth->GetName()) == authlist[i]) {
-			   PTRACE(4,"GKAUTH\tLoaded Authenticator " << Auth->GetName() << " from Policy");
-               m_h235authenticators.Append(Auth);
+	PFactory<H235Authenticator>::KeyList_T keyList = PFactory<H235Authenticator>::GetKeyList();
+	PFactory<H235Authenticator>::KeyList_T::const_iterator r;
+	for (r = keyList.begin(); r != keyList.end(); ++r) {
+		H235Authenticator * Auth = PFactory<H235Authenticator>::CreateInstance(*r);
+		if (authlist.GetSize() > 0) {
+			for (PINDEX i=0; i< authlist.GetSize(); i++) {
+				if (PString(Auth->GetName()) == authlist[i]) {
+					PTRACE(4,"GKAUTH\tLoaded Authenticator " << Auth->GetName() << " from Policy");
+					m_h235authenticators.Append(Auth);
+				}
 			}
-		}
-	  } else
+		} else {
 	        m_h235authenticators.Append(Auth);
+		}
 	}
 #endif
 }
@@ -831,7 +830,6 @@ void GkAuthenticatorList::OnReload()
 
 	if (mechanisms.GetSize() > 0 && algorithmOIDs.GetSize() > 0) {
 		if (PTrace::CanTrace(4)) {
-#if PTRACING
 			ostream& strm = PTrace::Begin(4,__FILE__,__LINE__);
 			strm <<"GkAuth\tH.235 capabilities selected for GCF:\n";
 			strm <<"\tAuthentication mechanisms: \n";
@@ -841,7 +839,6 @@ void GkAuthenticatorList::OnReload()
 			for (i = 0; i < algorithmOIDs.GetSize(); i++)
 				strm << "\t\t" << algorithmOIDs[i] << '\n';
 			PTrace::End(strm);
-#endif
 		}
 #ifdef OpenH323Factory
 	}
@@ -1164,11 +1161,6 @@ SimplePasswordAuth::~SimplePasswordAuth()
 	delete m_cache;
 }
 
-int SimplePasswordAuth::Check(RasPDU<H225_GatekeeperRequest> & request, unsigned &)
-{
-	return doCheck(request);
-}
-
 int SimplePasswordAuth::Check(
 	RasPDU<H225_RegistrationRequest> & request, 
 	RRQAuthData& /*authData*/
@@ -1662,19 +1654,16 @@ bool AliasAuth::doCheck(
 	)
 {
 	const PStringArray authrules(condition.Tokenise("&|", FALSE));
-#if PTRACING
 	if (authrules.GetSize() < 1) {
 		PTRACE(2, "GKAUTH\t" << GetName() << " contains an empty auth condition");
 		return false;
 	}
-#endif
 	for (PINDEX i = 0; i < authrules.GetSize(); ++i)
 		for (PINDEX j = 0; j < sigaddr.GetSize(); ++j)
 			if (CheckAuthRule(sigaddr[j], authrules[i])) {
 				PTRACE(5, "GKAUTH\t" << GetName() << " auth rule '"
 					<< authrules[i] << "' applied successfully to RRQ "
-					" from " << AsDotString(sigaddr[j])
-					);
+					" from " << AsDotString(sigaddr[j]));
 				return true;
 			}
 	return false;
@@ -1682,16 +1671,14 @@ bool AliasAuth::doCheck(
 
 bool AliasAuth::CheckAuthRule(
 	/// a signaling address for the endpoint that sent the request
-	const H225_TransportAddress& sigaddr,
+	const H225_TransportAddress & sigaddr,
 	/// the auth rule to be used for checking
-	const PString& authrule
+	const PString & authrule
 	)
 {
 	const PStringArray rule = authrule.Tokenise(":", false);
 	if (rule.GetSize() < 1) {
-		PTRACE(1, "GKAUTH\t" << GetName() << " found invalid empty auth rule '"
-			<< authrule << '\''
-			);
+		PTRACE(1, "GKAUTH\t" << GetName() << " found invalid empty auth rule '" << authrule << '\'');
 		return false;
 	}
 	
@@ -1708,8 +1695,7 @@ bool AliasAuth::CheckAuthRule(
 		//   sigaddr:.*ipAddress .* ip = .* c3 47 e2 a2 .*port = 1720.*
 		if (rule.GetSize() < 2) {
 			PTRACE(1, "GKAUTH\t" << GetName() << " found invalid empty sigaddr "
-				"auth rule '" << authrule << '\''
-				);
+				"auth rule '" << authrule << '\'');
 			return false;
 		}
 		return Toolkit::MatchRegex(AsString(sigaddr), rule[1].Trim()) != 0;
@@ -1718,28 +1704,23 @@ bool AliasAuth::CheckAuthRule(
 		//   sigip:195.71.129.69:1720
 		if (rule.GetSize() < 2) {
 			PTRACE(1, "GKAUTH\t" << GetName() << " found invalid empty sigip "
-				"auth rule '" << authrule << '\''
-				);
+				"auth rule '" << authrule << '\'');
 			return false;
 		}
+		PString allowed_ip = authrule.Mid(authrule.Find("sigip:")+6).Trim();
+		PStringArray ip_parts = SplitIPAndPort(allowed_ip, GK_DEF_ENDPOINT_SIGNAL_PORT);
 		PIPSocket::Address ip;
-		PIPSocket::GetHostAddress(rule[1].Trim(), ip);
-		const WORD port = (WORD)((rule.GetSize() < 3) 
-			? GK_DEF_ENDPOINT_SIGNAL_PORT : rule[2].Trim().AsInteger());
+		PIPSocket::GetHostAddress(ip_parts[0], ip);
+		const WORD port = (WORD)(ip_parts[1].AsUnsigned());
 		return (sigaddr == SocketToH225TransportAddr(ip, port));
 	} else {
-		PTRACE(1, "GKAUTH\t" << GetName() << " found unknown auth rule '"
-			<< rName << '\''
-			);
+		PTRACE(1, "GKAUTH\t" << GetName() << " found unknown auth rule '" << rName << '\'');
 		return false;
 	}
 }
 
 // class PrefixAuth
 
-// Initial author: Michael Rubashenkkov  2002/01/14 (GkAuthorize)
-// Completely rewrite by Chih-Wei Huang  2002/05/01
-
 class AuthRule;
 class AuthObj;
 
@@ -1779,9 +1760,7 @@ public:
 		);
 
 protected:
-	virtual int doCheck(
-		const AuthObj& aobj
-		);
+	virtual int doCheck(const AuthObj& aobj);
 
 private:
 	PrefixAuth();
@@ -1810,9 +1789,7 @@ public:
 class ARQAuthObj : public AuthObj 
 {
 public:
-	ARQAuthObj(
-		const H225_AdmissionRequest& arq
-		);
+	ARQAuthObj(const H225_AdmissionRequest & arq);
 
 	virtual bool IsValid() const { return m_ep; }
 
@@ -1834,9 +1811,7 @@ private:
 class LRQAuthObj : public AuthObj 
 {
 public:
-	LRQAuthObj(
-		const H225_LocationRequest& lrq
-		);
+	LRQAuthObj(const H225_LocationRequest& lrq);
 
 	virtual PStringArray GetPrefixes() const;
 
@@ -1849,7 +1824,7 @@ private:
 	LRQAuthObj& operator=(const LRQAuthObj&);
 	
 private:
-	const H225_LocationRequest& m_lrq;
+	const H225_LocationRequest & m_lrq;
 	PIPSocket::Address m_ipAddress;
 };
 
@@ -1869,21 +1844,13 @@ public:
 	
 	virtual ~AuthRule() { delete m_next; }
 
-	virtual bool Match(
-		const AuthObj& aobj
-		) = 0;
-		
-	int Check(
-		const AuthObj& aobj
-		);
-	
-	bool operator<(
-		const AuthRule& obj
-		) const { return m_priority < obj.m_priority; }
-	
-	void SetNext(
-		AuthRule* next
-		) { m_next = next; }
+	virtual bool Match(const AuthObj& aobj) = 0;
+
+	int Check(const AuthObj& aobj);
+
+	bool operator<(const AuthRule & obj) const { return m_priority < obj.m_priority; }
+
+	void SetNext(AuthRule* next) { m_next = next; }
 
 private:
 	AuthRule();
@@ -1905,32 +1872,24 @@ class NullRule : public AuthRule
 public:
 	NullRule() : AuthRule(e_nomatch, false) { SetName("NULL"); }
 	
-	virtual bool Match(
-		const AuthObj& /*aobj*/
-		) { return false; }
+	virtual bool Match(const AuthObj& /*aobj*/) { return false; }
 
 private:
 	NullRule(const NullRule&);
 	NullRule& operator=(const NullRule&);
 };
 
-class IPv4AuthRule : public AuthRule 
+class IPAuthRule : public AuthRule 
 {
 public:
-	IPv4AuthRule(
-		Result fate, 
-		const PString& ipStr, 
-		bool inverted
-		);
+	IPAuthRule(Result fate, const PString & ipStr, bool inverted);
 
-	virtual bool Match(
-		const AuthObj& aobj
-		);
+	virtual bool Match(const AuthObj& aobj);
 
 private:
-	IPv4AuthRule();
-	IPv4AuthRule(const IPv4AuthRule&);
-	IPv4AuthRule& operator=(const IPv4AuthRule&);
+	IPAuthRule();
+	IPAuthRule(const IPAuthRule&);
+	IPAuthRule& operator=(const IPAuthRule&);
 	
 private:
 	PIPSocket::Address m_network, m_netmask;
@@ -1946,11 +1905,9 @@ public:
 		) : AuthRule(fate, inverted), m_pattern(aliasStr) 
 	{ 
 		m_priority = -1;
-#if PTRACING
 		SetName(PString((fate == e_allow) ? "allow alias" : "deny alias")
 			+ (inverted ? ":!" : ":") + aliasStr
 			);
-#endif
 	}
 
 	virtual bool Match(
@@ -2053,34 +2010,23 @@ inline void delete_rule(PrefixAuth::Rules::value_type r)
 	r.second = NULL;
 }
 
-IPv4AuthRule::IPv4AuthRule(
-	Result fate, 
-	const PString& ipStr, 
-	bool inverted
-	) 
+IPAuthRule::IPAuthRule(Result fate, const PString & ipStr, bool inverted) 
 	: AuthRule(fate, inverted)
 {
 	Toolkit::GetNetworkFromString(ipStr, m_network, m_netmask);
 	DWORD n = ~PIPSocket::Net2Host(DWORD(m_netmask));
 	for (m_priority = 0; n; n >>= 1)
 		++m_priority;
-#if PTRACING
-	SetName(PString((fate == e_allow) ? "allow ipv4(" : "deny ipv4(")
-		+ PString(m_priority) + (inverted ? "):!" : "):") + ipStr
-		);
-#endif
+	SetName(PString((fate == e_allow) ? "allow ip(" : "deny ip(")
+		+ PString(m_priority) + (inverted ? "):!" : "):") + ipStr);
 }
 
-bool IPv4AuthRule::Match(
-	const AuthObj& aobj
-	)
+bool IPAuthRule::Match(const AuthObj & aobj)
 {
-	return ((aobj.GetIP() & m_netmask) == m_network);
+	return ((aobj.GetIP().GetVersion() == m_network.GetVersion()) && ((aobj.GetIP() & m_netmask) == m_network));
 }
 
-bool AliasAuthRule::Match(
-	const AuthObj& aobj
-	)
+bool AliasAuthRule::Match(const AuthObj& aobj)
 {
 	return aobj.GetAliases().FindRegEx(m_pattern) != P_MAX_INDEX;
 }
@@ -2096,11 +2042,12 @@ inline bool comp_authrule_priority(AuthRule *a1, AuthRule *a2)
 }
 
 namespace {
-const char* const prfflag="prf:";
-const char* const allowflag="allow";
-const char* const denyflag="deny";
-const char* const ipflag="ipv4:";
-const char* const aliasflag="alias:";
+const char* const allowflag = "allow";
+const char* const denyflag  = "deny";
+const char* const ipflag    = "ip:";
+const char* const ipv4flag  = "ipv4:";
+const char* const ipv6flag  = "ipv6:";
+const char* const aliasflag = "alias:";
 }
 
 // class PrefixAuth
@@ -2114,6 +2061,8 @@ PrefixAuth::PrefixAuth(
 	m_defaultRule = GetDefaultStatus();
 
 	const int ipfl = strlen(ipflag);
+	const int ipv4fl = strlen(ipv4flag);
+	const int ipv6fl = strlen(ipv6flag);
 	const int aliasfl = strlen(aliasflag);
 	
 	const PStringToString cfgs = GetConfig()->GetAllKeyValues(name);
@@ -2149,7 +2098,15 @@ PrefixAuth::PrefixAuth(
 				? AuthRule::e_allow : AuthRule::e_deny;
 			PINDEX pp;
 			if ((pp = rules[j].Find(ipflag)) != P_MAX_INDEX)
-				rls[j] = new IPv4AuthRule(fate, rules[j].Mid(pp + ipfl).Trim(), 
+				rls[j] = new IPAuthRule(fate, rules[j].Mid(pp + ipfl).Trim(), 
+					is_inverted(rules[j], pp)
+					);
+			else if ((pp = rules[j].Find(ipv4flag)) != P_MAX_INDEX)
+				rls[j] = new IPAuthRule(fate, rules[j].Mid(pp + ipv4fl).Trim(), 
+					is_inverted(rules[j], pp)
+					);
+			else if ((pp = rules[j].Find(ipv6flag)) != P_MAX_INDEX)
+				rls[j] = new IPAuthRule(fate, rules[j].Mid(pp + ipv6fl).Trim(), 
 					is_inverted(rules[j], pp)
 					);
 			else if ((pp = rules[j].Find(aliasflag)) != P_MAX_INDEX)
@@ -2257,14 +2214,13 @@ int PrefixAuth::doCheck(
 			default: // try next prefix...
 				j = iter;
 				PTRACE(4, "GKAUTH\t" << GetName() << " rule matched and "
-					" could not reject or accept destination prefix '" 
+					"could not reject or accept destination prefix '" 
 					<< ((iter->first == " ") ? PString("ALL") : iter->first)
 					<< "' for alias '" << destinationInfo[i] << '\''
 					);
 			}
 		}
 	}
-#if PTRACING
 	if (m_defaultRule == e_ok)
 		PTRACE(4, "GKAUTH\t" << GetName() << " default rule accepted "
 			"the request"
@@ -2277,7 +2233,6 @@ int PrefixAuth::doCheck(
 		PTRACE(4, "GKAUTH\t" << GetName() << " could not reject or "
 			"accept the request"
 			);
-#endif
 	return m_defaultRule;
 }
 
