@@ -2,17 +2,19 @@
 //
 // gkauth.h
 //
+// Copyright (c) 2001-2010, Jan Willamowius
+//
 // Gatekeeper authentication modules
 //
-// This work is published under the GNU Public License (GPL)
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKAUTH_H
-#define GKAUTH_H "@(#) $Id: gkauth.h,v 1.49 2010/01/24 21:04:46 willamowius Exp $"
+#define GKAUTH_H "@(#) $Id: gkauth.h,v 1.55 2011/11/23 23:22:50 willamowius Exp $"
 
 #include <map>
 #include <list>
@@ -57,7 +59,7 @@ template<class> class RasPDU;
 template<class> struct RasInfo;
 
 namespace Routing {
-struct Route;
+	class Route;
 }
 
 /// Data read/written during RRQ processing by all configured 
@@ -85,12 +87,10 @@ struct ARQAuthData
 		const endptr& ep,
 		const callptr& call
 		);
-	virtual ~ARQAuthData();
-		
+
 	ARQAuthData& operator=(const ARQAuthData& obj);
 		
-	void SetRouteToAlias(H225_AliasAddress* alias);
-	void SetRouteToAlias(const H225_AliasAddress& alias);
+	void SetRouteToAlias(const H225_ArrayOf_AliasAddress & alias);
 	void SetRouteToAlias(const PString& alias, int tag = -1);
 		
 	/// -1 if not set, H225_AdmissionRejectReason enum otherwise
@@ -116,7 +116,7 @@ struct ARQAuthData
 	/// H225_CallCreditServiceControl_billingMode or -1, if not defined
 	int m_billingMode;
 	/// if not NULL, route the call to the specified alias
-	H225_AliasAddress* m_routeToAlias;
+	H225_ArrayOf_AliasAddress m_routeToAlias;
 	/// if not empty, route the call to the specified destinations
 	std::list<Routing::Route> m_destinationRoutes;
 	/// override global proxy setting from the config (see #CallRec::ProxyMode enum#)
@@ -147,9 +147,8 @@ struct SetupAuthData
 		
 	SetupAuthData& operator=(const SetupAuthData& obj);
 	
-	void SetRouteToAlias(H225_AliasAddress* alias);
-	void SetRouteToAlias(const H225_AliasAddress& alias);
-	void SetRouteToAlias(const PString& alias, int tag = -1);
+	void SetRouteToAlias(const H225_ArrayOf_AliasAddress & alias);
+	void SetRouteToAlias(const PString & alias, int tag = -1);
 
 	/// -1 if not set, H225_ReleaseCompleteReason enum otherwise
 	int m_rejectReason;
@@ -170,7 +169,7 @@ struct SetupAuthData
 	/// number dialed by the user (Called-Station-Id before rewrite)
 	PString m_dialedNumber;
 	/// if not NULL, route the call to the specified alias
-	H225_AliasAddress* m_routeToAlias;
+	H225_ArrayOf_AliasAddress m_routeToAlias;
 	/// if not empty, route the call to the specified destinations
 	std::list<Routing::Route> m_destinationRoutes;
 	/// override global proxy setting from the config (see #CallRec::ProxyMode enum#)
@@ -528,8 +527,7 @@ class SimplePasswordAuth : public GkAuthenticator
 public:
 	enum SupportedRasChecks {
 		/// bitmask of RAS checks implemented by this module
-		SimplePasswordAuthRasChecks = RasInfo<H225_GatekeeperRequest>::flag
-			| RasInfo<H225_RegistrationRequest>::flag
+		SimplePasswordAuthRasChecks = RasInfo<H225_RegistrationRequest>::flag
 			| RasInfo<H225_UnregistrationRequest>::flag
 			| RasInfo<H225_BandwidthRequest>::flag
 			| RasInfo<H225_DisengageRequest>::flag
@@ -547,7 +545,6 @@ public:
 	virtual ~SimplePasswordAuth();
 
 	// overriden from class GkAuthenticator
-	virtual int Check(RasPDU<H225_GatekeeperRequest>& req, unsigned& rejectReason);
 	virtual int Check(RasPDU<H225_UnregistrationRequest>& req, unsigned& rejectReason);
 	virtual int Check(RasPDU<H225_BandwidthRequest>& req, unsigned& rejectReason);
 	virtual int Check(RasPDU<H225_DisengageRequest>& req, unsigned& rejectReason);
@@ -659,8 +656,8 @@ protected:
 			return e_ok;
 		}
 
-		PString username = PString();
-		PString password = PString();
+		PString username = PString::Empty();
+		PString password = PString::Empty();
 		if (!ResolveUserName(req.m_tokens, req.m_cryptoTokens,username)) {
             PTRACE(4, "GKAUTH\t" << GetName() << " No username resolved from tokens.");
 			return e_fail;
@@ -784,7 +781,6 @@ protected:
 		/// password string, if the match is found
 		PString& password
 		);
-
 };
 
 #endif
