@@ -52,7 +52,8 @@ RadAcct::RadAcct(
 	m_radiusClient = new RadiusClient(*cfg, cfgSec);
 
 	m_nasIpAddress = m_radiusClient->GetLocalAddress();
-	if (m_nasIpAddress == INADDR_ANY) {
+	UnmapIPv4Address(m_nasIpAddress);
+	if (m_nasIpAddress == GNUGK_INADDR_ANY) {
 		vector<PIPSocket::Address> interfaces;
 		Toolkit::Instance()->GetGKHome(interfaces);
 		if (!interfaces.empty())
@@ -109,7 +110,10 @@ GkAcctLogger::Status RadAcct::Log(
 	PIPSocket::Address addr;
 					
 	// Gk works as NAS point, so append NAS IP
-	pdu->AppendAttr(RadiusAttr::NasIpAddress, m_nasIpAddress);
+	if (m_nasIpAddress.GetVersion() == 6)
+		pdu->AppendAttr(RadiusAttr::NasIpv6Address, m_nasIpAddress);
+	else
+		pdu->AppendAttr(RadiusAttr::NasIpAddress, m_nasIpAddress);
 	pdu->AppendAttr(m_attrNasIdentifier);
 	pdu->AppendAttr(RadiusAttr::NasPortType, RadiusAttr::NasPort_Virtual);
 		
@@ -272,16 +276,14 @@ GkAcctLogger::Status RadAcct::Log(
 		
 	// send request and wait for response
 	RadiusPDU * response = NULL;
-	bool result = OnSendPDU(*pdu, evt, call);
-	
+	bool result = true;
+
 	// accounting updates must be fast, so we are just sending
 	// the request to the server and are not waiting for a response
-	if (result) {
-		if (evt & AcctUpdate) {
-			result = m_radiusClient->SendRequest(*pdu);
-		} else {
-			result = m_radiusClient->MakeRequest(*pdu, response) && (response != NULL);
-		}
+	if (evt & AcctUpdate) {
+		result = m_radiusClient->SendRequest(*pdu);
+	} else {
+		result = m_radiusClient->MakeRequest(*pdu, response) && (response != NULL);
 	}
 			
 	delete pdu;
@@ -294,35 +296,16 @@ GkAcctLogger::Status RadAcct::Log(
 	if (response) {
 		// check if Access-Request has been accepted
 		result = (response->GetCode() == RadiusPDU::AccountingResponse);
-		if (result)
-			result = OnReceivedPDU(*response, evt, call);
-		else
+		if (!result) {
 			PTRACE(4, "RADACCT\t" << GetName() << " - received response is not "
 				" an AccountingResponse, event " << evt << ", call no. "
 				<< (call ? call->GetCallNumber() : 0));
+		}
 		delete response;
 	}
 	return result ? Ok : Fail;
 }
 
-bool RadAcct::OnSendPDU(
-	RadiusPDU& /*pdu*/,
-	GkAcctLogger::AcctEvent /*evt*/,
-	const callptr& /*call*/
-	)
-{
-	return true;
-}
-
-bool RadAcct::OnReceivedPDU(
-	RadiusPDU& /*pdu*/,
-	GkAcctLogger::AcctEvent /*evt*/,
-	const callptr& /*call*/
-	)
-{
-	return true;
-}
-
 namespace {
 	// append RADIUS based accounting logger to the global list of loggers
 	GkAcctLoggerCreator<RadAcct> RadAcctCreator("RadAcct");
