@@ -5,7 +5,7 @@
  * Please see docs/radauth.txt for more details.
  *
  * Copyright (c) 2003, Quarcom FHU, Michal Zygmuntowicz
- * Copyright (c) 2005-2010, Jan Willamowius
+ * Copyright (c) 2005-2011, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -76,7 +76,8 @@ RadAuthBase::RadAuthBase(
 	/// build RADIUS client
 	m_radiusClient = new RadiusClient(*GetConfig(), configSectionName);
 	m_nasIpAddress = m_radiusClient->GetLocalAddress();
-	if (m_nasIpAddress == INADDR_ANY) {
+	UnmapIPv4Address(m_nasIpAddress);
+	if (m_nasIpAddress == GNUGK_INADDR_ANY) {
 		vector<PIPSocket::Address> interfaces;
 		Toolkit::Instance()->GetGKHome(interfaces);
 		if (!interfaces.empty())
@@ -100,9 +101,9 @@ RadAuthBase::~RadAuthBase()
 
 int RadAuthBase::Check(
 	/// RRQ RAS message to be authenticated
-	RasPDU<H225_RegistrationRequest>& rrqPdu, 
+	RasPDU<H225_RegistrationRequest> & rrqPdu, 
 	/// authorization data (reject reason, ...)
-	RRQAuthData& authData
+	RRQAuthData & authData
 	)
 {
 	H225_RegistrationRequest& rrq = (H225_RegistrationRequest&)rrqPdu;
@@ -119,7 +120,10 @@ int RadAuthBase::Check(
 	}
 	
 	// Gk works as NAS point, so append NAS IP
-	pdu->AppendAttr(RadiusAttr::NasIpAddress, m_nasIpAddress);
+	if (m_nasIpAddress.GetVersion() == 6)
+		pdu->AppendAttr(RadiusAttr::NasIpv6Address, m_nasIpAddress);
+	else
+		pdu->AppendAttr(RadiusAttr::NasIpAddress, m_nasIpAddress);
 	// NAS-Identifier as Gk name
 	pdu->AppendAttr(m_attrNasIdentifier);
 	// Gk does not have a concept of physical ports,
@@ -166,10 +170,6 @@ int RadAuthBase::Check(
 			);
 	}
 	
-	if (!OnSendPDU(*pdu, rrqPdu, authData)) {
-		delete pdu;
-		return e_fail;
-	}
 	// send request and wait for response
 	RadiusPDU* response = NULL;
 	bool result = m_radiusClient->MakeRequest(*pdu, response) && response;
@@ -316,9 +316,7 @@ int RadAuthBase::Check(
 		}
 	}
 
-	if (result)
-		result = OnReceivedPDU(*response, rrqPdu, authData);
-	else
+	if (!result)
 		authData.m_rejectReason = H225_RegistrationRejectReason::e_securityDenial;
 				
 	delete response;
@@ -379,7 +377,10 @@ int RadAuthBase::Check(
 	}
 	
 	// Gk acts as NAS, so include NAS IP
-	pdu->AppendAttr(RadiusAttr::NasIpAddress, m_nasIpAddress);
+	if (m_nasIpAddress.GetVersion() == 6)
+		pdu->AppendAttr(RadiusAttr::NasIpv6Address, m_nasIpAddress);
+	else
+		pdu->AppendAttr(RadiusAttr::NasIpAddress, m_nasIpAddress);
 	// NAS-Identifier as Gk name
 	pdu->AppendAttr(m_attrNasIdentifier);
 	// NAS-Port-Type as Virtual, since Gk does
@@ -456,10 +457,6 @@ int RadAuthBase::Check(
 		pdu->AppendAttr(m_attrH323GwId);
 	}
 				
-	if (!OnSendPDU(*pdu, arqPdu, authData)) {
-		delete pdu;
-		return e_fail;
-	}
 	// send the request and wait for a response
 	RadiusPDU* response = NULL;
 	bool result = m_radiusClient->MakeRequest(*pdu, response) && response;
@@ -716,9 +713,7 @@ int RadAuthBase::Check(
 		}
 	}
 
-	if (result)
-		result = OnReceivedPDU(*response, arqPdu, authData);
-	else
+	if (!result)
 		authData.m_rejectReason = H225_AdmissionRejectReason::e_securityDenial;
 					
 	delete response;
@@ -757,7 +752,10 @@ int RadAuthBase::Check(
 	}
 	
 	// Gk acts as NAS, so include NAS IP
-	pdu->AppendAttr(RadiusAttr::NasIpAddress, m_nasIpAddress);
+	if (m_nasIpAddress.GetVersion() == 6)
+		pdu->AppendAttr(RadiusAttr::NasIpv6Address, m_nasIpAddress);
+	else
+		pdu->AppendAttr(RadiusAttr::NasIpAddress, m_nasIpAddress);
 	// NAS-Identifier as Gk name
 	pdu->AppendAttr(m_attrNasIdentifier);
 	// NAS-Port-Type as Virtual, since Gk does
@@ -825,10 +823,6 @@ int RadAuthBase::Check(
 		pdu->AppendAttr(m_attrH323GwId);
 	}
 				
-	if (!OnSendPDU(*pdu, setup, authData)) {
-		delete pdu;
-		return e_fail;
-	}
 	// send the request and wait for a response
 	RadiusPDU* response = NULL;
 	bool result = m_radiusClient->MakeRequest(*pdu, response) && response;
@@ -1033,9 +1027,7 @@ int RadAuthBase::Check(
 		}
 	}
 
-	if (result)
-		result = OnReceivedPDU(*response, setup, authData);
-	else
+	if (!result)
 		authData.m_rejectCause = Q931::CallRejected;
 					
 	delete response;
@@ -1043,60 +1035,6 @@ int RadAuthBase::Check(
 	return result ? e_ok : e_fail;
 }		
 
-bool RadAuthBase::OnSendPDU(
-	RadiusPDU& /*pdu*/,
-	RasPDU<H225_RegistrationRequest>& /*rrqPdu*/,
-	RRQAuthData& /*authData*/
-	)
-{
-	return true;
-}
-
-bool RadAuthBase::OnSendPDU(
-	RadiusPDU& /*pdu*/,
-	RasPDU<H225_AdmissionRequest>& /*arqPdu*/,
-	ARQAuthData& /*authData*/
-	)
-{
-	return true;
-}
-
-bool RadAuthBase::OnSendPDU(
-	RadiusPDU& /*pdu*/,
-	SetupMsg &/*setup*/,
-	SetupAuthData& /*authData*/
-	)
-{
-	return true;
-}
-
-bool RadAuthBase::OnReceivedPDU(
-	RadiusPDU& /*pdu*/,
-	RasPDU<H225_RegistrationRequest>& /*rrqPdu*/,
-	RRQAuthData& /*authData*/
-	)
-{
-	return true;
-}
-
-bool RadAuthBase::OnReceivedPDU(
-	RadiusPDU& /*pdu*/,
-	RasPDU<H225_AdmissionRequest>& /*arqPdu*/,
-	ARQAuthData& /*authData*/
-	)
-{
-	return true;
-}
-
-bool RadAuthBase::OnReceivedPDU(
-	RadiusPDU& /*pdu*/,
-	SetupMsg &/*setup*/,
-	SetupAuthData& /*authData*/
-	)
-{
-	return true;
-}
-
 int RadAuthBase::AppendUsernameAndPassword(
 	RadiusPDU& /*pdu*/,
 	RasPDU<H225_RegistrationRequest>& /*rrqPdu*/,
