@@ -4,7 +4,7 @@
  * RADIUS protocol classes.
  *
  * Copyright (c) 2003, Quarcom FHU, Michal Zygmuntowicz
- * Copyright (c) 2003-2010, Jan Willamowius
+ * Copyright (c) 2003-2011, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -21,6 +21,7 @@
 #include <ptclib/cypher.h>
 #include <ptclib/random.h>
 #include "Toolkit.h"
+#include "h323util.h"
 #include "radproto.h"
 
 // ignore overflow warnings
@@ -29,7 +30,6 @@
 #endif
 
 namespace {
-#if PTRACING
 /// Human-readable attribute names
 const char* const radiusAttributeNames[] =
 {
@@ -55,8 +55,11 @@ const char* const radiusAttributeNames[] =
 /* 76*/	"Prompt", "Connect-Info", "Configuration-Token", "EAP-Message",
 /* 80*/	"Message-Authenticator", "Unknown", "Unknown", "Unknown",
 /* 84*/	"ARAP-Challenge-Response", "Acct-Interim-Interval", "Unknown", "NAS-Port-Id",
-/* 88*/	"Framed-Pool",
-/* 89*/	"Unknown"
+/* 88*/	"Framed-Pool", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown",
+/* RFC 3162 */
+/* 95*/ "NAS-IPv6-Address", "Framed-Interface-Id", "Framed-IPv6-Prefix",
+/* 98*/	"Login-IPv6-Host", "Framed-IPv6-Route", "Framed-IPv6-Pool",
+/*101*/	"Unknown",
 };
 
 /// Human readable RADIUS packet code names
@@ -90,7 +93,6 @@ const char* const PMAP_CODE_TO_NAME(unsigned code)
 		: radiusPacketCodeNames[code];
 }
 
-#endif /* #if PTRACING */
 
 // Cisco VSA attributes together with names and name lengths for fast lookup
 #define CISCO_ATTR_NAME(namestr) namestr, strlen(namestr)
@@ -240,20 +242,30 @@ RadiusAttr::RadiusAttr(
 	SetRadiusInteger(m_value, intValue);
 }
 
+static unsigned _addrLen(const PIPSocket::Address & addr)
+{
+	return (addr.GetVersion() == 6) ? 16 : 4;
+}
+
 RadiusAttr::RadiusAttr( 
 	unsigned char attrType, /// Attribute Type (see #enum AttrTypes#)
-	const PIPSocket::Address& addressValue /// IPv4 address to be stored in the attribute Value
-	) : m_type(attrType), m_length(FixedHeaderLength + 4)
+	const PIPSocket::Address & addressValue /// IP address to be stored in the attribute Value
+	) : m_type(attrType), m_length(FixedHeaderLength + _addrLen(addressValue))
 {
 	if (attrType == VendorSpecific)
 		PAssertAlways(PInvalidParameter);
 
-	const DWORD addr = (DWORD)addressValue;
-	
-	m_value[0] = ((const BYTE*)&addr)[0];
-	m_value[1] = ((const BYTE*)&addr)[1];
-	m_value[2] = ((const BYTE*)&addr)[2];
-	m_value[3] = ((const BYTE*)&addr)[3];
+	if (addressValue.GetVersion() == 6) {
+		for (unsigned i=0; i < 15; i++)
+			m_value[i] = addressValue[i];
+	} else {
+		const DWORD addr = (DWORD)addressValue;
+
+		m_value[0] = ((const BYTE*)&addr)[0];
+		m_value[1] = ((const BYTE*)&addr)[1];
+		m_value[2] = ((const BYTE*)&addr)[2];
+		m_value[3] = ((const BYTE*)&addr)[3];
+	}
 }
 
 RadiusAttr::RadiusAttr( 
@@ -291,21 +303,25 @@ RadiusAttr::RadiusAttr(
 	SetRadiusInteger(m_vendorValue, intValue);
 }
 
-RadiusAttr::RadiusAttr( 
-	const PIPSocket::Address& addressValue, /// IPv4 address to be stored in the attribute Value
+RadiusAttr::RadiusAttr(
+	const PIPSocket::Address& addressValue, /// IP address to be stored in the attribute Value
 	int vendorId, /// 32 bit vendor identifier
 	unsigned char vendorType /// vendor-specific attribute type
-	) : m_type(VendorSpecific), m_length(VsaRfc2865FixedHeaderLength + 4),
-		m_vendorType(vendorType), m_vendorLength(2 + 4)
+	) : m_type(VendorSpecific), m_length(VsaRfc2865FixedHeaderLength + _addrLen(addressValue)),
+		m_vendorType(vendorType), m_vendorLength(2 + _addrLen(addressValue))
 {
 	SetRadiusInteger(m_vendorId, vendorId);
 	
-	const DWORD addr = (DWORD)addressValue;
-	
-	m_vendorValue[0] = ((BYTE*)&addr)[0];
-	m_vendorValue[1] = ((BYTE*)&addr)[1];
-	m_vendorValue[2] = ((BYTE*)&addr)[2];
-	m_vendorValue[3] = ((BYTE*)&addr)[3];
+	if (addressValue.GetVersion() == 6) {
+		for (unsigned i=0; i < 15; i++)
+			m_vendorValue[i] = addressValue[i];
+	} else {
+		const DWORD addr = (DWORD)addressValue;
+		m_vendorValue[0] = ((BYTE*)&addr)[0];
+		m_vendorValue[1] = ((BYTE*)&addr)[1];
+		m_vendorValue[2] = ((BYTE*)&addr)[2];
+		m_vendorValue[3] = ((BYTE*)&addr)[3];
+	}
 }
 
 RadiusAttr::RadiusAttr(
@@ -423,13 +439,9 @@ void RadiusAttr::PrintOn(
 	}
 	
 	strm << "{\n";
-		
-#if PTRACING
+
 	strm << setw(indent+7) << "type = " << (unsigned)m_type
 		<< " (" << PMAP_ATTR_TYPE_TO_NAME(m_type) << ")\n";
-#else
-	strm << setw(indent+7) << "type = " << (unsigned)m_type << '\n';
-#endif
 	const PINDEX totalLen = m_length;
 	
 	strm << setw(indent+9) << "length = " << totalLen << " octets\n";
@@ -576,14 +588,16 @@ PIPSocket::Address RadiusAttr::AsAddress() const
 	if (m_length < (FixedHeaderLength+4) || m_type == VendorSpecific)
 		return 0;
 
-	DWORD addr = 0;
-	
-	((BYTE*)&addr)[0] = m_value[0];
-	((BYTE*)&addr)[1] = m_value[1];
-	((BYTE*)&addr)[2] = m_value[2];
-	((BYTE*)&addr)[3] = m_value[3];
-	
-	return addr;
+	if (m_length == (FixedHeaderLength+16)) {
+		return PIPSocket::Address(16, (const BYTE*)m_value);
+	} else {
+		DWORD addr = 0;
+		((BYTE*)&addr)[0] = m_value[0];
+		((BYTE*)&addr)[1] = m_value[1];
+		((BYTE*)&addr)[2] = m_value[2];
+		((BYTE*)&addr)[3] = m_value[3];
+		return addr;
+	}
 }
 
 PString RadiusAttr::AsVsaString() const
@@ -639,14 +653,16 @@ PIPSocket::Address RadiusAttr::AsVsaAddress() const
 	if (m_length < (VsaRfc2865FixedHeaderLength+4) || m_type != VendorSpecific)
 		return 0;
 	
-	DWORD addr = 0;
-	
-	((BYTE*)&addr)[0] = m_vendorValue[0];
-	((BYTE*)&addr)[1] = m_vendorValue[1];
-	((BYTE*)&addr)[2] = m_vendorValue[2];
-	((BYTE*)&addr)[3] = m_vendorValue[3];
-	
-	return addr;
+	if (m_length == (VsaRfc2865FixedHeaderLength + 16)) {
+		return PIPSocket::Address(16, (const BYTE*)m_vendorValue);
+	} else {
+		DWORD addr = 0;
+		((BYTE*)&addr)[0] = m_vendorValue[0];
+		((BYTE*)&addr)[1] = m_vendorValue[1];
+		((BYTE*)&addr)[2] = m_vendorValue[2];
+		((BYTE*)&addr)[3] = m_vendorValue[3];
+		return addr;
+	}
 }
 
 
@@ -655,9 +671,7 @@ RadiusPDU::RadiusPDU() : m_code(Invalid), m_id(0)
 	SetLength(FixedHeaderLength);
 }
 
-RadiusPDU::RadiusPDU( 
-	const RadiusPDU& pdu 
-	)
+RadiusPDU::RadiusPDU(const RadiusPDU & pdu)
 {
 	CopyContents(pdu);
 }
@@ -671,7 +685,7 @@ RadiusPDU::RadiusPDU(
 }
 
 RadiusPDU::RadiusPDU(
-	const void* rawData, /// raw data buffer
+	const void * rawData, /// raw data buffer
 	PINDEX rawLength /// raw data length
 	)
 {
@@ -688,13 +702,9 @@ void RadiusPDU::PrintOn(
 	const std::streamsize indent = strm.precision() + 2;
 
 	strm << ((!IsValid()) ? "(Invalid) {\n" : "{\n");
-	
-#if PTRACING
+
 	strm << setw(indent+7) << "code = " << (unsigned)m_code
 		<< " (" << PMAP_CODE_TO_NAME(m_code) << ")\n";
-#else
-	strm << setw(indent+7) << "code = " << (unsigned)m_code << '\n';
-#endif
 	strm << setw(indent+5) << "id = " << (unsigned)m_id << '\n';
 	strm << setw(indent+9) << "length = " << GetLength() << " octets\n";
 
@@ -754,7 +764,7 @@ bool RadiusPDU::IsValid() const
 	return currLen == len;
 }
 
-void RadiusPDU::GetAuthenticator(PBYTEArray& vector, PINDEX offset) const
+void RadiusPDU::GetAuthenticator(PBYTEArray & vector, PINDEX offset) const
 {
 	if (offset == P_MAX_INDEX)
 		offset = vector.GetSize();
@@ -763,7 +773,7 @@ void RadiusPDU::GetAuthenticator(PBYTEArray& vector, PINDEX offset) const
 		);
 }
 
-bool RadiusPDU::SetAuthenticator(const PBYTEArray& vector, PINDEX offset)
+bool RadiusPDU::SetAuthenticator(const PBYTEArray & vector, PINDEX offset)
 {
 	PINDEX len = vector.GetSize();
 	if (offset >= len)
@@ -779,7 +789,7 @@ bool RadiusPDU::SetAuthenticator(const PBYTEArray& vector, PINDEX offset)
 	return true;
 }
 
-bool RadiusPDU::SetAuthenticator(const void* data)
+bool RadiusPDU::SetAuthenticator(const void * data)
 {
 #ifdef _DEBUG
 	PAssertNULL(data);
@@ -791,7 +801,7 @@ bool RadiusPDU::SetAuthenticator(const void* data)
 	return true;
 }
 
-void RadiusPDU::SetAuthenticator(PRandom& random)
+void RadiusPDU::SetAuthenticator(PRandom & random)
 {
 	DWORD r = (DWORD)random;
 	m_authenticator[0] = ((const BYTE*)&r)[0];
@@ -816,8 +826,8 @@ void RadiusPDU::SetAuthenticator(PRandom& random)
 }
 
 void RadiusPDU::SetAuthenticator( 
-	const PString& secret,
-	PMessageDigest5& md5 
+	const PString & secret,
+	PMessageDigest5 & md5 
 	)
 {
 	if (m_code == AccountingRequest) {
@@ -841,7 +851,7 @@ void RadiusPDU::SetAuthenticator(
 }
 
 bool RadiusPDU::AppendAttr( 
-	const RadiusAttr& attr /// attribute to be appended
+	const RadiusAttr & attr /// attribute to be appended
 	)
 {
 	const PINDEX len = GetLength();
@@ -915,22 +925,27 @@ bool RadiusPDU::AppendAttr(
 
 bool RadiusPDU::AppendAttr( 
 	unsigned char attrType, /// Attribute Type
-	const PIPSocket::Address& addressValue /// IPv4 address to be stored in the attribute Value
+	const PIPSocket::Address & addressValue /// IP address to be stored in the attribute Value
 	)
 {
 	const PINDEX len = GetLength();
-	const PINDEX attrLen = RadiusAttr::FixedHeaderLength + 4;
+	const PINDEX attrLen = RadiusAttr::FixedHeaderLength + _addrLen(addressValue);
 	if (attrLen > RadiusAttr::MaxLength || (len + attrLen) > MaxPduLength)
 		return false;
-		
-	const DWORD addr = (DWORD)addressValue;
+
 	RadiusAttr* const attr = reinterpret_cast<RadiusAttr*>(m_data + len);
 	attr->m_type = attrType;
 	attr->m_length = attrLen;
-	attr->m_value[0] = ((const BYTE*)&addr)[0];
-	attr->m_value[1] = ((const BYTE*)&addr)[1];
-	attr->m_value[2] = ((const BYTE*)&addr)[2];
-	attr->m_value[3] = ((const BYTE*)&addr)[3];
+	if (addressValue.GetVersion() == 6) {
+		for (unsigned i=0; i < 15; i++)
+			attr->m_value[i] = addressValue[i];
+	} else {
+		const DWORD addr = (DWORD)addressValue;
+		attr->m_value[0] = ((const BYTE*)&addr)[0];
+		attr->m_value[1] = ((const BYTE*)&addr)[1];
+		attr->m_value[2] = ((const BYTE*)&addr)[2];
+		attr->m_value[3] = ((const BYTE*)&addr)[3];
+	}
 	SetLength(len + attrLen);
 	
 	return true;
@@ -1008,27 +1023,32 @@ bool RadiusPDU::AppendVsaAttr(
 }
 
 bool RadiusPDU::AppendVsaAttr( 
-	const PIPSocket::Address& addressValue, /// IPv4 address to be stored in the attribute Value
+	const PIPSocket::Address& addressValue, /// IP address to be stored in the attribute Value
 	int vendorId, /// 32 bit vendor identifier
 	unsigned char vendorType /// vendor-specific attribute type
 	)
 {
 	const PINDEX len = GetLength();
-	const PINDEX attrLen = RadiusAttr::VsaRfc2865FixedHeaderLength + 4;
+	const PINDEX attrLen = RadiusAttr::VsaRfc2865FixedHeaderLength + _addrLen(addressValue);
 	if (attrLen > RadiusAttr::MaxLength || (len + attrLen) > MaxPduLength)
 		return false;
-		
-	const DWORD addr = (DWORD)addressValue;
+
 	RadiusAttr* const attr = reinterpret_cast<RadiusAttr*>(m_data + len);
 	attr->m_type = RadiusAttr::VendorSpecific;
 	attr->m_length = attrLen;
 	SetRadiusInteger(attr->m_vendorId, vendorId);
 	attr->m_vendorType = vendorType;
-	attr->m_vendorLength = 4 + 2;
-	attr->m_vendorValue[0] = ((const BYTE*)&addr)[0];
-	attr->m_vendorValue[1] = ((const BYTE*)&addr)[1];
-	attr->m_vendorValue[2] = ((const BYTE*)&addr)[2];
-	attr->m_vendorValue[3] = ((const BYTE*)&addr)[3];
+	attr->m_vendorLength = _addrLen(addressValue) + 2;
+	if (addressValue.GetVersion() == 6) {
+		for (unsigned i=0; i < 15; i++)
+			attr->m_vendorValue[i] = addressValue[i];
+	} else {
+		const DWORD addr = (DWORD)addressValue;
+		attr->m_vendorValue[0] = ((const BYTE*)&addr)[0];
+		attr->m_vendorValue[1] = ((const BYTE*)&addr)[1];
+		attr->m_vendorValue[2] = ((const BYTE*)&addr)[2];
+		attr->m_vendorValue[3] = ((const BYTE*)&addr)[3];
+	}
 	SetLength(len + attrLen);
 	
 	return true;
@@ -1096,9 +1116,7 @@ PINDEX RadiusPDU::GetNumAttributes() const
 	return count;
 }
 
-const RadiusAttr* RadiusPDU::GetAttr(
-	const RadiusAttr* prevAttr
-	) const
+const RadiusAttr* RadiusPDU::GetAttr(const RadiusAttr * prevAttr) const
 {
 	const PINDEX len = GetLength();
 	PINDEX offset = FixedHeaderLength;
@@ -1219,8 +1237,8 @@ void RadiusPDU::CopyContents(const RadiusPDU& pdu)
 }
 
 bool RadiusPDU::EncryptPasswords( 
-	const PString& secret,
-	PMessageDigest5& md5 
+	const PString & secret,
+	PMessageDigest5 & md5 
 	)
 {
 	RadiusAttr* const pwdAttr = const_cast<RadiusAttr*>(FindAttr(RadiusAttr::UserPassword));
@@ -1329,13 +1347,13 @@ bool RadiusPDU::EncryptPasswords(
 #define DEFAULT_PERMANENT_SYNCPOINTS 8
 #endif
 
-RadiusSocket::RadiusSocket( 
-	WORD _port 
+RadiusSocket::RadiusSocket(
+	WORD _port
 	) : m_permanentSyncPoints(DEFAULT_PERMANENT_SYNCPOINTS),
-	m_isReading(false), m_nestedCount(0), 
+	m_isReading(false), m_nestedCount(0),
 	m_idCacheTimeout(RadiusClient::DefaultIdCacheTimeout)
 {
-	if (!Listen(0, _port)) {
+	if (!Listen(GNUGK_INADDR_ANY, 0, _port)) {
 		PTRACE(1, "RADIUS\tCould not bind socket to the port " << _port
 			<< " - error " << GetErrorCode(PSocket::LastGeneralError) << '/'
 			<< GetErrorNumber(PSocket::LastGeneralError) << ": " 
@@ -1343,7 +1361,10 @@ RadiusSocket::RadiusSocket(
 			);
 		Close();
 	}
-	
+	m_addr = GNUGK_INADDR_ANY;
+	m_port = _port;
+	Toolkit::Instance()->PortNotification(RadiusPort, PortOpen, "udp", GNUGK_INADDR_ANY, _port);
+
 	PRandom random;
 	const unsigned _id_ = random;
 	m_oldestId = m_nextId = (BYTE)(_id_^(_id_>>8)^(_id_>>16)^(_id_>>24));
@@ -1363,22 +1384,25 @@ RadiusSocket::RadiusSocket(
 	}
 }
 
-RadiusSocket::RadiusSocket( 
-	const PIPSocket::Address& addr, 
+RadiusSocket::RadiusSocket(
+	const PIPSocket::Address & addr,
 	WORD _port
 	) : m_permanentSyncPoints(DEFAULT_PERMANENT_SYNCPOINTS),
 	m_isReading(false), m_nestedCount(0),
 	m_idCacheTimeout(RadiusClient::DefaultIdCacheTimeout)
 {
 	if (!Listen(addr, 0, _port)) {
-		PTRACE(1, "RADIUS\tCould not bind socket to " << addr << ':' << _port
+		PTRACE(1, "RADIUS\tCould not bind socket to " << AsString(addr, _port)
 			<< " - error " << GetErrorCode(PSocket::LastGeneralError) << '/'
 			<< GetErrorNumber(PSocket::LastGeneralError) << ": " 
 			<< GetErrorText(PSocket::LastGeneralError)
 			);
 		Close();
 	}
-	
+	m_addr = addr;
+	m_port = _port;
+	Toolkit::Instance()->PortNotification(RadiusPort, PortOpen, "udp", addr, _port);
+
 	PRandom random;
 	const unsigned _id_ = random;
 	m_oldestId = m_nextId = (BYTE)(_id_^(_id_>>8)^(_id_>>16)^(_id_>>24));
@@ -1401,8 +1425,10 @@ RadiusSocket::RadiusSocket(
 
 RadiusSocket::~RadiusSocket()
 {
+	Toolkit::Instance()->PortNotification(RadiusPort, PortClose, "udp", m_addr, m_port);
+
 	PWaitAndSignal lock(m_readMutex);
-	
+
 	for (int i = 0; i < 256; i++)
 		delete m_readSyncPoints[i];
 }
@@ -1417,26 +1443,26 @@ void RadiusSocket::PrintOn(ostream& strm) const
 PINDEX RadiusSocket::AllocReadSyncPoint()
 {
 	PINDEX idx = 0;
-	
+
 	for (PINDEX k = 0; k < 8; k++)
 		if (m_syncPointMap[k] != 0xffffffff) {
 			for (PINDEX i = 0, j = 1; i < 32; i++, j <<= 1, idx++)
 				if ((m_syncPointMap[k] & ((DWORD)j)) == 0) {
 					m_syncPointMap[k] |= (DWORD)j;
-					if (m_readSyncPoints[idx] == NULL )
+					if (m_readSyncPoints[idx] == NULL)
 						m_readSyncPoints[idx] = new PSyncPoint();
 					return idx;
 				}
 		} else
 			idx += 32;
-			
+
 	return P_MAX_INDEX;
 }
 
 void RadiusSocket::FreeReadSyncPoint(PINDEX syncPointIndex)
 {
 	if (syncPointIndex < 256 && syncPointIndex >= 0) {
-		m_syncPointMap[(syncPointIndex >> 5) & 7] 
+		m_syncPointMap[(syncPointIndex >> 5) & 7]
 			&= ~(DWORD)(((DWORD)1)<<(syncPointIndex & 31));
 		if (syncPointIndex >= m_permanentSyncPoints) {
 			delete m_readSyncPoints[syncPointIndex];
@@ -1445,38 +1471,34 @@ void RadiusSocket::FreeReadSyncPoint(PINDEX syncPointIndex)
 	}
 }
 
-bool RadiusSocket::MakeRequest( 
-	const RadiusPDU* request, 
-	const Address& serverAddress, 
+bool RadiusSocket::MakeRequest(
+	const RadiusPDU* request,
+	const Address& serverAddress,
 	WORD serverPort,
 	RadiusPDU*& pdu
 	)
 {
 	if (!IsOpen() || request == NULL || !request->IsValid())
 		return false;
-	
-	const PINDEX length = request->GetLength();	
+
+	const PINDEX length = request->GetLength();
 	const unsigned char id = request->GetId();
 	const PTimeInterval timeout = GetReadTimeout();
 	const PTime startTime;
 	bool shouldRead = false;
 	PSyncPoint* syncPoint = NULL;
 	RadiusRequest* requestInfo = NULL;
-	
+
 	{
 		if (!m_readMutex.Wait(timeout)) {
-			PTRACE(4, "RADIUS\tMutex timed out for the request (id:" 
-				<< (PINDEX)id << ')'
-				);
+			PTRACE(4, "RADIUS\tMutex timed out for the request (id:" << (PINDEX)id << ')');
 			return false;
 		}
 
 		PWaitAndSignal lock(m_readMutex, FALSE);
 
 		if (m_pendingRequests[id] != NULL) {
-			PTRACE(1, "RADIUS\tDuplicate RADIUS socket request (id:" 
-				<< (PINDEX)id << ')'
-				);
+			PTRACE(1, "RADIUS\tDuplicate RADIUS socket request (id:" << (PINDEX)id << ')');
 			return false;
 		}
 
@@ -1485,23 +1507,19 @@ bool RadiusSocket::MakeRequest(
 		else {
 			const PINDEX index = AllocReadSyncPoint();
 			if (index == P_MAX_INDEX) {
-				PTRACE(1, "RADIUS\tFailed to allocate a new mutex for "
-					"the request (id:" << (PINDEX)id << ')'
-					);
+				PTRACE(1, "RADIUS\tFailed to allocate a new mutex for the request (id:" << (PINDEX)id << ')');
 				return false;
 			}
 			syncPoint = m_readSyncPoints[index];
 			if (syncPoint == NULL) {
-				PTRACE(1, "RADIUS\tFailed to allocate a new mutex for "
-					"the request (id:" << (PINDEX)id << ')'
-					);
+				PTRACE(1, "RADIUS\tFailed to allocate a new mutex for the request (id:" << (PINDEX)id << ')');
 				FreeReadSyncPoint(index);
 				return false;
 			}
 			m_readSyncPointIndices[id] = index;
 			m_nestedCount++;
 		}
-		
+	
 		requestInfo = m_pendingRequests[id] 
 			= new RadiusRequest(request, pdu, &serverAddress, serverPort);
 	}
@@ -1512,23 +1530,20 @@ bool RadiusSocket::MakeRequest(
 		PTRACE(5, "RADIUS\tError sending UDP packet ("
 			<< GetErrorCode(LastWriteError) << '/'
 			<< GetErrorNumber(LastWriteError) << ": "
-			<< GetErrorText(LastWriteError) << " (id:" << (PINDEX)id << ')'
-			);
+			<< GetErrorText(LastWriteError) << " (id:" << (PINDEX)id << ')');
 	}
 	m_writeMutex.Signal();
-	
+
 	if (result)
 	do {
 		result = FALSE;
-		
+
 		if (shouldRead) {
 			PIPSocket::Address remoteAddress;
 			WORD remotePort;
 
 			RadiusPDU* response = new RadiusPDU();
-			result = ReadFrom(response, sizeof(RadiusPDU), 
-				remoteAddress, remotePort 
-				);
+			result = ReadFrom(response, sizeof(RadiusPDU), remoteAddress, remotePort);
 			if (!result) {
 				if (GetErrorCode(LastReadError) == Timeout)
 					PTRACE(6, "RADIUS\tTimed out reading socket " << *this);
@@ -1536,44 +1551,39 @@ bool RadiusSocket::MakeRequest(
 					PTRACE(5, "RADIUS\tError reading socket " << *this
 						<< " (" << GetErrorCode(LastReadError) << '/'
 						<< GetErrorNumber(LastReadError) << ": "
-						<< GetErrorText(LastReadError) << ')'
-						);
+						<< GetErrorText(LastReadError) << ')');
 				delete response;
 				response = NULL;
 				break;
 			}
-			
+
 			result = FALSE;
-				
+
 			PINDEX bytesRead = GetLastReadCount();
 
 			if (bytesRead < RadiusPDU::MinPduLength) {
-				PTRACE(5, "RADIUS\tReceived packet is too small ("
-					<< bytesRead << ')'
-					);
+				PTRACE(5, "RADIUS\tReceived packet is too small ("<< bytesRead << ')');
 				delete response;
 				response = NULL;
 				continue;
 			}
-	
+
 			if (!response->IsValid()) {
 				PTRACE(5, "RADIUS\tReceived packet is not a valid Radius PDU");
 				delete response;
 				response = NULL;
 				continue;
 			}
-				
+
 			const BYTE newId = response->GetId();
-			
+
 			if (!m_readMutex.Wait(timeout)) {
-				PTRACE(5, "RADIUS\tTimed out (mutex) - dropping PDU (id:"
-					<< (PINDEX)newId
-					);
+				PTRACE(5, "RADIUS\tTimed out (mutex) - dropping PDU (id:" << (PINDEX)newId);
 				delete response;
 				response = NULL;
 				continue;
 			}
-				
+
 			PWaitAndSignal lock(m_readMutex, FALSE);
 
 			if (m_pendingRequests[newId] == NULL) {
@@ -1588,18 +1598,16 @@ bool RadiusSocket::MakeRequest(
 
 			if (remoteAddress != *(m_pendingRequests[newId]->m_addr) 
 				|| remotePort != m_pendingRequests[newId]->m_port) {
-				PTRACE(5, "RADIUS\tReceived PDU from unknown address: "
-					<< remoteAddress << ':' << remotePort
-					);
+				PTRACE(5, "RADIUS\tReceived PDU from unknown address: " << AsString(remoteAddress, remotePort));
 				delete response;
 				response = NULL;
 				continue;
 			}
-			
+
 			m_pendingRequests[newId]->m_response = response;
 			m_pendingRequests[newId] = NULL;
 			response = NULL;
-					
+
 			if (newId == id) {
 				m_isReading = false;
 
@@ -1611,7 +1619,7 @@ bool RadiusSocket::MakeRequest(
 							m_readSyncPoints[m_readSyncPointIndices[j] & 0xff]->Signal();
 							break;
 						}
-					
+
 				delete requestInfo;
 				requestInfo = NULL;
 				return true;
@@ -1701,7 +1709,7 @@ bool RadiusSocket::SendRequest(
 	PWaitAndSignal lock(m_writeMutex);
 	if (WriteTo(request, request->GetLength(), serverAddress, serverPort))
 		return true;
-		
+
 	PTRACE(5, "RADIUS\tError sending UDP packet ("
 		<< GetErrorCode(LastWriteError) << '/'
 		<< GetErrorNumber(LastWriteError) << ": "
@@ -1711,9 +1719,7 @@ bool RadiusSocket::SendRequest(
 	return false;
 }
 
-void RadiusSocket::RefreshIdCache(
-	const time_t now
-	)
+void RadiusSocket::RefreshIdCache(const time_t now)
 {
 	const PINDEX lastId = ((m_nextId >= m_oldestId) 
 		? m_nextId : ((PINDEX)m_nextId + 256));
@@ -1755,7 +1761,7 @@ RadiusClient::RadiusClient(
 	m_idCacheTimeout(DefaultIdCacheTimeout),
 	m_socketDeleteTimeout(DefaultSocketDeleteTimeout),
 	m_numRetries(DefaultRetries), m_roundRobinServers(false),
-	m_localAddress(INADDR_ANY)
+	m_localAddress(GNUGK_INADDR_ANY)
 {
 	GetServersFromString(servers);
 	
@@ -1768,7 +1774,6 @@ RadiusClient::RadiusClient(
 		}
 	}
 
-#if PTRACING
 	if (PTrace::CanTrace(4)) {
 		ostream& s = PTrace::Begin(4, __FILE__, __LINE__);
 		const std::streamsize indent = s.precision() + 2;
@@ -1783,11 +1788,10 @@ RadiusClient::RadiusClient(
 				<< ')';
 		PTrace::End(s);
 	}
-#endif
 }
 
 RadiusClient::RadiusClient( 
-	PConfig& config, /// config that contains RADIUS settings
+	PConfig & config, /// config that contains RADIUS settings
 	const PString& sectionName /// config section with the settings
 	)
 	: m_sharedSecret(Toolkit::Instance()->ReadPassword(sectionName, "SharedSecret")),
@@ -1806,7 +1810,7 @@ RadiusClient::RadiusClient(
 		DefaultRetries)),
 	m_roundRobinServers(config.GetBoolean(
 		sectionName, "RoundRobinServers", TRUE)),
-	m_localAddress(INADDR_ANY)
+	m_localAddress(GNUGK_INADDR_ANY)
 {
 	GetServersFromString(config.GetString(sectionName, "Servers", ""));
 		
@@ -1846,7 +1850,6 @@ RadiusClient::RadiusClient(
 		}
 	}
 
-#if PTRACING
 	if (PTrace::CanTrace(4)) {
 		ostream& os = PTrace::Begin(4, __FILE__, __LINE__);
 		const std::streamsize indent = os.precision() + 2;
@@ -1861,7 +1864,6 @@ RadiusClient::RadiusClient(
 				<< ')';
 		PTrace::End(os);
 	}
-#endif
 }
 
 RadiusClient::~RadiusClient()
@@ -1878,13 +1880,25 @@ RadiusClient::~RadiusClient()
 	m_radiusServers.clear();
 }
 
-void RadiusClient::GetServersFromString(
-	const PString& servers
-	)
+void RadiusClient::GetServersFromString(const PString & servers)
 {
 	const PStringArray tokens = servers.Tokenise(" ;,", FALSE);
 	for (PINDEX i = 0; i < tokens.GetSize(); i++) {
-		const PStringArray serverTokens = tokens[i].Tokenise(":");
+		PStringArray serverTokens;
+		if (tokens[i].Left(1) == "[") {
+			// IPv6 address
+			PINDEX ip_end = tokens[i].Find(']');
+			serverTokens.SetSize(1);
+			serverTokens[0] = tokens[i].Mid(1, ip_end-1);
+			const PStringArray detailsTokens = tokens[i].Mid(ip_end+1).Tokenise(":", FALSE);
+			for (PINDEX j=0; j < detailsTokens.GetSize(); ++j) {
+				serverTokens.SetSize(serverTokens.GetSize()+1);
+				serverTokens[j+1] = detailsTokens[j];
+			}
+		} else {
+			// IPv4 or DNS names
+			serverTokens = tokens[i].Tokenise(":", FALSE);
+		}
 		if (serverTokens.GetSize() > 0) {
 			const PString serverAddress = serverTokens[0].Trim();
 			if (!serverAddress) {
@@ -1905,7 +1919,7 @@ void RadiusClient::GetServersFromString(
 }
 
 bool RadiusClient::SetIdCacheTimeout( 
-	const PTimeInterval& timeout /// new time interval
+	const PTimeInterval & timeout /// new time interval
 	)
 {
 	PWaitAndSignal lock(m_socketMutex);
@@ -1924,8 +1938,8 @@ bool RadiusClient::SetIdCacheTimeout(
 }
 
 bool RadiusClient::MakeRequest( 
-	const RadiusPDU& requestPDU, /// PDU with request packet
-	RadiusPDU*& responsePDU /// filled with PDU received from RADIUS server
+	const RadiusPDU & requestPDU, /// PDU with request packet
+	RadiusPDU * & responsePDU /// filled with PDU received from RADIUS server
 	)
 {
 	if (!requestPDU.IsValid())
@@ -1935,7 +1949,7 @@ bool RadiusClient::MakeRequest(
 	RadiusSocket* socket = NULL;
 	unsigned char id;
 	const unsigned numServers = m_radiusServers.size();
-	const PString* secret = NULL;
+	const PString * secret = NULL;
 	
 	for (unsigned i = 0; i < (m_roundRobinServers ? m_numRetries * numServers : numServers); i++)
 	{
@@ -1956,7 +1970,7 @@ bool RadiusClient::MakeRequest(
 		PIPSocket::Address serverAddress;
 		if (!PIPSocket::GetHostAddress(server->m_serverAddress, serverAddress)
 			|| !serverAddress.IsValid()) {
-			PTRACE(3, "RADIUS\tCould not get IPv4 address for RADIUS server "
+			PTRACE(3, "RADIUS\tCould not get IP address for RADIUS server "
 				"host: " << server->m_serverAddress
 				);
 			continue;
@@ -1964,13 +1978,9 @@ bool RadiusClient::MakeRequest(
 
 		for (unsigned j = 0; j < (m_roundRobinServers ? 1 : m_numRetries); j++) {
  			RadiusPDU* const clonedRequestPDU = new RadiusPDU(requestPDU);
-			
+
 			bool requireNewId = false;
-			if (!OnSendPDU(*clonedRequestPDU, retransmission, requireNewId)) {
-				delete clonedRequestPDU;
-				return false;
-			}
-				
+
 			if (secretChanged || requireNewId || !retransmission)
 				if (!GetSocket(socket, id)) {
 					PTRACE(3, "RADIUS\tSocket allocation failed");
@@ -1992,11 +2002,10 @@ bool RadiusClient::MakeRequest(
 				return false;
 			}
 	
-#if PTRACING
 			if( PTrace::CanTrace(3) ) {
 				ostream& strm = PTrace::Begin(3, __FILE__, __LINE__);
 				strm << "RADIUS\tSending PDU to RADIUS server "
-					<< server->m_serverAddress << " (" << serverAddress << ':' << serverPort
+					<< server->m_serverAddress << " (" << AsString(serverAddress, serverPort)
 					<< ')' << " from " << (*socket) << ", PDU: ";
 				if( PTrace::CanTrace(5) )
 					strm << *clonedRequestPDU;
@@ -2005,7 +2014,6 @@ bool RadiusClient::MakeRequest(
 						<< ", id " << (PINDEX)(clonedRequestPDU->GetId());
 				PTrace::End(strm);
 			}
-#endif
 			RadiusPDU* response = NULL;
 			
 			retransmission = true;
@@ -2031,11 +2039,10 @@ bool RadiusClient::MakeRequest(
 
 			delete clonedRequestPDU;
 
-#if PTRACING
 			if (PTrace::CanTrace(3)) {
 				ostream& strm = PTrace::Begin(3, __FILE__, __LINE__);
 				strm << "RADIUS\tReceived PDU from RADIUS server "
-					<< server->m_serverAddress << " (" << serverAddress << ':' << serverPort
+					<< server->m_serverAddress << " (" << AsString(serverAddress, serverPort)
 					<< ')' << " by socket " << (*socket) << ", PDU: ";
 				if (PTrace::CanTrace(5))
 					strm << (*response);
@@ -2044,13 +2051,7 @@ bool RadiusClient::MakeRequest(
 						<< (PINDEX)(response->GetId());
 				PTrace::End(strm);
 			}
-#endif
-			if (!OnReceivedPDU(*response)) {
-				delete response;
-				response = NULL;
-				continue;
-			}
-			
+
 			responsePDU = response;
 			return true;
 		}
@@ -2060,7 +2061,7 @@ bool RadiusClient::MakeRequest(
 }
 
 bool RadiusClient::SendRequest( 
-	const RadiusPDU& requestPDU /// PDU with request packet
+	const RadiusPDU & requestPDU /// PDU with request packet
 	)
 {
 	if (!requestPDU.IsValid())
@@ -2085,20 +2086,14 @@ bool RadiusClient::SendRequest(
 	PIPSocket::Address serverAddress;
 	if (!PIPSocket::GetHostAddress(server->m_serverAddress, serverAddress)
 			|| !serverAddress.IsValid()) {
-		PTRACE(3, "RADIUS\tCould not get IPv4 address for RADIUS server host: "
+		PTRACE(3, "RADIUS\tCould not get IP address for RADIUS server host: "
 			<< server->m_serverAddress
 			);
 		return false;
 	}
 
 	clonedRequestPDU = new RadiusPDU(requestPDU);
-	
-	bool dummy;
-	if (!OnSendPDU(*clonedRequestPDU, false, dummy)) {
-		delete clonedRequestPDU;
-		return false;
-	}
-				
+
 	if (!GetSocket(socket, id)) {
 		PTRACE(3, "RADIUS\tSocket allocation failed");
 		delete clonedRequestPDU;
@@ -2116,12 +2111,11 @@ bool RadiusClient::SendRequest(
 		delete clonedRequestPDU;
 		return false;
 	}
-	
-#if PTRACING
+
 	if (PTrace::CanTrace(3)) {
 		ostream& strm = PTrace::Begin(3, __FILE__, __LINE__);
 		strm << "RADIUS\tSending PDU to RADIUS server "
-			<< server->m_serverAddress << " (" << serverAddress << ':' << serverPort
+			<< server->m_serverAddress << " (" << AsString(serverAddress, serverPort)
 			<< ')' << " from " << (*socket) << ", PDU: ";
 		if (PTrace::CanTrace(5))
 			strm << *clonedRequestPDU;
@@ -2130,7 +2124,6 @@ bool RadiusClient::SendRequest(
 				<< (PINDEX)(clonedRequestPDU->GetId());
 		PTrace::End(strm);
 	}
-#endif
 
 	if (!socket->SendRequest(clonedRequestPDU, serverAddress, serverPort)) {
 		PTRACE(3, "RADIUS\tError sending RADIUS request (id:" << (PINDEX)id << ')');
@@ -2171,23 +2164,7 @@ bool RadiusClient::VerifyResponseAuthenticator(
 			RadiusPDU::AuthenticatorLength) == 0;
 }
 
-bool RadiusClient::OnSendPDU( 
-	RadiusPDU& /*pdu*/,
-	bool /*retransmission*/,
-	bool& /*requireNewId*/
-	)
-{
-	return true;
-}
-
-bool RadiusClient::OnReceivedPDU( 
-	RadiusPDU& /*pdu*/
-	)
-{
-	return true;
-}
-
-bool RadiusClient::IsAcctPDU(const RadiusPDU& pdu) const
+bool RadiusClient::IsAcctPDU(const RadiusPDU & pdu) const
 {
 	const unsigned char c = pdu.GetCode();
 	return (c == RadiusPDU::AccountingRequest) 
@@ -2253,7 +2230,7 @@ bool RadiusClient::GetSocket(RadiusSocket*& socket, unsigned char& id)
 			delete newSocket;
 			newSocket = NULL;
 
-			if (m_localAddress == INADDR_ANY)
+			if (m_localAddress == GNUGK_INADDR_ANY)
 				newSocket = CreateSocket((WORD)(m_portBase + portIndex));
 			else
 				newSocket = CreateSocket(m_localAddress, (WORD)(m_portBase + portIndex));
@@ -2264,7 +2241,7 @@ bool RadiusClient::GetSocket(RadiusSocket*& socket, unsigned char& id)
 			delete newSocket;
 			newSocket = NULL;
 
-			if (m_localAddress == INADDR_ANY)
+			if (m_localAddress == GNUGK_INADDR_ANY)
 				newSocket = CreateSocket(p);
 			else
 				newSocket = CreateSocket(m_localAddress, p);
@@ -2297,17 +2274,12 @@ bool RadiusClient::GetSocket(RadiusSocket*& socket, unsigned char& id)
 	return true;
 }
 
-RadiusSocket* RadiusClient::CreateSocket( 
-	const PIPSocket::Address& addr, 
-	WORD port
-	)
+RadiusSocket* RadiusClient::CreateSocket(const PIPSocket::Address & addr, WORD port)
 {
 	return new RadiusSocket(addr, port);
 }
 	
-RadiusSocket* RadiusClient::CreateSocket( 
-	WORD port
-	)
+RadiusSocket* RadiusClient::CreateSocket(WORD port)
 {
 	return new RadiusSocket(port);
 }
