@@ -5,7 +5,7 @@
  * scalability and multithreading access. 
  *
  * Copyright (c) 2003, Quarcom FHU, Michal Zygmuntowicz
- * Copyright (c) 2003-2010, Jan Willamowius
+ * Copyright (c) 2003-2011, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -17,7 +17,7 @@
 #if HAS_RADIUS
 
 #ifndef __RADPROTO_H
-#define __RADPROTO_H "@(#) $Id: radproto.h,v 1.25 2011/02/10 22:43:56 willamowius Exp $"
+#define __RADPROTO_H "@(#) $Id: radproto.h,v 1.30 2011/12/07 22:42:32 willamowius Exp $"
 
 #include <vector>
 #include <ptlib/sockets.h>
@@ -72,7 +72,10 @@ public:
 		AcctTunnelConnectionId = 68, TunnelPassword, 
 		PasswordRetry = 75, 
 		Prompt = 76, ConnectInfo, ConfigurationToken,
-		AcctInterimInterval = 85, AcctTunnelPacketsLost, NasPortId
+		AcctInterimInterval = 85, AcctTunnelPacketsLost, NasPortId,
+		// RFC 3162
+		NasIpv6Address = 95, FramedInterfaceId, FramedIpv6Prefix,
+		LoginIpv6Host = 98, FramedIpv6Route, FramedIpv6Pool
 	};
 
 	/// Constants for Service-Type attribute values
@@ -191,12 +194,12 @@ public:
 		);
 
 	/** Create TLV RADIUS attribute of a given type,
-		initializing #value# field with 32 bit IPv4 address
+		initializing #value# field with 32 bit IPv4 or 128 bit IPv6 address
 		value passed with 'addressValue' parameter.
 	*/
 	RadiusAttr( 
 		unsigned char attrType, /// Attribute Type (see #enum AttrTypes#)
-		const PIPSocket::Address& addressValue /// IPv4 address to be stored in the attribute Value
+		const PIPSocket::Address& addressValue /// IP address to be stored in the attribute Value
 		);
 
 	/** Create TLV RADIUS vendor-specific (26) attribute of a given 
@@ -242,7 +245,7 @@ public:
 		#vsaVendorId# is set to 'vendorId'.
 	*/
 	RadiusAttr( 
-		const PIPSocket::Address& addressValue, /// IPv4 address to be stored in the attribute Value
+		const PIPSocket::Address& addressValue, /// IP address to be stored in the attribute Value
 		int vendorId, /// 32 bit vendor identifier
 		unsigned char vendorType /// vendor-specific attribute type
 		);
@@ -413,23 +416,23 @@ public:
 	*/
 	time_t AsVsaTime() const { return (time_t)AsVsaInteger(); }
 
-	/** Get attribute Value as a 32 bit IPv4 address. 
-		For VSA attributes this call will build the IPv4 address
-		from 4 bytes of data contained after vendorId field.
+	/** Get attribute Value as IPv4 or IPv6 address. 
+		For VSA attributes this call will build the IPv4 or IPv6 address
+		from 4 / 16 bytes of data contained after vendorId field.
 		If RFC 2865 guidelines conformant VSA value is to be retrieved
 		use rather #AsVsaAddress()#.
 		
 		@return
-		IPv4 address representing attribute Value.
+		IP address representing attribute Value.
 	*/
 	PIPSocket::Address AsAddress() const;
 
-	/** Get RFC 2865 guidelines conformant VSA Value as a 32 bit IPv4 address. 
-		This call will build the IPv4 address from 4 bytes of data contained 
+	/** Get RFC 2865 guidelines conformant VSA Value as IPv4 or IPv6 address. 
+		This call will build the IP address from 4 or 16 bytes of data contained 
 		after vendorId, vendorType and vendorLength fields.
 		
 		@return
-		IPv4 address representing attribute Value.
+		IP address representing attribute Value.
 	*/
 	PIPSocket::Address AsVsaAddress() const;
 
@@ -707,7 +710,7 @@ public:
 	/// Append an IP address attribute
 	bool AppendAttr( 
 		unsigned char attrType, /// Attribute Type
-		const PIPSocket::Address& addressValue /// IPv4 address to be stored in the attribute Value
+		const PIPSocket::Address& addressValue /// IP address to be stored in the attribute Value
 		);
 	/// Append a generic VSA attribute
 	bool AppendVsaAttr( 
@@ -730,7 +733,7 @@ public:
 		);
 	/// Append an IP address VSA attribute
 	bool AppendVsaAttr( 
-		const PIPSocket::Address& addressValue, /// IPv4 address to be stored in the attribute Value
+		const PIPSocket::Address& addressValue, /// IP address to be stored in the attribute Value
 		int vendorId, /// 32 bit vendor identifier
 		unsigned char vendorType /// vendor-specific attribute type
 		);
@@ -905,7 +908,7 @@ public:
 	*/
 	RadiusSocket( 
 		/// local network interface address
-		const PIPSocket::Address& addr, 
+		const PIPSocket::Address & addr, 
 		/// port number to send requests from
 		WORD port = 0
 		);
@@ -1044,6 +1047,9 @@ private:
 	PTime m_recentRequestTime;
 	/// time interval over that generated IDs must be unique
 	PTimeInterval m_idCacheTimeout;
+	/// save listen address for port notifications
+	Address m_addr;
+	WORD m_port;
 };
 
 class RadiusClient
@@ -1149,37 +1155,6 @@ public:
 	static WORD GetDefaultAcctPort() { return DefaultAcctPort; }
 
 protected:
-	/** Callback called before a request is sent to the RADIUS server.
-	    It gives the possibility to do something with the request PDU.
-	    The callback is supplied with information whether the packet
-	    is being retransmitted - and can trigger an update of some attributes.
-	    Set requireNewId if attributes with some dynamic contents have been
-	    added, so the request cannot be simply retransmitted with the same ID.
-
-	    @return
-	    True if proceed with this request
-	*/
-	virtual bool OnSendPDU( 
-		/// PDU that is to be sent
-		RadiusPDU& pdu, 
-		/// true if this is PDU retransmission
-		bool retransmision, 
-		/// set to true if a new ID and authenticator should be generated
-		/// for this request upon retransmission
-		bool& generateNewId
-		);
-
-	/** Callback called after response PDU had been received from RADIUS server
-		and before it is returned to the calling application.
-		It gives the possibility to do something with the PDU.
-	
-		@return
-		True if proceed with this response
-	*/
-	virtual bool OnReceivedPDU( 
-		RadiusPDU& pdu /// PDU that was received
-		);
-
 	/** Determine if the #pdu# should be send to auth port
 		of RADIUS server (FALSE) or acct port (TRUE).
 
