@@ -3,7 +3,7 @@
 // yasocket.h
 //
 // Copyright (c) Citron Network Inc. 2002-2003
-// Copyright (c) 2004-2010, Jan Willamowius
+// Copyright (c) 2004-2011, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,7 +13,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef YASOCKET_H
-#define YASOCKET_H "@(#) $Id: yasocket.h,v 1.38 2010/10/25 15:01:51 willamowius Exp $"
+#define YASOCKET_H "@(#) $Id: yasocket.h,v 1.49 2011/11/21 14:04:46 willamowius Exp $"
 
 #include <list>
 #include <vector>
@@ -99,6 +99,10 @@ public:
 	bool Listen(unsigned, WORD, PSocket::Reusability reuse = PSocket::AddressIsExclusive);
 	bool Listen(const Address &, unsigned, WORD, PSocket::Reusability reuse = PSocket::AddressIsExclusive);
 
+#ifdef hasIPV6
+	bool DualStackListen(WORD port);
+#endif
+
 	// new virtual function
 	virtual bool Accept(YaTCPSocket &);
 	virtual bool Connect(const Address &, WORD, const Address &);
@@ -114,15 +118,22 @@ private:
 	YaTCPSocket& operator=(const YaTCPSocket&);
 	
 private:
+#ifdef hasIPV6
+	sockaddr_in6 peeraddr;
+#else
 	sockaddr_in peeraddr;
+#endif
 };
 
 class YaUDPSocket : public YaSocket, public PObject {
 public:
-	YaUDPSocket();
+	YaUDPSocket(WORD port=0, int iAddressFamily=AF_INET);
 
 	bool Listen(unsigned, WORD, PSocket::Reusability reuse = PSocket::AddressIsExclusive);
 	bool Listen(const Address &, unsigned, WORD, PSocket::Reusability reuse = PSocket::AddressIsExclusive);
+#ifdef hasIPV6
+	bool DualStackListen(const Address & localAddr, WORD port);
+#endif
 	void GetLastReceiveAddress(Address &, WORD &) const;
 	void SetSendAddress(const Address &, WORD);
 	/// Get the address to use for connectionless Write().
@@ -144,7 +155,11 @@ private:
 	YaUDPSocket& operator=(const YaUDPSocket&);
 	
 private:
+#ifdef hasIPV6
+	sockaddr_in6 recvaddr, sendaddr;
+#else
 	sockaddr_in recvaddr, sendaddr;
+#endif
 };
 
 class YaSelectList {
@@ -225,7 +240,7 @@ typedef YaSocket IPSocket;
 typedef YaTCPSocket TCPSocket;
 typedef YaUDPSocket UDPSocket;
 
-#else
+#else // not LARGE_FDSET
 
 class SocketSelectList : public PSocket::SelectList {
 public:
@@ -251,11 +266,15 @@ typedef PIPSocket IPSocket;
 class TCPSocket : public PTCPSocket, public NamedObject {
 public:
 	PCLASSINFO( TCPSocket, PTCPSocket )
-	TCPSocket(WORD pt = 0) : PTCPSocket(pt) {}
+	TCPSocket(WORD pt = 0) : PTCPSocket(pt) { }
 	virtual ~TCPSocket() {}
 	// override from class PIPSocket
 	PString GetName() const { return (const char *)NamedObject::GetName(); }
-	
+
+#ifdef hasIPV6
+	bool DualStackListen(WORD port);
+#endif
+
 private:
 	TCPSocket(const TCPSocket&);
 	TCPSocket& operator=(const TCPSocket&);
@@ -265,11 +284,15 @@ class UDPSocket : public PUDPSocket, public NamedObject {
 public:
 	PCLASSINFO( UDPSocket, PUDPSocket )
 	
-	UDPSocket() {}
-	virtual ~UDPSocket() {}
+	UDPSocket(WORD port=0, int iAddressFamily=AF_INET);
+	virtual ~UDPSocket() { }
 
 	// override from class PIPSocket
 	PString GetName() const { return (const char *)NamedObject::GetName(); }
+
+#ifdef hasIPV6
+	bool DualStackListen(const PIPSocket::Address & localAddr, WORD port);
+#endif
 	
 private:
 	UDPSocket(const UDPSocket&);
@@ -411,9 +434,6 @@ protected:
 
 	// remove closed sockets
 	void RemoveClosed(bool);
-	// for historical reason, assume the list has been locked
-	void RemoveSocket(iterator);
-	void RemoveSocket(IPSocket *);
 
 private:
 	SocketsReader(const SocketsReader&);
